#include <QtGui>
#include <QMessageBox>
#include <math.h>
#include "mainwindow.h"
#include "document.h"
#include "default.h"
#include "ahhead.h"
#include "sei.h"
void delAndNullify(PropertyValue* &pPropVal) {
    delete  pPropVal;
    pPropVal = 0;
}

///////////////////// PropertyValue Implemenation ///////////////////////////////
PropertyValue::~PropertyValue() {
}
PropertyValue::PropertyValue( const int i,const QString defval,const QString name) :
        id(i),value(defval),initialvalue(defval),resetvalue(defval),defaultvalue(defval),name(name),changed(FALSE) {}
PropertyValue::PropertyValue(const int i, const QString resval,const QString defval,const QString name) :
        id(i),value(resval),initialvalue(resval),resetvalue(resval),defaultvalue(defval),name(name),changed(FALSE) {}
PropertyValue::PropertyValue( const int i,const  QString val, const QString resval,const QString defval,const QString name):
        id(i),value(val),initialvalue(val),resetvalue(resval),defaultvalue(defval),name(name),changed(FALSE) {}
void PropertyValue::setChanged( bool b) {
    if ( changed == b )
        return;
    changed = b;
    //repaint();

}

bool PropertyValue::isChanged() const {
    return changed;
}
bool PropertyValue::isReseted() const {
    if (value == resetvalue) return FALSE;
    else return TRUE;
}
QDataStream &operator<< (QDataStream & out, const PropertyValue * prop) {

    out <<   prop->GetId() <<    prop->GetValue();
    return out;
}
QDataStream &operator>>(QDataStream & in, PropertyValue & prop) {
    QString v;
    int id;
    in >>   id  >>  v;
    prop.SetId(id);
    prop.SetResetvalue(v);
    return in;
}
///////////////////// <: inherits from PropertyValue :> //////////////////////////

///////////////////// PropertyBoolValue Implemenation ///////////////////////////
PropertyBoolValue::PropertyBoolValue(const int i, const QString defval,const QString name ) :
        PropertyValue(i,defval ,name) {
    kind = PropertyValue::Property_Bool;
}
PropertyBoolValue::PropertyBoolValue( const int i, const QString resval, const QString defval ,const QString name) :
        PropertyValue(i, resval,defval ,name) {
    kind = PropertyValue::Property_Bool;
}
PropertyBoolValue::PropertyBoolValue( const int i, const QString val,const  QString resval,const QString defval,const QString name) :
        PropertyValue(i, val,resval,defval,name ) {
    kind = PropertyValue::Property_Bool;
}
///////////////////// PropertyIntValue Implemenation ///////////////////////////
PropertyIntValue::PropertyIntValue( const int i,const QString defval ,const QString name,const int min,const int max, const bool s):
        PropertyValue(i,defval ,name),
        min(min),max(max), signedValue(s) {
    kind = PropertyValue::Property_Int;
}
PropertyIntValue::PropertyIntValue( const int i,const  QString resval,const QString defval ,const QString name,const int min,const int max, const bool s):
        PropertyValue( i,resval,defval ,name),
        min(min),max(max), signedValue(s) {
    kind = PropertyValue::Property_Int;
}

PropertyIntValue::PropertyIntValue( const int i, const  QString val, const  QString resval,const QString defval,const QString name,const int min,const int max, const bool s):
        PropertyValue( i,val,resval,defval,name ),
        min(min),max(max), signedValue(s) {
    kind = PropertyValue::Property_Int;
}

///////////////////// PropertyIntSpinValue Implemenation ///////////////////////////
PropertyIntSpinValue::PropertyIntSpinValue( const int i,const QString defval ,const QString name,const int min,const int max, const int s):
        PropertyValue(i,defval ,name),
        min(min),max(max), step(s) {
    kind = PropertyValue::Property_IntSpin;
}
PropertyIntSpinValue::PropertyIntSpinValue( const int i,const  QString resval,const QString defval ,const QString name,const int min,const int max, const int s):
        PropertyValue( i,resval,defval ,name),
        min(min),max(max), step(s) {
    kind = PropertyValue::Property_IntSpin;
}

PropertyIntSpinValue::PropertyIntSpinValue( const int i, const  QString val, const  QString resval,const QString defval,const QString name,const int min,const int max, const int s):
        PropertyValue( i,val,resval,defval,name ),
        min(min),max(max), step(s) {
    kind = PropertyValue::Property_IntSpin;
}
///////////////////// PropertyFloatValue Implemenation ///////////////////////////

PropertyFloatValue::PropertyFloatValue( const int i,  const QString defval ,const QString name,
                                        const float min,const float max, const int dec) :
        PropertyValue(i,defval ,name),
        min(min),max(max), decimal(dec) {
    kind = PropertyValue::Property_Float;
}

PropertyFloatValue::PropertyFloatValue(  const int i, const QString resval,const QString defval,
        const QString name,
        const float min,const float max, const int dec ) :
        PropertyValue(i, resval,defval ,name),
        min(min),max(max), decimal(dec) {
    kind = PropertyValue::Property_Float;
}

PropertyFloatValue::PropertyFloatValue( const int i, const  QString val,const  QString resval,
                                        const QString defval,const QString name,
                                        const float min,const float max, const int dec):
        PropertyValue( i,val,resval,defval,name ),
        min(min),max(max), decimal(dec) {
    kind = PropertyValue::Property_Float;
}

///////////////////// PropertyEnumValue Implemenation ///////////////////////////

PropertyEnumValue::PropertyEnumValue( const int i,  const QString defval ,const QString name,const QString lst) :
        PropertyValue(i,defval ,name) {
    list = lst.split ('@');
    kind = PropertyValue::Property_Enum;
}
PropertyEnumValue::PropertyEnumValue(  const int i,  const QString resval,const QString defval ,const QString name,const QString lst) :
        PropertyValue( i,resval,defval ,name) {
    list = lst.split ('@');
    kind = PropertyValue::Property_Enum;
}
PropertyEnumValue::PropertyEnumValue( const int i,  const  QString val, const QString resval,const QString defval,const QString name,const QString lst):
        PropertyValue( i,val,resval,defval,name ) {
    list = lst.split ('@');
    kind = PropertyValue::Property_Enum;
}
void PropertyEnumValue::ChangeEumList(QStringList &lst) {
    list = lst;
}
///////////////////// PropertyStringValue Implemenation ///////////////////////////

PropertyStringValue::PropertyStringValue( const int i,  const QString defval ,const QString name) :
        PropertyValue(i,defval ,name) {
    kind = PropertyValue::Property_String;
}
PropertyStringValue::PropertyStringValue(  const int i,  const QString resval,const QString defval ,const QString name) :
        PropertyValue( i,resval,defval ,name) {
    kind = PropertyValue::Property_String;
}
PropertyStringValue::PropertyStringValue( const int i,  const  QString val, const QString resval,const QString defval,const QString name):
        PropertyValue( i,val,resval,defval,name ) {
    kind = PropertyValue::Property_String;
}
///////////////////// PropertyStringListValue Implemenation ///////////////////////////

PropertyStringListValue::PropertyStringListValue( const int i,  const QString defval ,const QString name) :
        PropertyValue(i,defval ,name) {
    if (!value.isEmpty()) list = value.split ('*');
    else  list = QStringList();
    kind = PropertyValue::Property_StringList;
}
PropertyStringListValue::PropertyStringListValue(  const int i,  const QString resval,const QString defval ,const QString name) :
        PropertyValue( i,resval,defval ,name) {
    if (!value.isEmpty()) list = value.split ('*');
    else  list = QStringList();
    kind = PropertyValue::Property_StringList;
}
PropertyStringListValue::PropertyStringListValue( const int i,  const  QString val, const QString resval,const QString defval,const QString name):
        PropertyValue( i,val,resval,defval,name ) {
    if (!value.isEmpty()) list = value.split ('*');
    else  list = QStringList();
    kind = PropertyValue::Property_StringList;
}
void PropertyStringListValue::SetListToValue(QStringList &lst) {
    if (!lst.empty ()) {
        value = lst.join("*");
        list = lst;


    } else {
        value = QString();
        list = QStringList();
    }
}
void PropertyStringListValue::SetValue  (const QString val)  {
    if (!val.isEmpty()) {
        value = val;
        list = value.split ('*');
    } else {
        value = QString();
        list = QStringList();
    }
}
void PropertyStringListValue::SetResetvalueToValue       ()  {
    if (!resetvalue.isEmpty()) {
        value   =  resetvalue;
        list = value.split ('*');
    } else {
        value = QString();
        list = QStringList();
    }
}
void PropertyStringListValue::SetDefault                 ()  {
    if (!defaultvalue.isEmpty()) {
        value   =  defaultvalue;
        list = value.split ('*');
    } else {
        value = QString();
        list = QStringList();
    }
}
void PropertyStringListValue::SetInitial                 ()  {
    if (!initialvalue.isEmpty()) {
        value   =  initialvalue;
        list = value.split ('*');
    } else {
        value = QString();
        list = QStringList();
    }
}
//////////////////////// PropertyTransformFunction /////////////////////
PropertyTransformFunction::PropertyTransformFunction(const int i, const QString defval ,const QString name):
        PropertyValue( i,defval ,name) {
    SetVal(defval);
    SetDef(defval);
    kind = PropertyValue::Property_TransformFunction;
};
PropertyTransformFunction::PropertyTransformFunction(const int i, const  QString resval,const QString defval,
        const QString name):
        PropertyValue( i,resval,defval ,name) {
    SetVal(resval);
    SetDef(defval);
    kind = PropertyValue::Property_TransformFunction;
};
PropertyTransformFunction::PropertyTransformFunction(const int i, const  QString val, const  QString resval,
        const QString defval,const QString name):
        PropertyValue( i,val,resval,defval,name ) {
    SetVal(val);
    SetDef(defval);
    kind = PropertyValue::Property_TransformFunction;
};
void PropertyTransformFunction::SetValue(PropertyTransformFunction* val) {
    valIsOn = val->valIsOn;
    valIndexMagnitude = val->valIndexMagnitude;
    valA = val->valA;
    valB = val->valB;
    valC = val->valC;
    valD = val->valD;
    defD = val->defD;
    defA  = val->defA;
    defB = val->defB;
    defC = val->defC;
    QString tmp = "";  // "1@0@0.0@1.0@0.0@0.0"
    if (valIsOn) tmp = "1";
    else tmp = "0";
    value = QString("%1@%2@%3@%4@%5@%6").arg(tmp).arg(valIndexMagnitude).arg(valD).arg(valA).arg(valB).arg(valC);
//qWarning(" '%s' ",value.toAscii().data());
}
void PropertyTransformFunction::SetVal(const QString val) {
// "1@0@0.0@1.0@0.0@0.0"
    QStringList param = val.split( '@');
    if (param[0].toInt() == 1) valIsOn = true;
    else valIsOn = false;
    valIndexMagnitude = param[1].toInt();
    valD = param[2];
    valA  = param[3];
    valB = param[4];
    valC = param[5];
}

void PropertyTransformFunction::SetDef(const QString val) {
// "1@0@0.0@1.0@0.0@0.0"
    QStringList param = val.split( '@');
    if (param[0].toInt() == 1) defIsOn = true;
    else defIsOn = false;
    defIndexMagnitude = param[1].toInt();
    defD = param[2];
    defA  = param[3];
    defB = param[4];
    defC = param[5];
}

void PropertyTransformFunction::SetMagnitude(float m) {
    if (m >= valD.toFloat()) magnitude = m*valA.toFloat() + valB.toFloat();
    else magnitude = valC.toFloat();
}
float PropertyTransformFunction::GetMagnitude(float m) {
    SetMagnitude(m);
    return  magnitude;
}
void  PropertyTransformFunction::GetM(float inm,float &outm) {
    if (inm >= valD.toFloat()) outm = inm*valA.toFloat() + valB.toFloat();
    else outm = valC.toFloat();

}
//--------------------------------------------------------------------------------------------------------
PropertyDateTimeValue::PropertyDateTimeValue( const int i,  const QString defval ,const QString name) :
        PropertyValue(i,defval ,name) {
    kind = PropertyValue::Property_DateTime;
}

PropertyDateTimeValue::PropertyDateTimeValue(  const int i, const QString resval,const QString defval,
        const QString name) :
        PropertyValue(i, resval,defval ,name) {
    kind = PropertyValue::Property_DateTime;
}

PropertyDateTimeValue::PropertyDateTimeValue( const int i, const  QString val,const  QString resval,
        const QString defval,const QString name):
        PropertyValue( i,val,resval,defval,name ) {
    kind = PropertyValue::Property_DateTime;
}

///////////////////// MDocument class implementation ////////////////////////////

MDocument::~MDocument() {
//QList<PropertyTransformFunction *> mb = GetTransformMagnitudeMb();

//qWarning("MDocument::~MDocument '%s'",mb.at(0)->valA.toAscii().data());
    for_each(values.begin(),values.end(),delAndNullify);
    values.erase(remove(values.begin(),values.end(),
                        static_cast<PropertyValue*>(0)),
                 values.end());
    if (!map.isEmpty()) map.clear();
    if (!recordmap.isEmpty())recordmap.clear();


    if ( mlt != 0 ) delete mlt;
    /*
    PropValIterator it; int i = 0;
    for (it = values.begin(); it!= values.end(); it++) {
    qWarning(" i %d  id %d '%s'",i,(*it)->GetId(),(*it)->GetValue().toAscii().data());
    delete *it; i++;
    }
    values.clear();
    */

}
//================================================================================
MDocument::MDocument(MDocument *doc) {



    PropValIterator i = doc->GetValues().begin();
    while (i != doc->GetValues().end()) {

//      if( (*i)->GetValue() != (*i)->GetResetvalue()) {mod = true; (*i)->setChanged(true);  }
//      else (*i)->setChanged(false);
        i++;
    }
}

void MDocument::SetPropertyIntValue(PropertyIntValue* &p, int id,QString reset,QString defaultvalue,
                                    QString name,int min,int max,bool sign) {
    p = new PropertyIntValue(id,reset,defaultvalue,name,min,max,sign);
    values.push_back(p);
}
void MDocument::SetPropertyTransformFunction(PropertyTransformFunction* &p, int id,
        QString reset,QString defaultvalue,
        QString name) {
    p = new PropertyTransformFunction(id,reset,defaultvalue,name);
    values.push_back(p);
}
void MDocument::SetPropertyFloatValue(PropertyFloatValue* &p, int id,QString reset,QString defaultvalue,
                                      QString name,float min,float max,int decimal) {
    p = new PropertyFloatValue(id,reset,defaultvalue,name,min,max,decimal);
    values.push_back(p);
}

void MDocument::SetPropertyEnumValue(PropertyEnumValue* &p, int id,QString reset,QString defaultvalue,
                                     QString name,QString enumeration) {
    p = new PropertyEnumValue(id,reset,defaultvalue,name,enumeration);
    values.push_back(p);
}
void MDocument::SetPropertyStringValue(PropertyStringValue* &p, int id,QString reset,QString defaultvalue,
                                       QString name) {
    p = new PropertyStringValue(id,reset,defaultvalue,name);
    values.push_back(p);
}
void MDocument::SetPropertyStringListValue(PropertyStringListValue* &p, int id,QString reset,QString defaultvalue,
        QString name) {
    p = new PropertyStringListValue(id,reset,defaultvalue,name);
    values.push_back(p);
}
bool MDocument::CheckModified() {
    PropValIterator i = values.begin();
    while (i != values.end()) {
        (*i)->setChanged();
        i++;
    }
    return  isModified();
}

bool MDocument::isModified() {
    bool mod = false;
    PropValIterator i = values.begin();
    while (i != values.end()) {
        if ( (*i)->GetValue() != (*i)->GetInitialvalue()) {
            mod = true;
            /*
            QMessageBox::warning(0,"",
            		                     QString("name %1\nval '%2'\ninival '%3'")
            		                     .arg((*i)->GetName())
            		                     .arg((*i)->GetValue())
            		                     .arg((*i)->GetInitialvalue()));
            */
            (*i)->setChanged(true);
        } else (*i)->setChanged(false);
        i++;
    }

    return mod;
}
/*
void    MDocument::printMDocument(){
    PrintDocument<PropValVector> P =
	    for_each(values.begin(),values.end(),
		     PrintDocument<PropValVector>(std::cout));
    cout << "-------------  "<< P.count << " parameters are in document  -------------" << endl<< endl;

    cout << "-------------  End of print document  ----------------" << endl<<endl;
}
*/
//================================================================================
MDocument::MDocument() {
//----------------------------------- Siesmograms -----------------------------
    mlt = 0;
///                 Seismograms                            ///

    SetPropertyStringListValue(ListSies,idListSies(),
                               defaultListSies(),defaultListSies(),
                               nameListSies());

    SetPropertyStringListValue(InListNameSies,idInListNameSies(),
                               defaultInListNameSies(),defaultInListNameSies(),
                               nameInListNameSies());

    SetPropertyStringListValue(ProjectListNameSeis,idListIndexes(),
                               defaultListIndexes(),defaultListIndexes(),
                               nameListIndexes());

    SetPropertyStringValue(InputDataDirectory,idDataDirectoryName(),
                           defaultDataDirectoryName(),defaultDataDirectoryName(),
                           nameDataDirectoryName());

    SetPropertyEnumValue(IsFtanProject,idIsFtanProject(),
                         defaultIsFtanProject(),defaultIsFtanProject(),
                         nameIsFtanProject(),enumBoolList());

///------------------------- Model Panel ------------------------------------------


    SetPropertyStringValue(ModelNameDirectory,idModelDirectoryName(),
                           defaultModelDirectoryName(),defaultModelDirectoryName(),
                           nameModelDirectoryName());

    SetPropertyStringListValue(StationList,idStationNameList(),
                               defaultStationNameList(),defaultStationNameList(),
                               nameStationNameList());

    SetPropertyStringListValue(StationModelList,idStationModelList(),
                               defaultStationModelList(),defaultStationModelList(),
                               nameStationModelList());


    SetPropertyEnumValue(SourceModelType,idSourceModelType(),
                         defaultSourceModelType(),defaultSourceModelType(),
                         nameSourceModelType(),enumSourceModelType());

    SetPropertyEnumValue(IsModelCalculate,idIsModelCalculate(),
                         defaultIsModelCalculate(),defaultIsModelCalculate(),
                         nameIsModelCalculate(),enumBoolList());

    SetPropertyStringValue(NameSourceModel,idNameSourceModel(),
                           defaultNameSourceModel(),defaultNameSourceModel(),
                           nameNameSourceModel());

///------------------------- Range Panel --------------------------------------------

    SetPropertyFloatValue(TMinRangeWave,idTMinRangeWave(),
                          defaultTMinRangeWave(),defaultTMinRangeWave(),
                          nameTMinRangeWave(),minTMinRangeWave(),
                          maxTMinRangeWave(),decimalTMinRangeWave());

    SetPropertyFloatValue(TMaxRangeWave,idTMaxRangeWave(),
                          defaultTMaxRangeWave(),defaultTMaxRangeWave(),
                          nameTMaxRangeWave(),minTMaxRangeWave(),
                          maxTMaxRangeWave(),decimalTMaxRangeWave());

    SetPropertyEnumValue(PhVel,idPhVel(),
                         defaultPhVel(),defaultPhVel(),
                         namePhVel(),enumBoolList());

    SetPropertyFloatValue(TMin,idTMin(),
                          defaultTMin(),defaultTMin(),
                          nameTMin(),minTMin(),
                          maxTMin(),decimalTMin());

    SetPropertyStringListValue(RangeWaveList,idRangeWaveList(),
                               defaultRangeWaveList(),defaultRangeWaveList(),
                               nameRangeWaveList());

    SetPropertyFloatValue(TMinRangeGrid,idTMinRangeGrid(),
                          defaultTMinRangeGrid(),defaultTMinRangeGrid(),
                          nameTMinRangeGrid(),minTMinRangeGrid(),
                          maxTMinRangeGrid(),decimalTMinRangeGrid());

    SetPropertyFloatValue(TMaxRangeGrid,idTMaxRangeGrid(),
                          defaultTMaxRangeGrid(),defaultTMaxRangeGrid(),
                          nameTMaxRangeGrid(),minTMaxRangeGrid(),
                          maxTMaxRangeGrid(),decimalTMaxRangeGrid());

    SetPropertyIntValue(NumberSamples,idNumberSamples(),
                        defaultNumberSamples(),defaultNumberSamples(),nameNumberSamples(),
                        minNumberSamples(),maxNumberSamples(),signedNumberSamples());

    SetPropertyEnumValue(NumberFft,idFftn2n(),
                         defaultFftn2n(),defaultFftn2n(),
                         nameFftn2n(),enumFftn2n());

    SetPropertyStringListValue(RangeGridList,idRangeGridList(),
                               defaultRangeGridList(),defaultRangeGridList(),
                               nameRangeGridList());

///------------------------- Bulletin file name  -------------------------------------

    SetPropertyEnumValue(IsPolaritySuplementary,idIsPolaritySuplementary(),
                         defaultIsPolaritySuplementary(),defaultIsPolaritySuplementary(),
                         nameIsPolaritySuplementary(),enumBoolList());

    SetPropertyEnumValue(IsPolarityBulletin,idIsPolarityBulletin(),
                         defaultIsPolarityBulletin(),defaultIsPolarityBulletin(),
                         nameIsPolarityBulletin(),enumBoolList());

    SetPropertyEnumValue(IsPolaritySmoothing,idIsPolaritiesSmoothing(),
                         defaultIsPolaritiesSmoothing(),defaultIsPolaritiesSmoothing(),
                         nameIsPolaritiesSmoothing(),enumBoolList());

    SetPropertyStringValue(SuplementaryFileName,idSuplementaryFileName(),
                           defaultSuplementaryFileName(),defaultSuplementaryFileName(),
                           nameSuplementaryFileName());

    SetPropertyStringValue(BulletinFileName,idBulletinFileName(),
                           defaultBulletinFileName(),defaultBulletinFileName(),
                           nameBulletinFileName());

    SetPropertyFloatValue(PolarityDepth,idPolarityDepth(),
                          defaultPolarityDepth(),defaultPolarityDepth(),
                          namePolarityDepth(),minPolarityDepth(),
                          maxtPolarityDepth(),decimalPolarityDepth());

    SetPropertyFloatValue(PolarityVpsource,idPolarityVpsource(),
                          defaultPolarityVpsource(),defaultPolarityVpsource(),
                          namePolarityVpsource(),minPolarityVpsource(),
                          maxtPolarityVpsource(),decimalPolarityVpsource());

    SetPropertyFloatValue(PolaritySmoothing,idPolaritySmoothing(),
                          defaultPolaritySmoothing(),defaultPolaritySmoothing(),
                          namePolaritySmoothing(),minPolaritySmoothing(),
                          maxtPolaritySmoothing(),decimalPolaritySmoothing());
    SetPropertyEnumValue(IsPolaritySmoothing,idIsPolaritiesSmoothing(),
                         defaultIsPolaritiesSmoothing(),defaultIsPolaritiesSmoothing(),
                         nameIsPolaritiesSmoothing(),enumBoolList());

///------------------------- Grid Panel --------------------------------------------

///------------------------- GridDepth ----------------------------------------------

    SetPropertyFloatValue(GridDepth,idGridDepth(),
                          defaultGridDepth(),defaultGridDepth(),
                          nameGridDepth(),minGridDepth(),
                          maxtGridDepth(),decimalGridDepth());

    SetPropertyFloatValue(GridDepthStep,idGridDepthStep(),
                          defaultGridDepthStep(),defaultGridDepthStep(),
                          nameGridDepthStep(),minGridDepthStep(),
                          maxtGridDepthStep(),decimalGridDepthStep());

    SetPropertyIntValue(DepthPoint,idDepthNumberPoint(),
                        defaultDepthNumberPoint(),defaultDepthNumberPoint(),nameDepthNumberPoint(),
                        minDepthNumberPoint(),maxDepthNumberPoint(),signedDepthNumberPoint());

///------------------------- GridDip -------------------------------------------------

    SetPropertyFloatValue(GridDip,idGridDip(),
                          defaultGridDip(),defaultGridDip(),
                          nameGridDip(),minGridDip(),
                          maxtGridDip(),decimalGridDip());

    SetPropertyFloatValue(GridDipStep,idGridDipStep(),
                          defaultGridDipStep(),defaultGridDipStep(),
                          nameGridDipStep(),minGridDipStep(),
                          maxtGridDipStep(),decimalGridDipStep());

    SetPropertyIntValue(DipPoint,idDipNumberPoint(),
                        defaultDipNumberPoint(),defaultDipNumberPoint(),nameDipNumberPoint(),
                        minDipNumberPoint(),maxDipNumberPoint(),signedDipNumberPoint());

///------------------------- GridStrike -------------------------------------------------

    SetPropertyFloatValue(GridStrike,idGridStrike(),
                          defaultGridStrike(),defaultGridStrike(),
                          nameGridStrike(),minGridStrike(),
                          maxtGridStrike(),decimalGridStrike());

    SetPropertyFloatValue(GridStrikeStep,idGridStrikeStep(),
                          defaultGridStrikeStep(),defaultGridStrikeStep(),
                          nameGridStrikeStep(),minGridStrikeStep(),
                          maxtGridStrikeStep(),decimalGridStrikeStep());

    SetPropertyIntValue(StrikePoint,idStrikeNumberPoint(),
                        defaultStrikeNumberPoint(),defaultStrikeNumberPoint(),nameStrikeNumberPoint(),
                        minStrikeNumberPoint(),maxStrikeNumberPoint(),signedStrikeNumberPoint());

///------------------------- GridSlip -------------------------------------------------

    SetPropertyFloatValue(GridSlip,idGridSlip(),
                          defaultGridSlip(),defaultGridSlip(),
                          nameGridSlip(),minGridSlip(),
                          maxtGridSlip(),decimalGridSlip());

    SetPropertyFloatValue(GridSlipStep,idGridSlipStep(),
                          defaultGridSlipStep(),defaultGridSlipStep(),
                          nameGridSlipStep(),minGridSlipStep(),
                          maxtGridSlipStep(),decimalGridSlipStep());

    SetPropertyIntValue(SlipPoint,idSlipNumberPoint(),
                        defaultSlipNumberPoint(),defaultSlipNumberPoint(),nameSlipNumberPoint(),
                        minSlipNumberPoint(),maxSlipNumberPoint(),signedSlipNumberPoint());

    SetPropertyEnumValue(IsMomtensCalculate,idIsMomtensCalculate(),
                         defaultIsMomtensCalculate(),defaultIsMomtensCalculate(),
                         nameIsMomtensCalculate(),enumBoolList());
}
void MDocument::GetRecordSeisMaterials(QStringList &FileNames) {
    ListSies->SetListToValue(FileNames);
    SetSeisToProject();

}
MDocument::MDocument(const QString &fileName) {
    mlt = 0;
    projectFileGood = true;
    if (!loadBinary(fileName)) {
        projectFileGood = false;
        return;
    }

    SetSeisToProject();

    /*
        if (!CheckPathesForProjectFiles()) {
            projectFileGood = false;
            return;
        }

        QFileInfo file(ListSies->GetList().at(0));

        addinsDirectory = file.absolutePath ();

        QStringList FileNamesAddIns;
        if ( !hasInputFiles(addinsDirectory,FileNamesAddIns) ) {
            projectFileGood = false;
            return;
        }

        mlt = new _MLT(FileNamesAddIns.size(),FileNamesAddIns,0);

        SortListSeis();

        QList<QString > keys = map.keys();
        QList<SEISMO *> values = map.values();

        for (int i = 0; i < values.size(); ++i)
            mapaddins.insert(keys.at(i),values.at(i)->Get_fname());

        delete mlt;
        mlt = 0;


        SetSeisToProject();
    */
}
bool MDocument::CheckPathesForProjectFiles() {
    QStringList list = ListSies->GetList();
    QString tmp = "These project files are not exist:\n";
    bool isnotfile = false;
    for (int i = 0; i < list.size() ; i++ ) {
        QString file = list.at(i);
        QFile f(file);
        if ( !f.exists() ) {
            tmp +=  file + "\n";
            isnotfile = true;
        }
    }
    if (isnotfile ) {
        int ret = QMessageBox::warning(0, mainWindowCaption(),
                                       QString("%1\nDo you want to try to find the directory which contains all project files? ").arg(tmp),
                                       QMessageBox::Yes | QMessageBox::Default,
                                       QMessageBox::No | QMessageBox::Escape);
        if (ret == QMessageBox::No) return false;
        else
            if (!FindNewLocationOfRecords()) return false;
    }
    return true;
}

bool MDocument::FindNewLocationOfRecords() {
    QString dir = QFileDialog::getExistingDirectory(0, QString("Open Directory"),
                  QDir::homePath (),
                  QFileDialog::ShowDirsOnly
                  | QFileDialog::DontResolveSymlinks);

    if (dir.isEmpty()) return false;
    QStringList list = ListSies->GetList();
    QString tmp = QString("New Directory\n'%1'\ndoes not contains files:\n").arg(dir);
    QDir fileDir(dir);
    QStringList entryList = fileDir.entryList();
    int numFiles = 0;
    for (int i = 0; i < list.size(); ++i) {
        QString str = list.at(i);
        str = str.right(str.length() - str.lastIndexOf("/") -1);
        bool findfile = false;
        for (int j = 0; j < entryList.size(); ++j) {
            QString file = entryList.at(j);
            if (str == file) {
                findfile = true;
                break;
            }
        }
        if (findfile) numFiles++;
        else tmp += str + "\n";
    }
    if (numFiles < list.size()) {
        QMessageBox::critical(0,"",tmp);
        return false;
    }
    QStringList newList = QStringList();
    for (int i = 0; i < list.size(); ++i) {
        QString str = list.at(i);
        str = str.right(str.length() - str.lastIndexOf("/") -1);
        str = dir +  "/"  + str;
        newList << str;
    }
    ListSies->SetListToValue(newList);
    MainWindow::self->modifiedProject = true;
    return true;
}



void MDocument::SetSeisToProject() {
    if (mlt != 0) delete mlt;
    mlt = new _MLT(ListSies->GetList().size(),ListSies->GetList(),0);
    if (!mlt->seiFilesGood) {
        projectFileGood = false;
        return;
    }
    SortListSeis();

    QStringList inList;
    SEISMO **cs = mlt->Get_CS();
    int n = mlt->Get_I_N_SEI();
    for (int i = 0; i < n; ++i) {
        inList << cs[i]->Get_fname();
    }

    ListSies->SetListToValue(inList);
    ListSies->SetValueToResetvalue();

    mlt->Recalc_parametrs();

}


void MDocument::SortListSeis() {
    if (!map.isEmpty()) map.clear();
    if (!recordmap.isEmpty()) recordmap.clear();
    SEISMO **cs = mlt->Get_CS();
    SEISMO **ss = mlt->Get_SS();
    QString tmp="";
    int n = mlt->Get_I_N_SEI();
    for (int i = 0; i < n; ++i) {
        QString name =  QString(cs[i]->Get_st_name());
        int length = name.length ();
//for(int j = name.length(); j  < 4; ++j) name.insert(j,' ');
        QString instr = QString(cs[i]->Get_instr());
//for(int j = instr.length(); j < 2; ++j) instr.insert(j,' ');

//QMessageBox::warning(0,"",QString("SortListSeis i %1 name '%2' lname %3 instr '%4' linstr %5").arg(i).
//		     arg(name).arg(name.length()).arg(instr).arg(instr.length()));
        QString orbit = "";
        QString date = "";
        instr.remove(0,1);
        QString filename = cs[i]->Get_fname();
        if (IsFtanProject->GetValue() == "True") {
            QStringList manelist = filename.split("_");
            orbit = manelist.at(2);
            date = manelist.at(3) + " " + manelist.at(4);
        } else {
            orbit = "?";
        }
        name += ":"+ orbit;
        name += ":"+instr;
        if (IsFtanProject->GetValue() == "True") {
            if (length == 3) name += "   ";
            name += " " + date;
        } else {
            QFileInfo info(filename);
            QDateTime dt = info.created ();
            QString dates = dt.toString(Qt::ISODate);
            dates.replace('-','.');
            dates.replace('T',' ');
            if (length == 3) name += "   ";
            name += " ";
            name += dates;
        }

        // QString comp = QString(cs[i]->Get_comp()[0]);
        // for (int j = comp.length(); j < 2; ++j) comp.insert(j,' ');
        // name += ":"+comp;
        //tmp += "'" +name + "'\n";
//QMessageBox::warning(0,"",QString("SortListSeis i %1 name '%2' lname %3 instr '%4' linstr %5").arg(i).
//		     arg(name).arg(name.length()).arg(instr).arg(instr.length()));
        map.insert(name, cs[i]);
    }


    QList<SEISMO *> values = map.values();
    for (int i = 0; i < values.size(); ++i)
        ss[i] = cs[i] = values.at(i);
}
bool  MDocument::loadBinary(const QString &fileName) {

    QFile file(fileName);
    if (!file.open(QIODevice::ReadOnly)) {
        ioError(file,"Cannot open file %1 for reading") ;
        return false;
    }
    QDataStream in (&file);
    quint32 magic;
    in >>  magic;
    if (magic != MagicNumber) {
        error(file,"File %1 is not a project file. Bad Format.");
        return false;
    }
    int pinvVersion;

    in >>  pinvVersion;
    pinvVersionFromProjectFile = pinvVersion;
    QString lst =  FmtVersionsForQt_4();
    QStringList words =  lst.split(" ");
    int minForQt4 =  words[0].toInt();
    int maxForQt4 =  words[1].toInt();

    if ( pinvVersion  >=  minForQt4 && pinvVersion  <=  maxForQt4 )
        in.setVersion(QDataStream::Qt_4_4);
// Here for next    VersionQDataStream

    readFromStream(in);

    if ( file.error() != QFile::NoError  ) {
        ioError(file,"Error reading from file %1") ;
        return false;
    }

    file.close() ;

//QMessageBox::critical(0,mainWindowCaption(),
//                         QString(" '%1'").arg(ListSies->GetValue()));
    return true;
}

bool  MDocument::saveBinary(const QString &fileName) {
    QFile file(fileName);
    if (!file.open(QIODevice::WriteOnly)) {
        ioError(file,"Cannot open file %1 for writing") ;
        return false;
    }
    QDataStream out (&file);
    out  <<  (quint32)MagicNumber;
    int pinvVersion =  FmtCurrentVersion().toInt();
    out  <<  pinvVersion;
    writeToStream(out);
    if ( file.error() != QFile::NoError  ) {
        ioError(file,"Error writing to file %1") ;
        return false;
    }

    file.close() ;
    return true;
}

void MDocument::error(const QFile &file ,const QString &message) {
    QMessageBox::warning(0,"Project Parameters",message.arg(file.fileName()));
}
void MDocument::ioError(const QFile &file ,const QString &message) {
    error(file, message + ": " + file.errorString());
}
void MDocument::writeToStream(QDataStream &out) {
    PropValIterator it = values.begin();
    while (it != values.end()) {
        out << *it;
        ++it;
    }
}
void MDocument::readFromStream(QDataStream &in) {
    PropertyValue prop;
    while (!in.atEnd()) {
        in >> prop;

//----------------------------------- Siesmograms -----------------------------

        if ( prop.GetId() ==   idListSies()) {
            SetPropertyStringListValue(ListSies,idListSies(),
                                       prop.GetResetvalue(),defaultListSies(),
                                       nameListSies());
        }

        if ( prop.GetId() ==   idInListNameSies()) {
            SetPropertyStringListValue(InListNameSies,idInListNameSies(),
                                       prop.GetResetvalue(),defaultInListNameSies(),
                                       nameInListNameSies());
        }

        if ( prop.GetId() ==   idListIndexes()) {
            SetPropertyStringListValue(ProjectListNameSeis,idListIndexes(),
                                       prop.GetResetvalue(),defaultListIndexes(),
                                       nameListIndexes());
        }

        if ( prop.GetId() ==   idDataDirectoryName()) {
            SetPropertyStringValue(InputDataDirectory,idDataDirectoryName(),
                                   prop.GetResetvalue(),defaultDataDirectoryName(),
                                   nameDataDirectoryName());
        }
        if ( prop.GetId() ==   idIsFtanProject()) {
            SetPropertyEnumValue(IsFtanProject,idIsFtanProject(),
                                 prop.GetResetvalue(),defaultIsFtanProject(),
                                 nameIsFtanProject(),enumBoolList());
        }


///------------------------- Model Panel ------------------------------------------


        if ( prop.GetId() ==   idModelDirectoryName()) {
            SetPropertyStringValue(ModelNameDirectory,idModelDirectoryName(),
                                   prop.GetResetvalue(),defaultModelDirectoryName(),
                                   nameModelDirectoryName());
        }

        if ( prop.GetId() ==   idStationNameList()) {
            SetPropertyStringListValue(StationList,idStationNameList(),
                                       prop.GetResetvalue(),defaultStationNameList(),
                                       nameStationNameList());
        }

        if ( prop.GetId() ==   idStationModelList()) {
            SetPropertyStringListValue(StationModelList,idStationModelList(),
                                       prop.GetResetvalue(),defaultStationModelList(),
                                       nameStationModelList());

        }

        if ( prop.GetId() ==   idSourceModelType()) {
            SetPropertyEnumValue(SourceModelType,idSourceModelType(),
                                 prop.GetResetvalue(),defaultSourceModelType(),
                                 nameSourceModelType(),enumSourceModelType());
        }

        if ( prop.GetId() ==   idIsModelCalculate()) {
            SetPropertyEnumValue(IsModelCalculate,idIsModelCalculate(),
                                 prop.GetResetvalue(),defaultIsModelCalculate(),
                                 nameIsModelCalculate(),enumBoolList());
        }

        if ( prop.GetId() ==   idNameSourceModel()) {
            SetPropertyStringValue(NameSourceModel,idNameSourceModel(),
                                   defaultNameSourceModel(),defaultNameSourceModel(),
                                   nameNameSourceModel());
         }

///------------------------- Range Panel --------------------------------------------

            if ( prop.GetId() ==   idTMinRangeWave()) {
                SetPropertyFloatValue(TMinRangeWave,idTMinRangeWave(),
                                      prop.GetResetvalue(),defaultTMinRangeWave(),
                                      nameTMinRangeWave(),minTMinRangeWave(),
                                      maxTMinRangeWave(),decimalTMinRangeWave());
            }
            if ( prop.GetId() ==   idTMaxRangeWave()) {
                SetPropertyFloatValue(TMaxRangeWave,idTMaxRangeWave(),
                                      prop.GetResetvalue(),defaultTMaxRangeWave(),
                                      nameTMaxRangeWave(),minTMaxRangeWave(),
                                      maxTMaxRangeWave(),decimalTMaxRangeWave());
            }

            if ( prop.GetId() ==   idPhVel()) {
                SetPropertyEnumValue(PhVel,idPhVel(),
                                     prop.GetResetvalue(),defaultPhVel(),
                                     namePhVel(),enumBoolList());
            }

            if ( prop.GetId() ==   idTMin()) {
                SetPropertyFloatValue(TMin,idTMin(),
                                      prop.GetResetvalue(),defaultTMin(),
                                      nameTMin(),minTMin(),
                                      maxTMin(),decimalTMin());
            }

            if ( prop.GetId() ==   idRangeWaveList()) {
                SetPropertyStringListValue(RangeWaveList,idRangeWaveList(),
                                           prop.GetResetvalue(),defaultRangeWaveList(),
                                           nameRangeWaveList());
            }

            if ( prop.GetId() ==   idTMinRangeGrid()) {
                SetPropertyFloatValue(TMinRangeGrid,idTMinRangeGrid(),
                                      prop.GetResetvalue(),defaultTMinRangeGrid(),
                                      nameTMinRangeGrid(),minTMinRangeGrid(),
                                      maxTMinRangeGrid(),decimalTMinRangeGrid());
            }

            if ( prop.GetId() ==   idTMaxRangeGrid()) {
                SetPropertyFloatValue(TMaxRangeGrid,idTMaxRangeGrid(),
                                      prop.GetResetvalue(),defaultTMaxRangeGrid(),
                                      nameTMaxRangeGrid(),minTMaxRangeGrid(),
                                      maxTMaxRangeGrid(),decimalTMaxRangeGrid());
            }

            if ( prop.GetId() ==   idNumberSamples()) {
                SetPropertyIntValue(NumberSamples,idNumberSamples(),
                                    prop.GetResetvalue(),defaultNumberSamples(),nameNumberSamples(),
                                    minNumberSamples(),maxNumberSamples(),signedNumberSamples());
            }

            if ( prop.GetId() ==   idFftn2n()) {
                SetPropertyEnumValue(NumberFft,idFftn2n(),
                                     prop.GetResetvalue(),defaultFftn2n(),
                                     nameFftn2n(),enumFftn2n());
            }

            if ( prop.GetId() ==   idRangeGridList()) {
                SetPropertyStringListValue(RangeGridList,idRangeGridList(),
                                           prop.GetResetvalue(),defaultRangeGridList(),
                                           nameRangeGridList());
            }
///------------------------- Bulletin file name  -------------------------------------

            if ( prop.GetId() ==   idIsPolaritySuplementary()) {
                SetPropertyEnumValue(IsPolaritySuplementary,idIsPolaritySuplementary(),
                                     prop.GetResetvalue(),defaultIsPolaritySuplementary(),
                                     nameIsPolaritySuplementary(),enumBoolList());

            }

            if ( prop.GetId() ==   idIsPolarityBulletin()) {
                SetPropertyEnumValue(IsPolarityBulletin,idIsPolarityBulletin(),
                                     prop.GetResetvalue(),defaultIsPolarityBulletin(),
                                     nameIsPolarityBulletin(),enumBoolList());

            }
            if ( prop.GetId() ==   idIsPolaritiesSmoothing()) {
                SetPropertyEnumValue(IsPolaritySmoothing,idIsPolaritiesSmoothing(),
                                     prop.GetResetvalue(),defaultIsPolaritiesSmoothing(),
                                     nameIsPolaritiesSmoothing(),enumBoolList());

            }

            if ( prop.GetId() ==   idSuplementaryFileName()) {
                SetPropertyStringValue(SuplementaryFileName,idSuplementaryFileName(),
                                       prop.GetResetvalue(),defaultSuplementaryFileName(),
                                       nameSuplementaryFileName());

            }

            if ( prop.GetId() ==   idBulletinFileName()) {
                SetPropertyStringValue(BulletinFileName,idBulletinFileName(),
                                       prop.GetResetvalue(),defaultBulletinFileName(),
                                       nameBulletinFileName());

            }

            if ( prop.GetId() ==   idPolarityDepth()) {
                SetPropertyFloatValue(PolarityDepth,idPolarityDepth(),
                                      prop.GetResetvalue(),defaultPolarityDepth(),
                                      namePolarityDepth(),minPolarityDepth(),
                                      maxtPolarityDepth(),decimalPolarityDepth());

            }

            if ( prop.GetId() ==   idPolarityVpsource()) {
                SetPropertyFloatValue(PolarityVpsource,idPolarityVpsource(),
                                      prop.GetResetvalue(),defaultPolarityVpsource(),
                                      namePolarityVpsource(),minPolarityVpsource(),
                                      maxtPolarityVpsource(),decimalPolarityVpsource());

            }

            if ( prop.GetId() ==   idPolaritySmoothing()) {
                SetPropertyFloatValue(PolaritySmoothing,idPolaritySmoothing(),
                                      prop.GetResetvalue(),defaultPolaritySmoothing(),
                                      namePolaritySmoothing(),minPolaritySmoothing(),
                                      maxtPolaritySmoothing(),decimalPolaritySmoothing());
            }
///------------------------- Grid Panel --------------------------------------------

///------------------------- GridDepth ----------------------------------------------

            if ( prop.GetId() ==   idGridDepth()) {
                SetPropertyFloatValue(GridDepth,idGridDepth(),
                                      prop.GetResetvalue(),defaultGridDepth(),
                                      nameGridDepth(),minGridDepth(),
                                      maxtGridDepth(),decimalGridDepth());
            }
            if ( prop.GetId() ==   idGridDepthStep()) {
                SetPropertyFloatValue(GridDepthStep,idGridDepthStep(),
                                      prop.GetResetvalue(),defaultGridDepthStep(),
                                      nameGridDepthStep(),minGridDepthStep(),
                                      maxtGridDepthStep(),decimalGridDepthStep());

            }
            if ( prop.GetId() ==   idDepthNumberPoint()) {
                SetPropertyIntValue(DepthPoint,idDepthNumberPoint(),
                                    prop.GetResetvalue(),defaultDepthNumberPoint(),nameDepthNumberPoint(),
                                    minDepthNumberPoint(),maxDepthNumberPoint(),signedDepthNumberPoint());
            }
///------------------------- GridDip -------------------------------------------------

            if ( prop.GetId() ==   idGridDip()) {
                SetPropertyFloatValue(GridDip,idGridDip(),
                                      prop.GetResetvalue(),defaultGridDip(),
                                      nameGridDip(),minGridDip(),
                                      maxtGridDip(),decimalGridDip());

            }
            if ( prop.GetId() ==   idGridDipStep()) {
                SetPropertyFloatValue(GridDipStep,idGridDipStep(),
                                      prop.GetResetvalue(),defaultGridDipStep(),
                                      nameGridDipStep(),minGridDipStep(),
                                      maxtGridDipStep(),decimalGridDipStep());

            }
            if ( prop.GetId() ==   idDipNumberPoint()) {
                SetPropertyIntValue(DipPoint,idDipNumberPoint(),
                                    prop.GetResetvalue(),defaultDipNumberPoint(),nameDipNumberPoint(),
                                    minDipNumberPoint(),maxDipNumberPoint(),signedDipNumberPoint());
            }
///------------------------- GridStrike -------------------------------------------------

            if ( prop.GetId() ==   idGridStrike()) {
                SetPropertyFloatValue(GridStrike,idGridStrike(),
                                      prop.GetResetvalue(),defaultGridStrike(),
                                      nameGridStrike(),minGridStrike(),
                                      maxtGridStrike(),decimalGridStrike());

            }
            if ( prop.GetId() ==   idGridStrikeStep()) {
                SetPropertyFloatValue(GridStrikeStep,idGridStrikeStep(),
                                      prop.GetResetvalue(),defaultGridStrikeStep(),
                                      nameGridStrikeStep(),minGridStrikeStep(),
                                      maxtGridStrikeStep(),decimalGridStrikeStep());

            }
            if ( prop.GetId() ==   idStrikeNumberPoint()) {
                SetPropertyIntValue(StrikePoint,idStrikeNumberPoint(),
                                    prop.GetResetvalue(),defaultStrikeNumberPoint(),nameStrikeNumberPoint(),
                                    minStrikeNumberPoint(),maxStrikeNumberPoint(),signedStrikeNumberPoint());
            }
///------------------------- GridSlip -------------------------------------------------

            if ( prop.GetId() ==   idGridSlip()) {
                SetPropertyFloatValue(GridSlip,idGridSlip(),
                                      prop.GetResetvalue(),defaultGridSlip(),
                                      nameGridSlip(),minGridSlip(),
                                      maxtGridSlip(),decimalGridSlip());

            }
            if ( prop.GetId() ==   idGridSlipStep()) {
                SetPropertyFloatValue(GridSlipStep,idGridSlipStep(),
                                      prop.GetResetvalue(),defaultGridSlipStep(),
                                      nameGridSlipStep(),minGridSlipStep(),
                                      maxtGridSlipStep(),decimalGridSlipStep());

            }
            if ( prop.GetId() ==   idSlipNumberPoint()) {
                SetPropertyIntValue(SlipPoint,idSlipNumberPoint(),
                                    prop.GetResetvalue(),defaultSlipNumberPoint(),nameSlipNumberPoint(),
                                    minSlipNumberPoint(),maxSlipNumberPoint(),signedSlipNumberPoint());

            }


            if ( prop.GetId() ==   idIsMomtensCalculate()) {
                SetPropertyEnumValue(IsMomtensCalculate,idIsMomtensCalculate(),
                                     prop.GetResetvalue(),defaultIsMomtensCalculate(),
                                     nameIsMomtensCalculate(),enumBoolList());
            }
        } //while (!in.atEnd())
    }
    void MDocument::SetResetvalueToValue() {
        for_each(values.begin(),values.end(),mem_fun(&PropertyValue::SetResetvalueToValue));
    }
    void MDocument::SetValueToResetvalue() {
        for_each(values.begin(),values.end(),mem_fun(&PropertyValue::SetValueToResetvalue));
    }

    void MDocument::SetValueToInitialValue() {
        for_each(values.begin(),values.end(),mem_fun(&PropertyValue::SetValueToInitialValue));
    }
    void MDocument::SetDefault() {
        for_each(values.begin(),values.end(),mem_fun(&PropertyValue::SetDefault));
    }
    void MDocument::SetInitial() {
        for_each(values.begin(),values.end(),mem_fun(&PropertyValue::SetInitial));
    }
///-------------------------------------------------------------------------------

    QString MDocument::GetSeiInfo(int i) {
        float  R_Earth =  6371.;
        float pi_r = (float)180./((float)4.*(float)atan(1.0));
//    float    z = (float)180./((float)4.*(float)atan(1.0));
        SEISMO **sp = mlt->Get_CS();
        SEISMO *sei = sp[i];
        QString tmp = "";
        tmp += QString(" File: %1\n").
               arg(QString(sei->Get_fname()));
// QMessageBox::warning(0,"",QString("GetSeiInfo 1 %1  %2 ").arg(i).arg(QString(sei->Get_fname())));
        tmp += QString(" Event:\n        Date: %1/%2/%3 %4:%5:%6\n").
               arg(sei->Get_year    (  )).
               arg(sei->Get_month   (  )).
               arg(sei->Get_day     (  )).
               arg(sei->Get_hour    (  )).
               arg(sei->Get_minute  (  )).
               arg(sei->Get_sec     (  ));
        tmp += QString("        Hipo: lat: %1 lon: %2 dep:%3\n").
               arg(sei->Get_elat    (  )).
               arg(sei->Get_elong   (  )).
               arg(sei->Get_depth   (  ));


        tmp += QString(" Station:  '%1'\n        lat: %2 lon:%3 elevation %4\n").
               arg(sei->Get_st_name    (  )).
               arg(sei->Get_slat   (  )).
               arg(sei->Get_slong   (  )).
               arg(sei->Get_elevation   (  ));

        tmp += QString("        type of seismometer: %1 component: %2\n").
               arg(sei->Get_instr   (  )).
               arg(sei->Get_comp    (  )[0]);

        tmp += QString("        polan (angle with vertical): %1\n").
               arg(sei->Get_polan   (  ));
        tmp += QString("        az (azimuth): %1\n").
               arg(sei->Get_az   (  ));

        float rdel =    pi_r* sei->Get_dist() /R_Earth;

        tmp += QString("        delta (distance (degrees) from station to epicenter): %1\n").
               arg(rdel);
        tmp += QString("        aztos (azimuth to station): %1\n").
               arg(sei->Get_aztos   (  ));
        tmp += QString("        azfrs (azimuth from station): %1\n").
               arg(sei->Get_azfrs   (  ));

        tmp += QString(" Record:\n        number of points: %1\n").
               arg(sei->Get_nsampl    (  ));

        tmp += QString("        Start time of record: %1\n").
               arg(sei->Get_t0    (  ));

        if (fabs(sei->Get_pickarrival() - sei->Get_t0( )) > 1.e-5)
            tmp += QString("        Picked first and p-wave arrival  : %2\n").
                   arg(sei->Get_pickarrival ( ));





        tmp += QString("        Time interval: %1\n").
               arg(sei->Get_dt    (  ));


//          tmp += QString("        amp (signal amplitude): %1\n").
//      arg(sei->Get_amp    (  ));

        tmp += QString(" Response:\n");

        tmp += QString("        amp1 (normalization factor): %1\n").
               arg(sei->Get_amp1    (  ));


        tmp += QString("        amp2 (sensitivity of instrument): %1\n").
               arg(sei->Get_amp2    (  ));

        tmp += QString("        number of zeros: %1\n").
               arg(sei->Get_nzer    (  ));

        for (long i = 0; i < sei->Get_nzer( );i++)  {
            tmp += QString("[%1 , %2]; ").
                   arg(sei->Get_re_zer(i)).
                   arg(sei->Get_im_zer(i));
        }
        tmp += "\n";
        tmp += QString("        number of poles:  %1\n").
               arg(sei->Get_npol    (  ));

//QMessageBox::warning(0,"",QString("GetSeiInfo 2 %1  %2 ").arg(sei->Get_nzer( )).arg(sei->Get_npol    ()));
        for (long i = 0; i < sei->Get_npol( );i++)  {
            tmp += QString("[%1 , %2]; ").
                   arg(sei->Get_re_pol(i)).
                   arg(sei->Get_im_pol(i));

        }


        return tmp;
    }
    void MDocument::AppendUsingRecordsMap(QString &key,int i,int k) {
        QString tmp = QString("%1@%2").arg(i).arg(k);
        recordmap.insert(key,tmp);
    }
    bool MDocument::IsAllRecordsProjectInInputDirectory() {
        QStringList inlist  = InListNameSies->GetList();
        QStringList prolist = ProjectListNameSeis->GetList();
        QString outOfRange = "Following project records did not found in Input record list:\n";
        bool find = false;
        for (int i = 0; i < prolist.size(); ++i) {
            QString prorec = prolist.at(i);
            if (inlist.indexOf(prorec) == -1) {
                find = true;
                outOfRange += prorec + "\n";
            }
        }
        if (find) {
            QMessageBox::critical(0,mainWindowCaption(),outOfRange);
            return false;
        }
        return true;
    }




