#ifndef FMT_MOMTENS_DOCUMENT_H
#define FMT_MOMTENS_DOCUMENT_H
#include <QMap>
#include <list>
#include <string>
#include <vector>
#include <map>
#include <set>
#include <iostream>
#include <fstream>
#include <functional>
#include <algorithm>
using namespace std;

class QDataStream;
class  QFile;
class PropertyValue;
class _MLT;
class _MLTS;
class KEN;
class PROBLEMS;
class SEISMO;
class SPECTR;
void delAndNullify(PropertyValue* &pPropVal);
class PropertyValue {
    friend QDataStream &operator<< (QDataStream &, const PropertyValue &);
    friend QDataStream &operator >> (QDataStream &, PropertyValue &);

public:
    enum Kind {
        Property_Bool,
        Property_Int,
        Property_IntSpin,
        Property_Float,
        Property_Enum,
        Property_TransformFunction,
        Property_DateTime,
        Property_String,
        Property_StringList
    };
    PropertyValue() {};
    PropertyValue(const int id,const QString defval,const QString name);
    PropertyValue(const int id,const QString resval,const QString defval,const QString name);
    PropertyValue(const int id,const QString val,const QString resval,const QString defval,const QString name);
    virtual     ~PropertyValue();
    virtual void setChanged( bool b);
    PropertyValue::Kind GetKind() {
        return kind;
    }
    bool isChanged() const;
    bool isReseted() const;
    void SetValueToResetvalue       ()  {
        resetvalue = value;
    }
    void SetValueToInitialValue       () {
        initialvalue = value;
    }
    void SetInitial                 ()  {
        value         =  initialvalue;
    }
    void setChanged                 ()  {
        if (value  ==  initialvalue)   setChanged( FALSE);
        else setChanged( TRUE);
    }
    int GetId                 () const       {
        return id;
    }
    const QString GetName           () const       {
        return name;
    }
    const QString GetInitialvalue   () const       {
        return initialvalue;
    }
    const QString GetDefaultvalue   () const       {
        return defaultvalue;
    }
    const QString GetResetvalue     ()  const      {
        return resetvalue;
    }
    const QString GetValue          ()  const      {
        return value;
    }
    void SetId             (const int i      )     {
        id = i;
    }
    void SetName           (const QString val)     {
        name = val;
    }
    void SetResetvalue     (const QString val)     {
        resetvalue = val;
    }
//---------------------------------------------------------------------------------------
    virtual void SetIntialvalue    (const QString ) {}
    virtual void SetValue          (const QString ) {}
    virtual void SetResetvalueToValue       () {}
    virtual void SetDefault                 () {}
//----------------------------------------------------------------------------------------
protected:
    int id;
    QString value;
    QString initialvalue;
    QString resetvalue;
    QString defaultvalue;
    QString name;
    PropertyValue::Kind kind;
private:
    bool changed;

};
//---------------------------------------------------------------------------------------------
class PropertyBoolValue : public PropertyValue {

public:
    PropertyBoolValue( const int id,const QString defval ,const QString name);
    PropertyBoolValue(const int id,const  QString resval, const QString defval ,const QString name);
    PropertyBoolValue( const int id,const QString val,const  QString resval,const QString defval,const QString name);
    ~PropertyBoolValue() {};
    void SetIntialvalue    (const QString val)     {
        initialvalue= val;
    }
    void SetValue          (const QString val)     {
        value = val;
    }
    void SetResetvalueToValue       ()  {
        value         = resetvalue ;
    }
    void SetDefault                 ()  {
        value         =  defaultvalue ;
    }
private:

};
//---------------------------------------------------------------------------------------------
class PropertyIntValue : public PropertyValue {

public:
    PropertyIntValue( const int id,const QString defval ,const QString name,const int min,const int max, const bool s);
    PropertyIntValue( const int id,const  QString resval,const QString defval ,const QString name,const int min,const int max, const bool s);
    PropertyIntValue( const int id, const  QString val, const  QString resval,const QString defval,const QString name,const int min,const int max, const bool s);
    ~PropertyIntValue() {};
    int  GetMin()          const     {
        return min;
    }
    int  GetMax()          const     {
        return max;
    }
    bool GetSignedValue() const     {
        return   signedValue;
    }

    void SetIntialvalue    (const QString val)     {
        initialvalue= val;
    }
    void SetValue          (const QString val)     {
        value = val;
    }
    void SetResetvalueToValue       ()  {
        value         = resetvalue ;
    }
    void SetDefault                 ()  {
        value         =  defaultvalue ;
    }
private:
    int min;
    int max;
    bool signedValue;
};
//---------------------------------------------------------------------------------------------
class PropertyIntSpinValue : public PropertyValue {

public:
    PropertyIntSpinValue( const int id,const QString defval ,const QString name,const int min,const int max, const int s);
    PropertyIntSpinValue( const int id,const  QString resval,const QString defval ,const QString name,const int min,const int max, const int s);
    PropertyIntSpinValue( const int id, const  QString val, const  QString resval,const QString defval,const QString name,const int min,const int max, const int s);
    ~PropertyIntSpinValue() {};
    int  GetMin()          const     {
        return min;
    }
    int  GetMax()          const     {
        return max;
    }
    int GetStep()         const     {
        return  step;
    }
    void SetIntialvalue    (const QString val)     {
        initialvalue= val;
    }
    void SetValue          (const QString val)     {
        value = val;
    }
    void SetResetvalueToValue       ()  {
        value         = resetvalue ;
    }
    void SetDefault                 ()  {
        value         =  defaultvalue ;
    }

private:
    int min;
    int max;
    int step;
};
//---------------------------------------------------------------------------------------------
class PropertyFloatValue : public PropertyValue {

public:
    PropertyFloatValue(const int id, const QString defval ,const QString name,const float min,const float max, const int dec);
    PropertyFloatValue(const int id, const  QString resval,const QString defval ,const QString name,const float min,const float max, const int dec);
    PropertyFloatValue(const int id, const  QString val, const  QString resval,const QString defval,const QString name,const float min,const float max, const int dec);
    ~PropertyFloatValue() {};
    float GetMin()           const     {
        return min;
    }
    float GetMax()          const     {
        return max;
    }
    void SetMin(float val)        {
        min = val;
    }
    void SetMax(float val)        {
        max = val;
    }
    int GetDecimal()     const     {
        return  decimal;
    }
    void SetIntialvalue    (const QString val)     {
        initialvalue= val;
    }
    void SetValue          (const QString val)     {
        value = val;
    }
    void SetResetvalueToValue       ()  {
        value         = resetvalue ;
    }
    void SetDefault                 ()  {
        value         =  defaultvalue ;
    }
    void SetDefault         (QString val) {
        defaultvalue  =  val;
    }
private:
    float min;
    float max;
    int decimal;
};
//---------------------------------------------------------------------------------------------
class PropertyStringListValue : public PropertyValue {
public:
    PropertyStringListValue(const int id, const QString defval ,const QString name);
    PropertyStringListValue(const int id, const  QString resval,const QString defval ,const QString name);
    PropertyStringListValue(const int id, const QString val,const QString resval,const QString defval,const QString name);
    ~PropertyStringListValue() {
        list.clear();
    }
    void SetValue  (const QString val);
    void SetResetvalueToValue       ();
    void SetDefault                 ();
    void SetInitial                 ();
    QStringList &GetList   ()       {
        return list;
    }
    void SetListToValue(QStringList &list);
private:
    QStringList list;

};

//---------------------------------------------------------------------------------------------
class PropertyStringValue : public PropertyValue {
public:
    PropertyStringValue(const int id, const QString defval ,const QString name);
    PropertyStringValue(const int id, const  QString resval,const QString defval ,const QString name);
    PropertyStringValue(const int id, const QString val,const QString resval,const QString defval,const QString name);
    ~PropertyStringValue() {};

    void SetIntialvalue    (const QString val)     {
        initialvalue= val;
    }
    void SetValue          (const QString val)     {
        value = val;
    }
    void SetResetvalueToValue       ()  {
        value         = resetvalue ;
    }
    void SetDefault                 ()  {
        value         =  defaultvalue ;
    }
private:


};

//---------------------------------------------------------------------------------------------
class PropertyEnumValue : public PropertyValue {
public:
    PropertyEnumValue(const int id, const QString defval ,const QString name,const QString list);
    PropertyEnumValue(const int id, const  QString resval,const QString defval ,const QString name,const QString list);
    PropertyEnumValue(const int id, const QString val,const QString resval,const QString defval,const QString name,const QString list);
    ~PropertyEnumValue() {
        list.clear();
    }
    QStringList &GetList   ()       {
        return list;
    }
    void ChangeEumList(QStringList &list);
    void SetIntialvalue    (const QString val)     {
        initialvalue= val;
    }
    void SetValue          (const QString val)     {
        value = val;
    }
    void SetResetvalueToValue       ()    {
        value         = resetvalue ;
    }
    void SetDefault                 ()    {
        value         =  defaultvalue ;
    }
    void SetDefault (const QString val)  {
        defaultvalue = val;
    }
    QString GetEnumValue (int i) {
        if (i < list.size() && i  >= 0 )return list.at(i);
        else return QString();
    }
private:
    QStringList list;

};
//---------------------------------------------------------------------------------------------
class PropertyTransformFunction : public PropertyValue {

public:
    PropertyTransformFunction(const int id, const QString defval ,const QString name);
    PropertyTransformFunction(const int id, const  QString resval,const QString defval ,const QString name);
    PropertyTransformFunction(const int id, const  QString val, const  QString resval,const QString defval,const QString name);
    ~PropertyTransformFunction() {};

    void SetValue  (const QString val)  {
        value = val;
        SetVal(value);
    }
    void SetResetvalueToValue       ()  {
        value   =  resetvalue;
        SetVal(value);
    }
    void SetDefault                 ()  {
        value   =  defaultvalue;
        SetVal(value);
    }
    void SetInitial                 ()  {
        value   =  initialvalue;
        SetVal(value);
    }
    void SetVal(const QString val);
    void SetDef(const QString val);
    void SetValue(PropertyTransformFunction* val);
    float GetMagnitude() {
        return  magnitude;
    }
    float GetMagnitude(float m);
    void  GetM(float inm,float &outm);
    void  SetMagnitude(float m);
    bool  valIsOn,defIsOn;
    int   valIndexMagnitude,defIndexMagnitude;
    QString valA,valB,valC,valD,defA,defB,defC,defD;

private:
    float magnitude;
};
//---------------------------------------------------------------------------------------------
class PropertyDateTimeValue : public PropertyValue {

public:
    PropertyDateTimeValue( const int id,const QString defval ,const QString name);
    PropertyDateTimeValue( const int id,const  QString resval,const QString defval ,const QString name);
    PropertyDateTimeValue( const int id, const  QString val, const  QString resval,const QString defval,const QString name);
    ~PropertyDateTimeValue() {};
    void SetIntialvalue    (const QString val)     {
        initialvalue= val;
    }
    void SetValue          (const QString val)     {
        value = val;
    }
    void SetResetvalueToValue       ()  {
        value         = resetvalue ;
    }
    void SetDefault                 ()  {
        value         =  defaultvalue ;
    }
};
//---------------------------------------------------------------------------------------------
//---------------------------------------------------------------------------------------------
class MDocument {
    typedef vector<PropertyValue* > PropValVector;
    typedef vector<PropertyValue* >::iterator PropValIterator;
public:
    enum ActionType {
        Value,
        Default,
        Reset,
        Initial
    };
    MDocument();
    MDocument(const QString &fileName);
    MDocument(MDocument *doc);
    QString GetSeiInfo(int i);
    ~MDocument();
    void GetRecordSeisMaterials(QStringList &FileNames);
    bool loadBinary(const QString &fileName);
    bool saveBinary(const QString &fileName);
    bool ReadFileListCtachan();
    bool isModified();
    void SetResetvalueToValue();
    void SetValueToResetvalue();
    void SetValueToInitialValue();
    void SetDefault();
    void SetInitial();
    void SetSeisToProject();
    void SortListSeis();
    bool CheckPathesForProjectFiles();
    bool FindNewLocationOfRecords();
    void AppendUsingRecordsMap(QString &key,int i,int k);
    bool IsAllRecordsProjectInInputDirectory();
    bool GetFtanRange(QString &ftanPeriodFile);
    _MLT  * GetMlt() {
        return mlt;
    }
    QMultiMap<QString,SEISMO *>& GetMapSeies() {
        return map;
    };

    QMultiMap<QString,QString >& GetModelMap() {
        return modelmap;
    }

    QMultiMap<QString,QString>&  GetRecordMap() {
        return recordmap;
    }
///                 Seismograms                            ///
    PropertyStringListValue* &GetListSies() {
        return ListSies;
    }

    PropertyStringListValue* &GetInListNameSies() {
        return InListNameSies;
    }

    PropertyStringListValue* &GetProjectListNameSeis() {
        return ProjectListNameSeis;
    }
    PropertyStringValue* &GetInputDataDirectory() {
        return InputDataDirectory;
    }

    PropertyEnumValue*   &GetIsFtanProject() {
        return IsFtanProject;
    };

///------------------------- Model Panel ------------------------------------------
    PropertyStringValue* &GetModelNameDirectory() {
        return ModelNameDirectory;
    }
    PropertyStringListValue* &GetStationList() {
        return StationList;
    }
    PropertyStringListValue* &GetStationModelList() {
        return StationModelList;
    }
    PropertyEnumValue*   &GetSourceModelType() {
        return SourceModelType;
    }

    PropertyEnumValue*   &GetIsModelCalculate() {
        return IsModelCalculate;
    }

    PropertyStringValue* &GetNameSourceModel() {
        return NameSourceModel;
    }
///------------------------- Range Panel --------------------------------------------
    PropertyFloatValue*  &GetTMinRangeWave() {
        return TMinRangeWave;
    }
    PropertyFloatValue*  &GetTMaxRangeWave() {
        return TMaxRangeWave;
    }
    PropertyEnumValue*  &GetPhVel() {
        return PhVel;
    }
    PropertyFloatValue*  &GetTMin() {
        return TMin;
    }
    PropertyStringListValue* &GetRangeWaveList() {
        return RangeWaveList;
    }


    PropertyFloatValue*  &GetTMinRangeGrid() {
        return TMinRangeGrid;
    }
    PropertyFloatValue*  &GetTMaxRangeGrid() {
        return TMaxRangeGrid;
    }
    PropertyIntValue*    &GetNumberSamples() {
        return NumberSamples;
    }
    PropertyEnumValue*   &GetNumberFft() {
        return NumberFft;
    }
    PropertyStringListValue* &GetRangeGridList() {
        return RangeGridList;
    }



///------------------------- Polarity Panel --------------------------------------------
    PropertyEnumValue*   &GetIsPolaritySuplementary() {
        return IsPolaritySuplementary;
    }
    PropertyEnumValue*   &GetIsPolarityBulletin() {
        return IsPolarityBulletin;
    }

    PropertyStringValue* &GetSuplementaryFileName() {
        return SuplementaryFileName;
    }
    PropertyStringValue* &GetBulletinFileName() {
        return BulletinFileName;
    }

    PropertyFloatValue*  &GetPolarityDepth() {
        return PolarityDepth;
    }
    PropertyFloatValue*  &GetPolarityVpsource() {
        return PolarityVpsource;
    }
    PropertyFloatValue*  &GetPolaritySmoothing() {
        return PolaritySmoothing;
    }
    PropertyEnumValue*   &GetIsPolaritySmoothing() {
        return IsPolaritySmoothing;
    }

   PropertyEnumValue*   &GetIsMomtensCalculate() {
        return IsMomtensCalculate;
    }
///------------------------- Grid Panel --------------------------------------------

///------------------------- GridDepth ----------------------------------------------

    PropertyFloatValue* &GetGridDepth() {
        return GridDepth;
    }
    PropertyFloatValue* &GetGridDepthStep() {
        return GridDepthStep;
    }
    PropertyIntValue*   &GetDepthPoint() {
        return DepthPoint;
    }
///------------------------- GridDip -------------------------------------------------

    PropertyFloatValue* &GetGridDip() {
        return GridDip;
    }
    PropertyFloatValue* &GetGridDipStep() {
        return GridDipStep;
    }
    PropertyIntValue*   &GetDipPoint() {
        return DipPoint;
    }
///------------------------- GridStrike -------------------------------------------------

    PropertyFloatValue* &GetGridStrike() {
        return GridStrike;
    }
    PropertyFloatValue* &GetGridStrikeStep() {
        return GridStrikeStep;
    }
    PropertyIntValue*   &GetStrikePoint() {
        return StrikePoint;
    }

///------------------------- GridSlip -------------------------------------------------

    PropertyFloatValue* &GetGridSlip() {
        return GridSlip;
    }
    PropertyFloatValue* &GetGridSlipStep() {
        return GridSlipStep;
    }
    PropertyIntValue*   &GetSlipPoint() {
        return SlipPoint;
    }

///===================================================================================
    vector <PropertyValue* >  &GetValues() {
        return values;
    }
    bool GetProjectFileGood()  {
        return projectFileGood;
    }
    bool isChanged() const;
    bool CheckModified();
    void SetPropertyEnumValue(PropertyEnumValue* &p, int id,QString reset,QString defaultvalue,
                              QString name,QString enumeration);
    void SetPropertyIntValue(PropertyIntValue* &p, int id,QString reset,QString defaultvalue,
                             QString name,int min,int max,bool sign);
    void SetPropertyFloatValue(PropertyFloatValue* &p, int id,QString reset,QString defaultvalue,
                               QString name,float min,float max,int decimal);
    void SetPropertyTransformFunction(PropertyTransformFunction* &p, int id,
                                      QString reset,QString defaultvalue, QString name);
    void SetPropertyStringValue(PropertyStringValue* &p, int id,
                                QString reset,QString defaultvalue, QString name);
    void SetPropertyStringListValue(PropertyStringListValue* &p, int id,
                                    QString reset,QString defaultvalue, QString name);
private:
    bool projectFileGood;
    bool changed;
    int pinvVersionFromProjectFile;
    enum { MagicNumber = 0x73c56f12 };
    void writeToStream(QDataStream &out);
    void readFromStream(QDataStream &in);
    void error(const QFile &file ,const QString &message);
    void ioError(const QFile &file ,const QString &message);

///                 Seismograms                          ///
    PropertyStringValue* InputDataDirectory;
    PropertyStringListValue* ListSies;
    PropertyStringListValue* InListNameSies;
    PropertyStringListValue* ProjectListNameSeis;
    PropertyEnumValue*   IsFtanProject;

///------------------------- Model Panel ------------------------------------------
    PropertyStringValue* ModelNameDirectory;
    PropertyStringListValue* StationList;
    PropertyStringListValue* StationModelList;
    PropertyEnumValue*   SourceModelType;
    PropertyEnumValue*   IsModelCalculate;
    PropertyStringValue* NameSourceModel;
///------------------------- Range Panel --------------------------------------------
    PropertyFloatValue*  TMinRangeWave;
    PropertyFloatValue*  TMaxRangeWave;
    PropertyEnumValue*   PhVel;
    PropertyFloatValue*  TMin;
    PropertyStringListValue* RangeWaveList;


    PropertyFloatValue*  TMinRangeGrid;
    PropertyFloatValue*  TMaxRangeGrid;
    PropertyIntValue*    NumberSamples;
    PropertyEnumValue*   NumberFft;
    PropertyStringListValue* RangeGridList;

///------------------------- Polarity Panel --------------------------------------------
    PropertyEnumValue*   IsPolaritySuplementary;
    PropertyEnumValue*   IsPolarityBulletin;

    PropertyStringValue* SuplementaryFileName;
    PropertyStringValue* BulletinFileName;

    PropertyFloatValue*  PolarityDepth;
    PropertyFloatValue*  PolarityVpsource;
    PropertyFloatValue*  PolaritySmoothing;

    PropertyEnumValue*   IsPolaritySmoothing;

///------------------------- Grid Panel --------------------------------------------

///------------------------- GridDepth ----------------------------------------------

    PropertyFloatValue* GridDepth;
    PropertyFloatValue* GridDepthStep;
    PropertyIntValue*   DepthPoint;
///------------------------- GridDip -------------------------------------------------

    PropertyFloatValue* GridDip;
    PropertyFloatValue* GridDipStep;
    PropertyIntValue*   DipPoint;
///------------------------- GridStrike -------------------------------------------------

    PropertyFloatValue* GridStrike;
    PropertyFloatValue* GridStrikeStep;
    PropertyIntValue*   StrikePoint;

///------------------------- GridSlip -------------------------------------------------

    PropertyFloatValue* GridSlip;
    PropertyFloatValue* GridSlipStep;
    PropertyIntValue*   SlipPoint;

   PropertyEnumValue*   IsMomtensCalculate;

///========================================================================================

    _MLT * mlt;
    PropValVector values;
    QMultiMap<QString,SEISMO *> map;
    QMultiMap<QString,QString>  recordmap;
    QMultiMap<QString,QString>  modelmap;
};


#endif
