#ifndef MAINWINDOW_H
#define MAINWINDOW_H
#include <QMainWindow>
#include <QString>
#include <QDir>
class QAction;
class QMenu;
class QToolBar;
class InfoProject;
class QSplitter;
class MDocument;
class ViewResult;
class AppKonsole;
bool hasInputFiles(QString directory,QStringList &list);
class MainWindow : public QMainWindow {
    Q_OBJECT

public:
    MainWindow();
    ~MainWindow();
    static MainWindow *self;
    void removeWorkingMaterials(QString & temporaryDirectory,bool removeTempDirectory = true);
    QDir homeDirectory;
    QDir tempDirectory;
    QString    tempDirectoryPath;
    QString version;
    QString projectVersion;
    QString strippedName(const QString &fullFileName);
    const QString AbsDir(const QString fileName) const;
    bool modifiedProject;
    QString openUserProjectDirectory,saveUserProjectDirectory,openInputDataDirectory,openModelDirectory;
    QString openUserPolarityDirectory;
    QString projectFileFilters;
    QString GetCurFile() {
        return curFile;
    }
    MDocument* GetDocument() {
        return doc;
    }
    AppKonsole* GetKonsole() {
        return appkonsole;
    }
    QPoint projectWizardPos;
    QSize  projectWizardSize;
protected:
    void closeEvent(QCloseEvent *event);
private slots:
    void openRecentProjectLocation();
    void about();
    void zoomin();
    void zoomout();
    void editProject();
    void newProject();
    void openProject();
    bool saveProject();
    bool saveAsProject();
    void execute();
private:
    void setCurrentProjectFile(const QString &fileName);
    void updateRecentProjectFileActions();
    QString curFile;
    enum { MaxRecentFiles = 5 };
    QAction *recentProjectFileActs[MaxRecentFiles];
    bool maybeProjectSave();
    bool saveProjectLocation(QString &fileName);
    bool SaveWorkingMaterials(QString &dirName);
    void CopyDirectory(QString &from,QString &to);
    bool loadProjectLocation(QString &fileName);
    void Get3SmacModels();
    void createActions();
    void createMenus();
    void createToolBars();
    void createStatusBar();
    void readSettings();
    void writeSettings();
    void initUI();
    QString  homeDirectoryPath;
    void MakeTempDirectory();
    void setDocumentModified ();
    bool maybeSave();
    void FileActsShowHide(bool b);
    QList<QAction *> showHideFileActs;
    QMenu *fileMenu;
    QMenu *viewMenu;
    QMenu *editMenu;
    QMenu *helpMenu;
    QToolBar *fileToolBar;
    QAction *newAct;
    QAction *openAct;
    QAction *editAct;
    QAction *executeAct;
    QAction *saveAct;
    QAction *saveAsAct;
    QAction *aboutAct;
    QAction *aboutQtAct;
    QAction *exitAct;
    QAction *zoominAct;
    QAction *zoomoutAct;

    QAction *separatorProjectAct;
    MDocument* doc;
    InfoProject* infoproject;
    ViewResult* viewresult;
    AppKonsole* appkonsole;
    QSplitter * splitter;
    QSplitter * splitterVert;
};
#endif
