#include <QtGui>
#include <QMessageBox>
#include <QProcess>
#include "default.h"
#include "document.h"
#include "mainwindow.h"
#include "projectwizard.h"
#include "editline.h"
#include "records.h"
#include "models.h"
#include "range.h"
#include "grid.h"
#include "polarities.h"
static bool firstWizardVisit  = true;
ProWizard *ProWizard::wizardSelf = 0;

ProWizard::~ProWizard() {

}
ProWizard::ProWizard(QWidget *parent,MDocument* doc,DocumentUsing documentUsing)
		: QWizard(parent),doc(doc),documentUsing(documentUsing) {
	setFont( parent->font() );
	wizardSelf = this;
	firstWizardVisit  = true;

	recordsPage  = new RecordsPage(this,doc,documentUsing);
	modelsPage  = new ModelsPage(this,doc,documentUsing);
	rangePage  = new RangePage(this,doc,documentUsing);
	gridPage  = new GridPage(this,doc,documentUsing);
	polarPage  = new PolarPage(this,doc,documentUsing);

	//setPage(Page_Directories,directoriesPage);
	setPage(Page_Records, recordsPage);
	setPage(Page_Models, modelsPage);
	setPage(Page_Range, rangePage);
	setPage(Page_Polar, polarPage);
	setPage(Page_Grid, gridPage);

	setStartId(Page_Records);

	if (documentUsing == ProWizard::ProjectNew)
		setWindowTitle(tr("Project Wizard - New Project"));
	if (documentUsing == ProWizard::ProjectEdit)
		setWindowTitle(QString("Project Wizard - Edit Project: %1").
		               arg(MainWindow::self->strippedName(MainWindow::self->GetCurFile())));


	if (documentUsing == ProWizard::ProjectRepair)
		setWindowTitle(QString("Project Wizard - Repair Project"));

	connect( button ( QWizard::FinishButton ), SIGNAL( clicked() ), this, SLOT(finishButton()));


}

void ProWizard::finishButton() {
//QMessageBox::critical(0,"","ProWizard::finishButton()");
	QList<int> listPages = visitedPages ();
	for (int i = 0; i < listPages.size(); ++i) {
		QWizardPage * pg  = page(listPages.at(i));
		if (WizardPage* wd = qobject_cast<WizardPage *>(pg)) wd->FinishWizard();
	}

}
///====================================================================================
RecordsPage::RecordsPage(QWidget *parent,MDocument* doc,ProWizard::DocumentUsing documentUsing)
		: WizardPage(parent) {
	setTitle(tr("The lists of records page."));
	setSubTitle(tr("Browse the input data directory.Define the project list without double records."));

	if (documentUsing == ProWizard::ProjectNew)
		setPixmap(QWizard::LogoPixmap, QPixmap(":/images/wizardnew.png"));
	if (documentUsing == ProWizard::ProjectEdit)
		setPixmap(QWizard::LogoPixmap, QPixmap(":/images/wizardedit.png"));
	if (documentUsing == ProWizard::ProjectRepair)
		setPixmap(QWizard::LogoPixmap, QPixmap(":/images/wizardrepair.png"));

	recordsImpl = new RecordsImpl(this,doc,documentUsing);

	recordsImpl->setWindowFlags(Qt::SubWindow);
	recordsImpl->setSizeGripEnabled ( FALSE );
	recordsImpl->setModal (false);

	QGridLayout *layout = new QGridLayout;
	layout->addWidget(recordsImpl);
	setLayout(layout);


//    if (documentUsing == ProWizard::ProjectNew) {
//        registerField("directory.DbaseName*", directoriesPage->GetLineEditDbaseName());
//        registerField("directory.DbaseDirectory*", directoriesPage->GetLineEditDbaseDirectory());
//    }


}


/*
void RecordsPage::cleanupPage () {
//QMessageBox::critical(0,"", QString("DirectoriesPage::cleanupPage"));
}

*/
bool RecordsPage::validatePage() {
//QMessageBox::critical(0,"", QString("DirectoriesPage::validatePage"));
	if (!recordsImpl->CheckDialogParameeters() ) return false;
	return true;
}

void RecordsPage::FinishWizard() {
	recordsImpl->FinishWizard();
}

///====================================================================================
ModelsPage::ModelsPage(QWidget *parent,MDocument* doc,ProWizard::DocumentUsing documentUsing)
		: WizardPage(parent) {
	setTitle(tr("Models Info"));
	setSubTitle(tr("Define the date and the time of the event and\nthe time duration of output records."));

	if (documentUsing == ProWizard::ProjectNew)
		setPixmap(QWizard::LogoPixmap, QPixmap(":/images/wizardnew.png"));
	if (documentUsing == ProWizard::ProjectEdit)
		setPixmap(QWizard::LogoPixmap, QPixmap(":/images/wizardedit.png"));
	if (documentUsing == ProWizard::ProjectRepair)
		setPixmap(QWizard::LogoPixmap, QPixmap(":/images/wizardrepair.png"));

	modelsPageImpl = new ModelsPageImpl(this,doc,documentUsing);

	modelsPageImpl->setWindowFlags(Qt::SubWindow);
	modelsPageImpl->setSizeGripEnabled ( FALSE );
	modelsPageImpl->setModal (false);

	QGridLayout *layout = new QGridLayout;
	layout->addWidget(modelsPageImpl);
	setLayout(layout);
}
void ModelsPage::FinishWizard() {
	modelsPageImpl->FinishWizard();
}
bool ModelsPage::validatePage() {
	if (!modelsPageImpl->CheckDialogParameeters() ) return false;
	return true;
}
///====================================================================================
RangePage::RangePage(QWidget *parent,MDocument* doc,ProWizard::DocumentUsing documentUsing)
		: WizardPage(parent) {
	setTitle(tr("Range Info"));
	setSubTitle(tr("Define the hypocenter of the events and magnitudes."));

	if (documentUsing == ProWizard::ProjectNew)
		setPixmap(QWizard::LogoPixmap, QPixmap(":/images/wizardnew.png"));
	if (documentUsing == ProWizard::ProjectEdit)
		setPixmap(QWizard::LogoPixmap, QPixmap(":/images/wizardedit.png"));
	if (documentUsing == ProWizard::ProjectRepair)
		setPixmap(QWizard::LogoPixmap, QPixmap(":/images/wizardrepair.png"));

	rangeImpl = new RangeImpl(this,doc,documentUsing);

	rangeImpl->setWindowFlags(Qt::SubWindow);
	rangeImpl->setSizeGripEnabled ( FALSE );
	rangeImpl->setModal (false);

	QGridLayout *layout = new QGridLayout;
	layout->addWidget(rangeImpl);
	setLayout(layout);

}
bool RangePage::validatePage() {

	if (!rangeImpl->CheckDialogParameeters() ) return false;
	return true;
}
void RangePage::FinishWizard() {
	rangeImpl->FinishWizard();
}

///====================================================================================
GridPage::GridPage(QWidget *parent,MDocument* doc,ProWizard::DocumentUsing documentUsing)
		: WizardPage(parent) {
	setTitle(tr("Grid Info"));
	setSubTitle(tr("Define the hypocenter of the events and magnitudes."));

	if (documentUsing == ProWizard::ProjectNew)
		setPixmap(QWizard::LogoPixmap, QPixmap(":/images/wizardnew.png"));
	if (documentUsing == ProWizard::ProjectEdit)
		setPixmap(QWizard::LogoPixmap, QPixmap(":/images/wizardedit.png"));
	if (documentUsing == ProWizard::ProjectRepair)
		setPixmap(QWizard::LogoPixmap, QPixmap(":/images/wizardrepair.png"));

	gridImpl = new GridImpl(this,doc);

	gridImpl->setWindowFlags(Qt::SubWindow);
	gridImpl->setSizeGripEnabled ( FALSE );
	gridImpl->setModal (false);

	QGridLayout *layout = new QGridLayout;
	layout->addWidget(gridImpl);
	setLayout(layout);

}
bool GridPage::validatePage() {

	if (!gridImpl->CheckDialogParameters() ) return false;
	return true;
}
void GridPage::FinishWizard() {
	gridImpl->FinishWizard();
}
///====================================================================================
PolarPage::PolarPage(QWidget *parent,MDocument* doc,ProWizard::DocumentUsing documentUsing)
		: WizardPage(parent) {
	setTitle(tr("Polarity Info"));
	setSubTitle(tr("Define the hypocenter of the events and magnitudes."));

	if (documentUsing == ProWizard::ProjectNew)
		setPixmap(QWizard::LogoPixmap, QPixmap(":/images/wizardnew.png"));
	if (documentUsing == ProWizard::ProjectEdit)
		setPixmap(QWizard::LogoPixmap, QPixmap(":/images/wizardedit.png"));
	if (documentUsing == ProWizard::ProjectRepair)
		setPixmap(QWizard::LogoPixmap, QPixmap(":/images/wizardrepair.png"));

	polarImpl = new PolarImpl(this,doc,documentUsing);

	polarImpl->setWindowFlags(Qt::SubWindow);
	polarImpl->setSizeGripEnabled ( FALSE );
	polarImpl->setModal (false);

	QGridLayout *layout = new QGridLayout;
	layout->addWidget(polarImpl);
	setLayout(layout);

}
bool PolarPage::validatePage() {
	if (!polarImpl->CheckDialogParameeters() ) return false;
	return true;
}
void PolarPage::FinishWizard() {
	polarImpl->FinishWizard();
}
