#ifndef PROJECT_WIZARD_H
#define PROJECT_WIZARD_H
#include <QWizard>

class WizardPage;
class LineDocProperty;
class PropertyValue;

class RecordsPage;
class RecordsImpl;

class ModelsPage;
class ModelsPageImpl;

class RangePage;
class RangeImpl;

class GridPage;
class GridImpl;

class PolarPage;
class PolarImpl;

class MDocument;

class ProWizard : public QWizard {
    Q_OBJECT

public:

    enum Pages { Page_Directories,
                 Page_Records,
                 Page_Models,
                 Page_Range,
                 Page_Polar,
                 Page_Grid
               };

    enum DocumentUsing {
        ProjectNew,
        ProjectEdit,
        ProjectRepair
    };
    ProWizard(QWidget *parent,MDocument* doc,DocumentUsing documentUsing);
    ~ProWizard();
    static ProWizard *wizardSelf;

    RecordsPage* recordsPage;
    ModelsPage* modelsPage;
    RangePage* rangePage;
    GridPage* gridPage;
    PolarPage* polarPage;

    DocumentUsing GetDocumentUsing() {
        return documentUsing;
    }
private slots:
    void finishButton();
private:
    MDocument* doc;
    DocumentUsing documentUsing;
};
///=====================================================================
class WizardPage : public QWizardPage {

    Q_OBJECT

public:
    WizardPage(QWidget * ) {};
    virtual     ~WizardPage() {}
    virtual void FinishWizard() = 0;
private:
};
///=====================================================================
class RecordsPage : public WizardPage {
    Q_OBJECT

public:
    RecordsPage(QWidget *parent,MDocument* doc,ProWizard::DocumentUsing documentUsing);
    RecordsImpl* recordsImpl;
    void FinishWizard();
    bool validatePage();
//    void cleanupPage ();
private slots:
private:
};

///=====================================================================
class ModelsPage : public WizardPage {
    Q_OBJECT

public:
    ModelsPage(QWidget *parent,MDocument* doc,ProWizard::DocumentUsing documentUsing);
    ModelsPageImpl* modelsPageImpl;
    void FinishWizard();
    bool validatePage();

private:
};
///=====================================================================
class RangePage : public WizardPage {
    Q_OBJECT

public:
    RangePage(QWidget *parent,MDocument* doc,ProWizard::DocumentUsing documentUsing);
    RangeImpl* rangeImpl;
    void FinishWizard();
    bool validatePage();

private:
};
///=====================================================================
class GridPage : public WizardPage {
    Q_OBJECT

public:
    GridPage(QWidget *parent,MDocument* doc,ProWizard::DocumentUsing documentUsing);
    GridImpl* gridImpl;
    void FinishWizard();
    bool validatePage();

private:
};
///=====================================================================
class PolarPage : public WizardPage {
    Q_OBJECT

public:
    PolarPage(QWidget *parent,MDocument* doc,ProWizard::DocumentUsing documentUsing);
    PolarImpl* polarImpl;
    void FinishWizard();
    bool validatePage();

private:
};
#endif
