//
// C++ Implementation: range
//
// Description:
//
//
// Author: fmt <fmt@localhost.localdomain>, (C) 2008
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include <QtGui>
#include <QMessageBox>
#include <QProcess>
#include "default.h"
#include "document.h"
#include "mainwindow.h"
#include "projectwizard.h"
#include "range.h"
#include "editline.h"
#include "sei.h"
#include <math.h>

const float Pi  = 3.1415926;
const float Pi2 = 6.2831852;
RangeImpl::RangeImpl ( QWidget *parent,MDocument* doc,ProWizard::DocumentUsing documentUsing )
		: QDialog ( parent ),doc ( doc ),documentUsing ( documentUsing )
{
	setFont ( parent->font() );
	setWindowIcon ( QIcon ( ":/images/icon.png" ) );
	setupUi ( this );
	InitDialog();
	SetDocumentToDialog();

}
void RangeImpl::FinishWizard()
{
	GetDocumentFromDialog();
//    foreach (PropertyValue* val, values) val->SetValueToResetvalue();
}
void RangeImpl::InitDialog()
{

	labelTotalNum->setText ( "" );
	values.append ( doc->GetTMinRangeWave() );
	values.append ( doc->GetTMaxRangeWave() );
	values.append ( doc->GetPhVel() );
	values.append ( doc->GetTMin() );
	values.append ( doc->GetRangeWaveList() );
	values.append ( doc->GetTMinRangeGrid() );
	values.append ( doc->GetTMaxRangeGrid() );
	values.append ( doc->GetNumberSamples() );
	values.append ( doc->GetNumberFft() );
	values.append ( doc->GetRangeGridList() );


	propDocLine.append ( new LineDocProperty ( lineEditTMinIn,  doc->GetTMinRangeWave() ) );
	propDocLine.append ( new LineDocProperty ( lineEditTMaxIn, doc->GetTMaxRangeWave() ) );
	propDocLine.append ( new LineDocProperty ( lineEditTMin, doc->GetTMin() ) );
	propDocLine.append ( new LineDocProperty ( lineEditTMinRange, doc->GetTMinRangeGrid() ) );
	propDocLine.append ( new LineDocProperty ( lineEditTMaxRange, doc->GetTMaxRangeGrid() ) );
	propDocLine.append ( new LineDocProperty ( spinBoxNw, doc->GetNumberSamples() ) );

	comboBoxNFFT->clear();
	QStringList lst = doc->GetNumberFft()->GetList();
	for ( int i = 0; i < lst.size(); i++ ) comboBoxNFFT->addItem ( lst.at ( i ) );
	comboBoxNFFT->setCurrentIndex ( doc->GetNumberFft()->GetValue().toInt() );
	comboBoxNFFT->adjustSize();
	listWidgetWave->setSelectionMode ( QAbstractItemView::MultiSelection );
	listWidgetGrid->setSelectionMode ( QAbstractItemView::SingleSelection );
	listWidgetLines->setSelectionMode ( QAbstractItemView::NoSelection );
	connect ( pushButtonSetSels, SIGNAL ( clicked() ), this, SLOT ( setsel() ) );
	connect ( pushButtonSetAll,  SIGNAL ( clicked() ), this, SLOT ( setall() ) );
	connect ( pushButtonApplay,  SIGNAL ( clicked() ), this, SLOT ( applay() ) );
	connect ( listWidgetGrid, SIGNAL ( doubleClicked ( const QModelIndex & ) ), this, SLOT ( doubleClickGridList ( const QModelIndex & ) ) );
}

bool RangeImpl::CheckDialogParameeters(){
return true;
}
void RangeImpl::doubleClickGridList ( const QModelIndex & index )
{
	if ( index == QModelIndex() ) return;
	int row = index.row();
	QListWidgetItem * item = listWidgetGrid->item ( row );
	QString str = item->data ( Qt::DisplayRole ).toString();
	QStringList lst = str.split ( ' ',QString::SkipEmptyParts );
	float per = lst.at ( 1 ).toFloat();
	float freq = Pi2/per;
	int points = lst.at ( 2 ).toInt();
//fprintf (stderr,"freq = %f per = %f points %d stepFreqDEscr %f\n",freq,per,points,stepFreqDEscr);
//float stfreq = 0.04008;//wmax[0];
//float stper = Pi2/stfreq;
//fprintf (stderr,"stfreq = %f stper = %f\n",stfreq,stper);
	float dw = wdw0[row];
	float d_w = wmax[row];

	listWidgetLines->clear();
	QStringList list;
	for ( int i = 0; i< points;++i )
	{
                
		float frequency = d_w;
		float period = Pi2/frequency;
		QString tmp = QString ( "%1: %2 %3" ).arg ( i+1 ).arg ( frequency ).arg ( period );
		list << tmp;
                d_w -= dw;
	}

	listWidgetLines->addItems ( list );

}
void RangeImpl::SetDocumentToDialog()
{
	foreach ( LineDocProperty *docprop, propDocLine ) docprop->SetValueFromDoc();
	QStringList list = doc->GetRangeWaveList()->GetList();
	listWidgetWave->addItems ( list );
	if ( doc->GetPhVel()->GetValue() == "True" ) checkBox->setChecked ( true );
	else checkBox->setChecked ( false );
}
void RangeImpl::GetDocumentFromDialog()
{
	QStringList list;
	for ( int j = 0; j < listWidgetWave->count (); ++j )
	{
		QListWidgetItem * itemProj = listWidgetWave->item ( j );
		QString str = itemProj->data ( Qt::DisplayRole ).toString();
		list << str;
	}
	doc->GetRangeWaveList()->SetListToValue ( list );
	doc->GetTMinRangeWave()->SetValue ( lineEditTMinIn->text() );
	doc->GetTMinRangeGrid()->SetValue ( lineEditTMinRange->text() );
	doc->GetTMaxRangeWave()->SetValue ( lineEditTMaxIn->text() );
	doc->GetTMaxRangeGrid()->SetValue ( lineEditTMaxRange->text() );
	doc->GetNumberSamples()->SetValue ( QString ( "%1" ).arg ( spinBoxNw->value() ) );
	doc->GetNumberFft()->SetValue ( QString ( "%1" ).arg ( comboBoxNFFT->currentIndex() ) );
	if ( checkBox->isChecked () ) doc->GetPhVel()->SetValue ( "True" );
	else doc->GetPhVel()->SetValue ( "False" );

}
void RangeImpl::SetRangeWaveList ( QStringList &RangeWaveListTmp,float min,float max )
{
	listWidgetWave->addItems ( RangeWaveListTmp );
	lineEditTMinIn->setText ( QString ( "%1" ).arg ( min ) );
	lineEditTMinRange->setText ( QString ( "%1" ).arg ( min ) );
	lineEditTMaxIn->setText ( QString ( "%1" ).arg ( max ) );
	lineEditTMaxRange->setText ( QString ( "%1" ).arg ( max ) );

}

void RangeImpl::setsel()
{
	QItemSelectionModel * selmod = listWidgetWave->selectionModel ();
	QItemSelection sel = selmod->selection ();
	if ( sel.count () == 0 ) return;
	QItemSelectionRange range;
	for ( int i = 0; i < sel.count (); ++i )
	{
		range = sel.at ( i );
		int selTopRow    =   range.top();
		int selNumRows   =   range.height();
		for ( int j = 0; j < selNumRows; ++j )
		{
			int row = selTopRow + j;
			QListWidgetItem * item = listWidgetWave->item ( row );
			QString str = item->data ( Qt::DisplayRole ).toString();
			QStringList list = str.split ( ' ',QString::SkipEmptyParts );
			QString name = list.at ( 0 );
			QString min = lineEditTMinIn->text();
			QString max = lineEditTMaxIn->text();
			str = QString ( "%1 %2 %3" ).arg ( name ).arg ( min ).arg ( max );
			item->setData ( Qt::DisplayRole, str );
		}
	}
	listWidgetWave->clearSelection ();
}
void RangeImpl::setall()
{

	QStringList list;
	QString name;
	QString min;
	QString max;

	for ( int i = 0; i< listWidgetWave->count (); ++i )
	{

		QListWidgetItem * item = listWidgetWave->item ( i );
		QString str = item->data ( Qt::DisplayRole ).toString();
		list = str.split ( ' ',QString::SkipEmptyParts );
		name = list.at ( 0 );
		min = lineEditTMinIn->text();
		max = lineEditTMaxIn->text();
		str = QString ( "%1 %2 %3" ).arg ( name ).arg ( min ).arg ( max );
		item->setData ( Qt::DisplayRole, str );

	}

}
void RangeImpl::applay()
{


	float Min_X = Pi2/lineEditTMaxRange->text().toFloat(); //Pi2/ComTmax;
	float Max_X = Pi2/lineEditTMinRange->text().toFloat();; //Pi2/ComTmin;
	QString name;
	QString ss;
	float r_min,r_max;
	int j;
	int i;
	int N2N = comboBoxNFFT->currentText().toInt();
	int NPoints = spinBoxNw->value();
	int N_RecDescrList = listWidgetWave->count();
	float ComTmin = lineEditTMinRange->text().toFloat();
	float ComTmax = lineEditTMaxRange->text().toFloat();
///============================================================================================

	float * m_w_min = new float[N_RecDescrList];
	float * m_w_max = new float[N_RecDescrList];
	float * m_t_min = new float[N_RecDescrList];
	float * m_t_max = new float[N_RecDescrList];
	float * m_dw0 = new float[N_RecDescrList];
	int   * m_ipoint= new int[N_RecDescrList];


///============================================================================================

	_MLT* mlt = doc->GetMlt();
	SEISMO **cs = mlt->Get_CS();
	float m_dt = cs[0]->Get_dt();
	float m_dw   = Pi2/ ( m_dt*N2N );
	int m_nff = N2N;
	int m_npoints = NPoints;

	int n_item = N_RecDescrList;

	int m_nitems  = N_RecDescrList;

	int numdw = m_npoints - 1;
	float dw1 = ( Max_X - Min_X ) /numdw;

//fprintf (stderr,"Info:> dw = %f dt = %f m_dw = %f dw1 = %f\n",m_dw,m_dt,m_dw,dw1);

	if ( dw1  < m_dw )
	{
		int nn = ( int ) ( m_dw/ dw1 + 1 );
		QMessageBox::warning ( 0,mainWindowCaption(),
		                       QString ( "Number of frequencies for FTT should be %1 times bigger!" ).arg ( nn ) );
		return;
	}
	long idw = ( long ) ( dw1/m_dw ); //+ 0.5
	dw1 = m_dw * idw;

	long iwmax = ( long ) ( Max_X/m_dw + 0.5 );
	long n2n1 = m_nff/2;
	if ( iwmax > n2n1 )
	{
		QMessageBox::warning ( 0,mainWindowCaption(),
		                       QString ( "T_min (%1) is too small! " ).arg ( ComTmin ) );
		return;
	}

	if ( dw1 <  1.e-4 )
	{
		QMessageBox::warning ( 0,mainWindowCaption(),
		                       QString ( "Omega step (%1) is too small!" ).arg ( dw1 ) );
	}

	iwmax = ( long ) ( Min_X/m_dw + 0.5 );
	if ( ( Min_X = m_dw*iwmax ) < Pi2/ComTmax ) Min_X += m_dw;
	float m_tmin_com = ComTmin;
	//float m_tmax_com =  ComTmax;


	int  iw;

	if ( ( Max_X = Min_X + dw1*numdw ) > Pi2 / m_tmin_com )
	{
		Max_X -= dw1 ;
		m_npoints -=  1;
	}
	else
	{

		iw = ( int ) ( ( Pi2 / m_tmin_com - Max_X ) /dw1 ) ;

		Max_X += dw1*iw;
		m_npoints += iw;
	}
	float m_tmin_com_e = Pi2/   Max_X;
	float m_tmax_com_e = Pi2/   Min_X;
	NPoints =   m_npoints;
	// float min_max;
	// long total_n;
	float wrk;
	wmax.clear();
	wdw0.clear();
	for ( i = 0; i < n_item; i++ )
	{
		//  sscanf(RecPeriodList[i],"%s%e%e",name,&r_min,&r_max);


		QListWidgetItem * item = listWidgetWave->item ( i );
		QString str = item->data ( Qt::DisplayRole ).toString();
		QStringList list = str.split ( ' ',QString::SkipEmptyParts );
		name = list.at ( 0 );
		r_min =  list.at ( 1 ).toInt();
		r_max =  list.at ( 2 ).toInt();
		m_w_min[i] = Pi2/r_max;
		m_w_max[i] = Pi2/r_min ;
		m_ipoint[i] = 0;
		if ( ! ( Max_X < m_w_min[i] || Min_X > m_w_max[i] ) )
		{
			wrk = Min_X - dw1;
			if ( m_w_min[i] < Min_X ) m_w_min[i] = Min_X; else
			{
				while ( ( wrk += dw1 ) < m_w_min[i] );
				m_w_min[i] = wrk;
			}
			m_ipoint[i]  += 1;
			wrk = m_w_min[i];
			if ( m_w_max[i] > Max_X ) m_w_max[i] = Max_X; //Pi2/m_tmin_com;
			while ( ( wrk + dw1 ) <= m_w_max[i] + dw1* 1.e-3 )   // + m_dw
			{
				wrk += dw1; m_ipoint[i]  += 1;
			}
			m_w_max[i] = wrk;
			wmax.append ( wrk );
			m_t_min[i] = Pi2/m_w_max[i];
			m_t_max[i] = Pi2/m_w_min[i];
			m_dw0[i]   = dw1;
			wdw0.append ( dw1 );

		}
		else
		{
			m_t_min[i] = ( float ) 0.;
			m_t_max[i] = ( float ) 0.;

		}


	}

	// m_tmin_com_e = m_t_min[0];
	// m_tmax_com_e = m_t_max[0];
	int TotalNum = 0;
	for ( j = 0; j < n_item; j++ )
	{

		if ( m_ipoint[j] > 0 )
		{
			// if (m_t_min[j]  < m_tmin_com_e ) m_tmin_com_e = m_t_min[j];
			// if (m_t_max[j]  > m_tmax_com_e ) m_tmax_com_e  = m_t_max[j];
		}
		TotalNum +=  m_ipoint[j];
	}


	float  CorrComTmin = m_tmin_com_e;
	float  CorrComTmax = m_tmax_com_e;
	labelTMinOut->setText ( QString ( "%1" ).arg ( CorrComTmin ) );
	labelTMaxOut->setText ( QString ( "%1" ).arg ( CorrComTmax ) );
	labelTotalNum->setText ( QString ( "Total:%1; " ).arg ( TotalNum ) );

	listWidgetGrid->clear ();
	QStringList list;
	for ( int j = 0; j < listWidgetWave->count (); ++j )
	{
		QListWidgetItem * itemProj = listWidgetWave->item ( j );
		QString str = itemProj->data ( Qt::DisplayRole ).toString();
		QStringList lst = str.split ( ' ',QString::SkipEmptyParts );
		QString tmp = lst.at ( 0 );
		tmp += QString ( "  %1 %2 %3" ).arg ( m_t_min[j] ).arg ( m_ipoint[j] ).arg ( m_t_max[j] );
		list << tmp;
	}

	listWidgetGrid->addItems ( list );

	/*
		    sprintf(ss,"%8.3f",CorrComTmin);
		    SetString(CorrComTminId,ss);

		    sprintf(ss,"%8.3f",CorrComTmax);
		    SetString(CorrComTmaxId,ss);

		    sprintf(ss,"%d",TotalNum);
		    SetString(TotalNumId,ss);
	*/


	/*
	 sprintf(ss,"%d",NPoints);
	 SetString(NPointsId,ss);


	 for (i = 0; i < n_item; i++) {

		*strchr(RecDescrList[i],' ') = '\0';

	    }
	    int pLenght = 0;int k;
	    for (i = 0; i < n_item; i++)
	    if (pLenght <  (int)strlen(RecDescrList[i]) )
	    pLenght =    strlen(RecDescrList[i]);



		for (i = 0; i < n_item; i++) {
		for (k = strlen(RecDescrList[i]) ; k < pLenght; k++)  strcat(RecDescrList[i]," ");

		strcat(RecDescrList[i],"   ");



		sprintf(ss,"%7.3f",m_t_min[i]);
	        sprintf(RecDescrList[i],"%s%7s",RecDescrList[i],ss);



		strcat(RecDescrList[i],"  ");



		sprintf(ss,"%2d",m_ipoint[i]);
	        sprintf(RecDescrList[i],"%s%2s",RecDescrList[i],ss);
		strcat(RecDescrList[i],"  ");

		sprintf(ss,"%7.3f",m_t_max[i]);
	        sprintf(RecDescrList[i],"%s%7s",RecDescrList[i],ss);



		}
	*/
///============================================================================================

	delete [] m_w_min;
	delete [] m_w_max;
	delete [] m_t_min;
	delete [] m_t_max;
	delete [] m_dw0;
	delete [] m_ipoint;


///============================================================================================

}



