
#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QString>
#include <QDir>
#include <QRect>

#include <qwt_plot.h>
#include <qwt_plot_printfilter.h>
class QAction;
class QMenu;
class QToolBar;
class QwtPlotCurve;
class QwtPlotPicker;
class QwtPlotPanner;
class Plot;
class MainWindow : public QMainWindow {
    Q_OBJECT

public:
    MainWindow ( QList<float> &XData,QList<float> &YData,QString psFileName );
    ~MainWindow();
    static MainWindow *self;
    QString strippedName ( const QString &fullFileName );
    const QString AbsDir ( const QString fileName ) const;
    QString saveUserTableDirectory;
protected:
    void closeEvent ( QCloseEvent *event );
private slots:
    void about();
    void aboutQwt();
    void zoomin();
    void zoomout();
    void print();
    void pdf();
    void ps();
    void exportData();
private:
    void createActions();
    void createMenus();
    void createToolBars();
    void createStatusBar();
    void readSettings();
    void writeSettings();
    void SavePSFile ( QString fileName );
    QMenu *fileMenu;
    QMenu *helpMenu;
    QToolBar *fileToolBar;
    QAction *exitAct;
    QAction *zoominAct;
    QAction *zoomoutAct;
    QAction *printAct;
    QAction *pdfAct;
    QAction *psAct;
    QAction *exportAct;
    QAction *aboutAct;
    QAction *aboutQwtAct;
    QAction *aboutQtAct;

    Plot* plot;

    QwtPlotPicker *d_picker;
    QwtPlotPanner* d_panner;
    QList<float>* Xdata;
    QList<float>* Ydata;
};
class Plot : public QwtPlot {
public:
    Plot ( QWidget *parent,QList<float> &X,QList<float> &Y );
    QwtPlotCurve *cSin;
    void    ChangeFont ( QFont &f );
    void print ( QPainter *painter,const QRect &plotRect,
                 const QwtPlotPrintFilter &pfilter = QwtPlotPrintFilter() ) const;
};
#endif
