#include <qwt_math.h>
//#include <qwt_scldiv.h>
#include <qwt_plot_canvas.h>
#include <qwt_plot_layout.h>
#include <qwt_paint_buffer.h>
#include <qwt_text_label.h>
#include <qwt_plot_curve.h>
#include <qwt_scale_map.h>
#include <qwt_plot_marker.h>
///////////////////////////////////////
#include <qwt_plot_printfilter.h>
#include <qrect.h>
//#include <qwt_dimap.h>
//#include <qwt_plot_classes.h>
#include <QtGui>
#include <float.h>

#include "mainwindow.h"
#include "bode_plot.h"
#include "dataset.h"
#include "default.h"
BodePlotHide::~BodePlotHide() {

}
BodePlotHide::BodePlotHide (QWidget *parent, CDataSet* data,CDataInfo* dataInfo,int indNormalization) :
        QwtPlot ( parent ),data ( data ),dataInfo ( dataInfo ), indNormalization ( indNormalization ) {
    setMargin ( 5);
//   setTitleFont(parent->font());
    QPalette pl = MainWindow::self->palette();
    QBrush br = pl.window();
    setCanvasBackground(br.color());
/////////////////////////////////////////////////////////////////////

    // QwtTextLabel  * lb = QwtPlot::titleLabel();

    QString titleStr = "";
    if ( data->GetAz().isEmpty () && data->GetAzOrbit().isEmpty () && data->GetNameStation().isEmpty () )
        titleStr = QString ( "%1" ).arg ( data->GetSortPrefix() );
    else {
        titleStr = QString ( "%1  az: %2 " ).arg ( data->GetNameStation() ).arg ( data->GetAzOrbit() );
        if ( ! data->GetAz().isEmpty () && data->GetAzOrbit().isEmpty () && data->GetNameStation().isEmpty () )
            titleStr = QString ( "%1" ).arg ( data->GetSortPrefix() );
    }

    QwtText qtext ( titleStr );

    qtext.setColor (br.color());
    setTitle ( qtext );
/////////////////////////////////////////////////////////////////////////////////////



    // setAutoLegend(TRUE); // We want a legend
    //  enableLegend(false);
///   setLegendPosition(QwtPlot::Right);

    // Set axis titles
    // setAxisTitle(xBottom, "x -->");
    // setAxisTitle(yLeft, "y -->");

//    enableGridX (FALSE);
//    enableGridY (FALSE);
    enableAxis ( QwtPlot::yLeft );
    enableAxis ( QwtPlot::xBottom );



    // Insert new curves
    vector<CDoubleData* > curves =  data->GetDoubleData();

    for ( vector<CDoubleData* >::iterator itt = curves.begin(); itt != curves.end(); ++itt ) {
        QwtPlotCurve * crv1 = new QwtPlotCurve();
        //long  crv1 = insertCurve("Amplitude");
        QPen penCurve ( QColor ( "red" ) );
        crv1->setPen ( penCurve );
        // setCurveXAxis(crv1, QwtPlot::xBottom);
        // setCurveYAxis(crv1, QwtPlot::yLeft);
        int size = ( *itt )->GetSize();

        double *y = new double[size ];

        NormalizationCurve ( y, ( *itt )->GetY(), ( *itt )->GetMaxY(),size );
        crv1->setData ( ( *itt )->GetX(), y, ( *itt )->GetSize() );
        delete [] y;

    }
    // Hide canvas

    setAxisMaxMinor(QwtPlot::xBottom, 0);
    setAxisMaxMajor(QwtPlot::yLeft, 0);
    setAxisMaxMinor(QwtPlot::yLeft, 0);

    setCanvasLineWidth ( 0 );
    QCursor curs = cursor();
    QwtPlotCanvas* cnv = canvas();
    cnv->setCursor ( curs );
    /////////////////////////////// X  axis ////////////////////////////
    setAxisScale ( xBottom,
                   dataInfo->GetMinXCommon(),
                   dataInfo->GetMaxXCommon() );
//////////////////////////////////////////////////////////////////////

    //      Draw yLeft Scale with BackgroundColor Pen ( Invisible Scale)

    CdScaleDraw* cd = new CdScaleDraw (br.color() );

    setAxisScaleDraw ( QwtPlot::xBottom, cd );

    //////////////////////////////////////////// Y Axis ////////////////////////////////////////////////////////
    setAxisScale ( yLeft, 0,1 );

//      Draw yLeft Scale with BackgroundColor Pen ( Invisible Scale)

    CdScaleDraw* cd1 = new CdScaleDraw (br.color() );
    setAxisScaleDraw ( QwtPlot::yLeft, cd1 );



    replot();
}
void BodePlotHide::NormalizationCurve ( double *y,double* Y,double maxY,int n ) {
    double normFactor = 1.0;

    if ( indNormalization == 0 ) normFactor = dataInfo->GetMaxYCommon();
    if ( indNormalization == 1 ) {
        if ( data->GetSortPrefix().toLower() [0] == 'l' ) normFactor = dataInfo->GetMaxYLove();
        if ( data->GetSortPrefix().toLower() [0] == 'r' ) normFactor = dataInfo->GetMaxYRayleigh();
        //QString justfileName =  (*itt)->GetFileName();
        //justfileName =  justfileName.right( justfileName.length() - justfileName.findRev(QDir::convertSeparators("/") ) -1 );
        //  if ( justfileName.lower()[0] == 'l' )

    }
    if ( indNormalization == 2 ) normFactor = maxY;
    for ( int i = 0; i < n; i++ ) y[i]  = Y[i] / normFactor;


}
//-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
BodePlot::~BodePlot() {

}

/////////////////////////////////////////////////////////////////////////////////////////
CdScaleDraw::CdScaleDraw ( QColor col ) : QwtScaleDraw(),color ( col ) {}
void CdScaleDraw::draw(QPainter *p, const QPalette &pal) const {
//QMessageBox::about(0,"","CdScaleDraw");
    p->setPen ( QColor ( color ) );
    // QwtScaleDraw::draw ( p,pal);

}

//void BodePlot::printCanvas(QPainter *p, const QRect & r, const QwtArray< QwtDiMap > &m, const QwtPlotPrintFilter &f)const {

void   BodePlot::printCanvas ( QPainter *p, const QRect &br,const QRect &r, const QwtScaleMap m[axisCnt], const QwtPlotPrintFilter & f) const {
//qWarning("x %d y %d w %d h %d",r.x(),r.y(),r.width(),r.height());
//QMessageBox::about(0,"","printCanvas");
    int a11 = m[0].transform (m[0].s1());
    int a12 = m[0].transform (m[0].s2());
    // qWarning("XXXX a11 %d a12 %d",a11,a12);
//int a21 = m[1].i1();
//int a22 = m[1].i2();
    /// qWarning("XXXX a21 %d a22 %d",a21,a22);
    int a31 = m[2].transform (m[2].s1());
    int a32 = m[2].transform (m[2].s2());
    // qWarning("XXXX a31 %d a32 %d",a31,a32);
//int a41 = m[3].i1();
//int a42 = m[3].i2();
    //qWarning("XXXX a41 %d a42 %d",a41,a42);

//p->drawLine(a31,a11,a31 ,a12);

//qWarning("x %d y %d w %d h %d",r.x(),r.y(),r.width(),r.height());



    QwtPlot::printCanvas ( p, br,r, m, f );
    QPen oldPen1 = p->pen();
    p->setPen ( QPen ( Qt::black ) );
//    p->drawRect(r);
    p->drawLine ( a31,a11,a31 ,a12 );
    p->drawLine ( a32,a11,a32 ,a12 );
    p->drawLine ( a31,a12,a32 ,a12 );
    p->drawLine ( a31,a11,a32 ,a11 );
    p->setPen ( oldPen1 );
}


BodePlot::BodePlot ( QWidget *parent,CDataSet* data, CDataInfo* dataInfo,const QString name,int  normalization,bool prn ) :
        QwtPlot ( parent),data ( data ),dataInfo ( dataInfo ), indNormalization ( normalization ),prn ( prn ) {

//   setTitleFont(parent->font());

/////////////////////////////////////////////////////////////////////
    setMargin ( 5);
    QwtTextLabel  * lb = QwtPlot::titleLabel();

    titleStr = "";
    if ( data->GetAz().isEmpty () && data->GetAzOrbit().isEmpty () && data->GetNameStation().isEmpty () )
        titleStr = QString ( "%1" ).arg ( data->GetSortPrefix() );
    else {
        titleStr = QString ( "%1  az: %2 " ).arg ( data->GetNameStation() ).arg ( data->GetAzOrbit() );
        if ( ! data->GetAz().isEmpty () && data->GetAzOrbit().isEmpty () && data->GetNameStation().isEmpty () )
            titleStr = QString ( "%1" ).arg ( data->GetSortPrefix() );
    }



    enableAxis ( QwtPlot::yLeft );
    enableAxis ( QwtPlot::xBottom );



    // Insert new curves

    vector<CDoubleData* > curves =  data->GetDoubleData();
    // QMessageBox::critical(0 ,"",QString("size %1").arg(curves.size()));
    QStringList indObsCal = momentTensorLocatorFileName().split(' ', QString::SkipEmptyParts);
    QStringList fields = secondMomentLocatorFileName().split(' ', QString::SkipEmptyParts);

    for ( vector<CDoubleData* >::iterator itt = curves.begin(); itt != curves.end(); ++itt ) {

        QwtPlotCurve * crv1 = new QwtPlotCurve();
        QPen penCurve ( QColor ("red" ));
if (MainWindow::self->projectVersion == momentTensorProfileName()) {
        if (( *itt )->GetFileName().toLower().contains(indObsCal.at(0).toLower()) )
            penCurve.setColor("red");
        if (( *itt )->GetFileName().toLower().contains(indObsCal.at(1).toLower()) )
            penCurve.setColor("blue");
        if ( prn ) {
            if (( *itt )->GetFileName().toLower().contains(indObsCal.at(0).toLower()) )
                penCurve.setColor("black");
            if (( *itt )->GetFileName().toLower().contains(indObsCal.at(1).toLower()) )
                penCurve.setColor("gray");

        }
}
if (MainWindow::self->projectVersion == secondMomentProfileName()) {
        if (( *itt )->GetFileName().toLower().contains(fields.at(0).toLower()) )
            penCurve.setColor("red");
        if (( *itt )->GetFileName().toLower().contains(fields.at(1).toLower()) )
            penCurve.setColor("blue");
        if (( *itt )->GetFileName().toLower().contains(fields.at(2).toLower()) )
            penCurve.setColor("green");
        if ( prn ) {
            if (( *itt )->GetFileName().toLower().contains(fields.at(0).toLower()) )
                penCurve.setColor("black");
            if (( *itt )->GetFileName().toLower().contains(fields.at(1).toLower()) )
                penCurve.setColor("darkGray");
            if (( *itt )->GetFileName().toLower().contains(fields.at(2).toLower()) )
                penCurve.setColor("lightGray");

        }
}
        crv1->setPen ( penCurve );

        int size = ( *itt )->GetSize();

        double *y = new double[size ];

        NormalizationCurve ( y, ( *itt )->GetY(), ( *itt )->GetMaxY(),size );
//QMessageBox::warning(0,"",QString("BodePlot 0  size %1 x %2y  %3").arg(( *itt )->GetSize()).arg(( *itt )->GetX()[4]).arg(y[4]));

        crv1->setData ( ( *itt )->GetX(), y, ( *itt )->GetSize() );
        crv1->attach(this);

        delete [] y;
    }
    //setAxisMaxMajor(QwtPlot::xBottom, 10);
    setAxisMaxMinor(QwtPlot::xBottom, 0);
    setAxisMaxMajor(QwtPlot::yLeft, 0);
    setAxisMaxMinor(QwtPlot::yLeft, 0);
    // Canvas
    //   setCanvasLineWidth(1);

    ////////////////// X  axis /////////////////////////////////////////////////////
    setAxisScale ( xBottom,
                   dataInfo->GetMinXCommon(),
                   dataInfo->GetMaxXCommon() );

//////////////////////////////////////////////////////////////////////////////////
    QPalette pl = MainWindow::self->palette();
    QBrush br = pl.window();
    setCanvasBackground(br.color());
    if ( prn ) {
        setCanvasBackground ( QColor ( Qt::white ) );
        setCanvasLineWidth ( 0 );
    }

//       CdScaleDraw* cdXAxis = new CdScaleDraw ( project->GetXAxisFontColor()->GetValue() );
//       setAxisScaleDraw ( QwtPlot::xBottom, cdXAxis );

    ////////////////// Y Axis ////////////////////////////////////////////////
    setAxisScale ( yLeft, 0,1 );


//       CdScaleDraw* cdYAxis = new CdScaleDraw ( project->GetYAxisFontColor()->GetValue() );
//       setAxisScaleDraw ( QwtPlot::xBottom, cdYAxis );

    if ( !prn ) {
        // marker vertical
        mrk1 =  new QwtPlotMarker();
        mrk1->setLineStyle(QwtPlotMarker::VLine);
        mrk1->setValue (dataInfo->GetMinXCommon(),1.0);
        mrk1->setLabelAlignment(Qt::AlignBottom|Qt::AlignTop );
        mrk1->setLinePen (QPen ( Qt::gray, 0, Qt::DotLine ));
        mrk1->attach(this);

        mrk3 =  new QwtPlotMarker();
        mrk3->setLineStyle(QwtPlotMarker::VLine);
        mrk3->setValue (dataInfo->GetMaxXCommon(),1.0 );
        mrk3->setLabelAlignment(Qt::AlignBottom|Qt::AlignTop);
        mrk3->setLinePen (QPen ( Qt::gray, 0, Qt::DotLine ));
        mrk3->attach(this);

        mrk2 =  new QwtPlotMarker();
        mrk2->setLineStyle(QwtPlotMarker::HLine);
        mrk2->setValue (dataInfo->GetMaxXCommon(),1.0);
        mrk2->setLabelAlignment(Qt::AlignBottom|Qt::AlignRight );
        mrk2->setLinePen (QPen ( Qt::gray, 0, Qt::DotLine ));
        mrk2->attach(this);

        mrk4 =  new QwtPlotMarker();
        mrk4->setLineStyle(QwtPlotMarker::HLine);
        mrk4->setValue (dataInfo->GetMaxXCommon(),0.0 );
        mrk4->setLabelAlignment(Qt::AlignBottom|Qt::AlignRight);
        mrk4->setLinePen (QPen ( Qt::gray, 0, Qt::DotLine ));
        mrk4->attach(this);

    }
    
}

void BodePlot::ChangeFont(QFont& f) {
    QFont AxisFont = f;
    AxisFont.setPointSize(AxisFont.pointSize() + 1 );
    AxisFont.setBold(true);
    if ( prn ) AxisFont.setBold(false);
    QwtText qtext("");
    qtext.setFont (AxisFont);
    qtext.setColor (Qt::darkBlue);
    if ( prn ) qtext.setColor (Qt::black);

    //setAxisTitle(QwtPlot::xBottom, qtext);

    qtext.setText(titleStr);
    setTitle(qtext);
    setAxisFont(QwtPlot::xBottom,f);
    setAxisFont(QwtPlot::yLeft,f);
    setAxisFont(QwtPlot::yRight,f);
replot();
}
void BodePlot::NormalizationCurve ( double *y,double* Y,double maxY,int n ) {
    double normFactor = 1.0;

    if ( indNormalization == 0 ) normFactor = dataInfo->GetMaxYCommon();
    if ( indNormalization == 1 ) {
        if ( data->GetSortPrefix().toLower() [0] == 'l' ) normFactor = dataInfo->GetMaxYLove();
        if ( data->GetSortPrefix().toLower() [0] == 'r' ) normFactor = dataInfo->GetMaxYRayleigh();
        //QString justfileName =  (*itt)->GetFileName();
        //justfileName =  justfileName.right( justfileName.length() - justfileName.findRev(QDir::convertSeparators("/") ) -1 );
        //  if ( justfileName.lower()[0] == 'l' )

    }

    if ( indNormalization == 2 ) normFactor = maxY;

    for ( int i = 0; i < n; i++ ) y[i]  = Y[i] / normFactor;


}
