/*
class Bulletin
Bulletin Abstract Data Type

class Mchedr : public Bulletin
 Mchedr Monthly bulletin

National Earthquake Information Center
World Data Center A for Seismology
http://wwwneic.cr.usgs.gov/neis/data_services/ftp_files.html
Data Files Available Through FTP

ISC-on-line bulletin:   http://www.isc.ac.uk/Bulletin/rectang.htm
NEIC monthly mchedr (bulletin) FTP:  ftp://ghtftp.cr.usgs.gov/pub/edr/mchedr/
NEIC monthly ehdf (catalogue) FTP:  ftp://ghtftp.cr.usgs.gov/pub/pde/
NEIC weekly mchedr, ehdf (bulletin, catalogue) FTP:         ftp://ghtftp.cr.usgs.gov/pub/weekly/
4.  Both the PDE weekly EHDF and phase data files from 1999 to the
    current PDE are available in the /weekly directory.  The filenames
    are ehdfYYYY##w.dat and mchedrYYYY##w.dat, where YYYY is the year and ##
    is the corresponding WEEK of the year (e.g. ehdf199641w.dat and
    mchedr199641w.dat).  The QED file, which is updated daily, contains the
    previous 3 weeks of events and is called qedevents.txt.  The phase
    data file for the QED is called mchedrqed.dat.  Check the read_me file
    for more information.
*/

// First motion indications:
//  Quality  != ' ' if not   'e'   'i'
//  Polar    != ' ' if not   'c' === '+'     'd' ===  '-'
// Correct:  Mchedr::GetStationInfo(char *line)
// Correct:  Bulletin::StatisticFMotion(char *bulletinInFileName,FILE *outst)

#ifndef SELDC_BULLETIN__H
#define SELDC_BULLETIN__H
// RETURN CODE ENUMERATION:
enum {
    ErrorRead         = 10,
    ErrorWrite        = 20,
    ErrorFormat       = 21,
    EofFind           = 30,
    EndOfEpicentreRecs     = 40,
    EndOfStationRecs    = 41,

    ErrorCollectEpicentre = 50,
    SkipCollectEpicentre  = 51,

    ErrorCollectStation = 52,
    SkipCollectStation  = 53,
    ReadEpicentreComplete = 60,
    ReadStationComplete = 61


};
const float maxUseDistance = 89.0;
const float minUseDistance = 3.0;

#include <stdio.h>
#include <stdlib.h>

#include <list>
#include <iostream>
#include <string>

using namespace std;

class Station {
public:
    Station(char *str) {
        itsString = new char[strlen(str) + 1];
        strcpy(itsString,str);
        itsStationCode = new char[6];
        itsPhaseCode   = new char[9];
    }
    ~Station() {
        // cout << "Deleting Station.\n";
        //cout << itsString.GetString() << "\n";
        delete itsString;
        delete itsStationCode;
        delete itsPhaseCode;
    }

    void Show() {
        cout << "This Station is " << itsString << "\n";
    }
    char* GetString()  const {
        return itsString;
    }
    char* &GetStationCode      ()  {
        return itsStationCode;
    }
    char* &GetPhaseCode          ()  {
        return itsPhaseCode;
    }
    float  &GetDistance        ()  {
        return itsDistance;
    }
    float  &GetAzimuthToStation()  {
        return itsAzimuthToStation;
    }
    float  &GetIscResidual     ()  {
        return itsIscResidual;
    }
    char   &GetQuality         ()  {
        return itsQuality;
    }
    char   &GetPolar           ()  {
        return itsPolar;
    }

private:
    char* itsString;
    char* itsStationCode;
    float  itsDistance;
    float  itsAzimuthToStation;
    char   itsQuality;
    char* itsPhaseCode;
    float  itsIscResidual;
    char   itsPolar;
};
class Epicentre {
public:
    Epicentre(char *str) {
        itsString = new char[strlen(str) + 1];
        strcpy(itsString,str);
        itsMbCode = new char[5];
        itsMsCode = new char[5];
        itsAgencyCode = new char[12];


    }
    ~Epicentre() {
        //cout << "Deleting Epicentre object.\n";
        //cout << GetString() << "\n";
        delete itsString;
        delete itsMbCode;
        delete itsMsCode;
        delete itsAgencyCode;

    }
    int    &GetYear  ()           {
        return itsYear;
    }
    int    &GetMonth ()           {
        return itsMonth;
    }
    int    &GetDay   ()           {
        return itsDay;
    }
    int    &GetHour  ()           {
        return itsHour;
    }
    int    &GetMinute()           {
        return itsMinute;
    }
    float  &GetSec   ()           {
        return itsSec;
    }
    float  &GetLat   ()           {
        return itsLat;
    }
    float  &GetLon   ()           {
        return itsLon;
    }
    float  &GetDepth ()           {
        return itsDepth;
    }
    float  &GetMagMb ()           {
        return itsMagMb;
    }
    float  &GetMagMs ()           {
        return itsMagMs;
    }
    char* &GetMbCode()           {
        return itsMbCode;
    }
    char* &GetMsCode()           {
        return itsMsCode;
    }
    char* &GetAgencyCode()           {
        return itsAgencyCode;
    }
    int &GetNumberObservations()  {
        return itsNumberObservations;
    }
    char* GetString ()  const    {
        return  itsString;
    }
    void Show() {
        cout << itsString << endl;
    }
private:
    char* itsString;

    int itsYear;
    int itsMonth;
    int itsDay;
    int itsHour;
    int itsMinute;
    int itsNumberObservations;


    float itsSec;
    float itsLat;
    float itsLon;
    float itsDepth;
    float itsMagMb;
    float itsMagMs;

    char *itsMbCode;
    char *itsMsCode;
    char *itsAgencyCode;
};


// Abstract Data Type
class Bulletin {
private:
    int itsNumberFirstMotion;

protected:
    char *itsOutEpicentre;
    list<Station *> itsOutStationList;
    Station * itsPtrOutStation;
    list<Station *>::iterator itsItrOutStationList;
    Epicentre* itsPtrOutEpicentre;
public:
    Bulletin();
    virtual ~Bulletin();
    Epicentre* GetEpicenter(){ return itsPtrOutEpicentre;}
    virtual int ReadEpicentre(FILE *inbul) = 0;
    virtual int GetEpicenterInfo(char *lineHY,char *lineE) = 0;
    virtual int GetStationInfo(char *line) = 0;
    int ReadStation(FILE *inbul);
    int WriteFirstMotions(FILE *outbul,float depth,float pWaveVelVal);
    int StatisticFMotion(FILE *outst);
    int indMaxDistance;
    int indMinDistance;
    int itsNumberStation;


};


class Mchedr : public Bulletin {
public:

    Mchedr() {}
    ~Mchedr() {
//      cout << "Deleting Mchedr object.\n";

    }
    virtual int ReadEpicentre(FILE *inbul);
    virtual int GetEpicenterInfo(char *lineHY,char *lineE);
    virtual int GetStationInfo(char *line);
private:
};

class Cors : public Bulletin {
public:

    Cors() {}
    ~Cors() {
//      cout << "Deleting Cors object.\n";

    }
    virtual int ReadEpicentre(FILE *inbul) = 0;
    virtual int GetEpicenterInfo(char *lineHY,char *lineE) = 0;
    virtual int GetStationInfo(char *line) = 0;
private:
};

class StaticPhase {
private:
    int  itsNumberPhase;
    char* itsPhaseName;
    char itsPhaseCode;
public:
    StaticPhase(char* PhaseName,char PhaseCode ):itsNumberPhase(1) {
        itsPhaseName = new char[strlen(PhaseName)+1];
        strcpy(itsPhaseName,PhaseName);
        itsPhaseCode = PhaseCode;
    }
    virtual ~StaticPhase() {
        delete [] itsPhaseName;
    }
    char* &GetPhaseName()           {
        return itsPhaseName;
    }
    char  &GetPhaseCode()           {
        return itsPhaseCode;
    }
    int   &GetNumberPhase()         {
        return itsNumberPhase;
    }
};

#endif // __BULLETIN__H
