//
// C++ Implementation: polarities
//
// Description:
//
//
// Author: fmt <fmt@localhost.localdomain>, (C) 2008
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include <QtGui>
#include <QMessageBox>
#include <QProcess>
#include "default.h"
#include "document.h"
#include "mainwindow.h"
#include "projectwizard.h"
#include "polarities.h"
#include "editline.h"
#include "bulletin.h"
#include "appkonsole.h"
extern "C" {
    extern int vploc_(float *dep,float *vpsource,char* modelname,long *length);
    extern int polsmu_(float *an,char* namein,long *lennamein,char* nameout,long *lennameout,char* projori,long *lenprojori,char* polproj,long *lenpolproj);
}
PolarImpl::PolarImpl(QWidget *parent,MDocument* doc,ProWizard::DocumentUsing documentUsing)
        : QDialog(parent),doc(doc),documentUsing(documentUsing) {

    bul = 0;
    setFont( parent->font() );
    setWindowIcon(QIcon(":/images/icon.png"));
    setupUi(this);
    InitDialog();
    SetDocumentToDialog();

    connect( toolButtonSmoothing, SIGNAL( clicked() ), this, SLOT( buttonSmoothing() ) );
    connect( checkBoxSup, SIGNAL( stateChanged ( int ) ), this, SLOT( stateChangedSup(int ) ) );
    connect( checkBoxBul, SIGNAL( stateChanged ( int ) ), this, SLOT( stateChangedBul(int ) ) );
    connect( checkBoxSmoothing, SIGNAL( stateChanged ( int ) ), this, SLOT( stateChangedSmoothing(int ) ) );

    connect( toolButton, SIGNAL( clicked() ), this, SLOT( fileBrowse() ) );
    connect( toolButtonBul, SIGNAL( clicked() ), this, SLOT( fileBrowseBul() ) );
}
bool PolarImpl::CheckDialogParameeters() {
    return true;
}
void PolarImpl::FinishWizard() {
    GetDocumentFromDialog();
    foreach (PropertyValue* val, values) val->SetValueToResetvalue();
}
void PolarImpl::InitDialog() {

    values.append(doc->GetIsPolaritySuplementary());
    values.append(doc->GetSuplementaryFileName());
    values.append(doc->GetIsPolarityBulletin());
    values.append(doc->GetBulletinFileName());
    values.append(doc->GetPolarityDepth());
    values.append(doc->GetPolarityVpsource());
    values.append(doc->GetPolaritySmoothing());
    values.append(doc->GetIsPolaritySmoothing());

    propDocLine.append(new LineDocProperty(lineEditDepth,doc->GetPolarityDepth()));
    propDocLine.append(new LineDocProperty(lineEditVel,doc->GetPolarityVpsource()));

    doubleSpinBoxSmooth->setDecimals (doc->GetPolaritySmoothing()->GetDecimal());
    doubleSpinBoxSmooth->setMinimum (doc->GetPolaritySmoothing()->GetMin());
    doubleSpinBoxSmooth->setMaximum (doc->GetPolaritySmoothing()->GetMax());
    doubleSpinBoxSmooth->setSingleStep (1.0);
    doubleSpinBoxSmooth->setSuffix(" deg.");
    doubleSpinBoxSmooth->setValue(doc->GetPolaritySmoothing()->GetValue().toFloat());


    if (doc->GetIsPolaritySuplementary()->GetValue() == "False") {
        SuplementaryPolarFileName = "";
        labelName->setText("");
        checkBoxSup->setChecked (false);
        checkBoxSup->setText("Disable");
        toolButton->setDisabled ( true );
    } else {
        if (!GetSuplementaryPolar(doc->GetSuplementaryFileName()->GetValue())) {
            checkBoxSup->setChecked (false);
            checkBoxSup->setText(" Disable");
            labelName->setText("");
            SuplementaryPolarFileName = "";
            toolButton->setDisabled ( true );
        } else {
            checkBoxSup->setChecked (true);
            SuplementaryPolarFileName = doc->GetSuplementaryFileName()->GetValue();
            labelName->setText(MainWindow::self->strippedName(doc->GetSuplementaryFileName()->GetValue()));
            toolButton->setDisabled ( false );
            checkBoxSup->setText(" Enable");
        }
    }

    if (doc->GetIsPolarityBulletin()->GetValue() == "False") {
        BulletinFileName = "";
        labelNameBul->setText("");
        checkBoxBul->setChecked (false);
        toolButtonBul->setDisabled ( true );
        checkBoxBul->setText("Disable");
    } else {

        if (!GetBulletinPolar(doc->GetBulletinFileName()->GetValue())) {
            checkBoxBul->setChecked (false);
            labelNameBul->setText("");
            BulletinFileName = "";
            toolButtonBul->setDisabled ( true );
            checkBoxBul->setText("Disable");
        } else {
            checkBoxBul->setChecked (true);
            checkBoxBul->setText(" Enable");
            labelNameBul->setText(MainWindow::self->strippedName(doc->GetBulletinFileName()->GetValue()));
            BulletinFileName = doc->GetBulletinFileName()->GetValue();
            toolButtonBul->setDisabled ( false );
        }

    }
    if (doc->GetIsPolaritySmoothing()->GetValue() == "False") {

        checkBoxSmoothing->setChecked (false);
        checkBoxSmoothing->setText("Disable");
        toolButtonSmoothing->setDisabled ( true );
    } else {
        checkBoxSmoothing->setChecked (true);
        checkBoxSmoothing->setText(" Enable");
        toolButtonSmoothing->setDisabled ( false );
    }



}
bool  PolarImpl::GetSuplementaryPolar(QString fileName) {

    labelName->setText(MainWindow::self->strippedName(fileName));

    QFile file(fileName);
    if (!file.open(QFile::ReadOnly | QFile::Text)) {
        QMessageBox::warning(0,mainWindowCaption(),
                             QString("Cannot read file %1:\n%2.")
                             .arg(fileName)
                             .arg(file.errorString()));
        return false;
    }
    QTextStream in(&file);
    QString line;
    do {
        line = in.readLine();

        polarityData << line;
    } while (!line.isNull());

    file.close();
    return true;
}
bool PolarImpl::GetBulletinPolar(QString fileName) {

    if (!ReadBuletin(fileName)) return false;

    labelNameBul->setText(MainWindow::self->strippedName(fileName));
    if (!GetVpSource()) return false;

    return true;
}
bool PolarImpl::GetVpSource() {

    Epicentre* ep = bul->GetEpicenter();
    float  depth = ep->GetDepth ();
    float vpsource;

    QString tblname = QString("%1").arg(rootPath());
    tblname.append("/KennetTables/ak135");
    char* modelname = new char[256];
    strcpy(modelname,tblname.toAscii().data());
    long len = strlen(modelname);
    vploc_(&depth,&vpsource,modelname,&len);
    lineEditVel->setText(QString("%1").arg(vpsource));
    //doubleSpinBoxSmooth->setValue (0.0);
    return true;
}
bool PolarImpl::ReadBuletin(QString fileName) {
    FILE *inbul;
    if ( (inbul  = fopen(fileName.toAscii(), "r" )) == NULL ) {
        QMessageBox::warning(0,mainWindowCaption(),
                             QString("Cannot read file %1")
                             .arg(fileName));
        return false;
    }

    if (bul != 0)   delete bul;

    bul = new Mchedr;
    int errReadingEpicentreCode,errReadingStationCode;

    if ( (errReadingEpicentreCode = bul->ReadEpicentre(inbul) )) {
        switch (errReadingEpicentreCode) {
        case ErrorRead: {
            QMessageBox::warning(0,mainWindowCaption(),
                                 QString("Fatal Error:  Error reading  Epicentre"));

            return false;
        }

        case ErrorFormat: {
            QMessageBox::warning(0,mainWindowCaption(),
                                 QString("Fatal Error: Information of Epicentre not complete"));

            return false;
        }

        case EofFind: {
            QMessageBox::warning(0,mainWindowCaption(),
                                 QString("Fatal Error: EndOfFile found when Epicentre read."));

            return false;
        }
        case ReadEpicentreComplete: {
            //    QMessageBox::warning(0,mainWindowCaption(),
            //                        QString("Info: Epicentre was read"));

            break;
        }


        }


    }


    // Mchedr* pDynamic = dynamic_cast<Mchedr*>(bul);
//if (pDynamic != 0) { }// p reffered an object of type Mchedr

    if ( errReadingEpicentreCode == ReadEpicentreComplete ) {

        // if (pDynamic != 0)
        //    QMessageBox::warning(0,mainWindowCaption(),
        //                        QString("Info: Epicentre Information is read."));

        if ( (errReadingStationCode = bul->ReadStation(inbul) )) {
            switch (errReadingStationCode) {
            case ErrorRead: {
                QMessageBox::warning(0,mainWindowCaption(),
                                     QString("Fatal Error:  Error reading  Station."));

                return false;
            }

            case ErrorFormat: {
                QMessageBox::warning(0,mainWindowCaption(),
                                     QString("Fatal Error: Information of Station not complete."));

                return false;
            }

            case EofFind: {
                QMessageBox::warning(0,mainWindowCaption(),
                                     QString("Error: EndOfFile found when Station read.\nNothing First Motion Information."));

                return false;
            }
            case ReadStationComplete: {
                //       QMessageBox::warning(0,mainWindowCaption(),
                //                          QString("Info: Stations Information was read."));

                break;
            }


            }


        }



        if ( errReadingStationCode == ReadStationComplete ) {
        }



    } //if ( errReadingEpicentreCode == ReadEpicentreComplete )
    fclose(inbul);
    Epicentre* ep = bul->GetEpicenter();
    float  depth = ep->GetDepth ();
    lineEditDepth->setText(QString("%1").arg(depth));
    return true;
}
void  PolarImpl::buttonSmoothing() {
    if (!checkBoxBul->isChecked ()) return;
    // MainWindow::self->removeWorkingMaterials(MainWindow::self->tempDirectoryPath,false);



    if (!WriteFirstMotions(true) ) return;
    float depth = lineEditDepth->text ().toFloat();
    float vpsource = lineEditVel->text ().toFloat();
    float an = (float)doubleSpinBoxSmooth->value ();
    if (!(fabs(an) > 1.e-5)) return;

    QString script = rootPath()+"/bin/fpol.com";

    QStringList arguments;
    arguments << MainWindow::self->tempDirectoryPath + "/polarity.ps" << MainWindow::self->tempDirectoryPath + "/proj_ori" << MainWindow::self->tempDirectoryPath + "/pol_proj";
    QApplication::setOverrideCursor(Qt::WaitCursor);
    MainWindow::self->GetKonsole()->PolaritySmoothingProcess(script,MainWindow::self->tempDirectoryPath,arguments,depth,vpsource,an);
    QApplication::restoreOverrideCursor();

}
bool PolarImpl::WriteFirstMotions(bool smoothing) {
    QString jobProtocolFileName = MainWindow::self->tempDirectoryPath + "/ProtocolBulletin.txt";
    bulletinOutFileName = MainWindow::self->tempDirectoryPath + "/OutPutOriginalBulletin.txt";
    bulletinOutSmuFileName = MainWindow::self->tempDirectoryPath + "/OutPutSmuthingBulletin.txt";
    FILE *outbul;
    FILE *prot;
    if ( (prot  = fopen(jobProtocolFileName.toAscii(), "w" )) == NULL ) {
        QMessageBox::warning(0,mainWindowCaption(),
                             QString("Fatal Error: The protocol file '%1' was not opened:").arg(jobProtocolFileName));
        return false;
    }
    if ( (outbul = fopen( bulletinOutFileName.toAscii(), "w" )) == NULL ) {
        QMessageBox::warning(0,mainWindowCaption(),
                             QString("Fatal Error: The protocol file '%1' was not opened:").arg(bulletinOutFileName));
        return false;
    }

    float depth = lineEditDepth->text().toFloat();
    float pWaveVelocityValue = lineEditVel->text().toFloat();
    bul->WriteFirstMotions(outbul,depth,pWaveVelocityValue);
    bul->StatisticFMotion(prot);
    fclose(outbul);
    fclose(prot);

    float an = (float)doubleSpinBoxSmooth->value ();
    if (smoothing) {
        if (!(fabs(an) > 1.e-5) ) {
            QMessageBox::warning(0,mainWindowCaption(),
                                 QString("Angle for smoothing is equal zero. Nothing to smooth!"));
        } else {


            char* namein = new char[256];
            strcpy(namein,bulletinOutFileName.toAscii().data());
            long lennamein = strlen(namein);

            char* nameout = new char[256];
            strcpy(nameout,bulletinOutSmuFileName.toAscii().data());
            long lennameout = strlen(nameout);

            QString Strprojori = MainWindow::self->tempDirectoryPath + "/proj_ori";
            char* projori = new char[256];
            strcpy(projori,Strprojori.toAscii().data());
            long lenprojori = strlen(projori);


            QString Strpolproj = MainWindow::self->tempDirectoryPath + "/pol_proj";
            char* polproj = new char[256];
            strcpy(polproj,Strpolproj.toAscii().data());
            long lenpolproj = strlen(polproj);


            polsmu_(&an,namein,&lennamein,nameout,&lennameout,projori,&lenprojori,polproj,&lenpolproj);

            bulletinOutFileName = bulletinOutSmuFileName;
        }

    }
    bulletinData.clear();
    QFile file(bulletinOutFileName);
    if (!file.open(QFile::ReadOnly | QFile::Text)) {
        QMessageBox::warning(0,mainWindowCaption(),
                             QString("Cannot read file %1:\n%2.")
                             .arg(bulletinOutFileName)
                             .arg(file.errorString()));
        return false;
    }

    QTextStream in(&file);
    QString line;
    do {
        line = in.readLine();
        QStringList lst = line.split(' ', QString::SkipEmptyParts);
        if (lst.size() < 3) break;
        bulletinData << line;
    } while (!line.isNull());
    file.close();

    return true;

}
void  PolarImpl::stateChangedSup(int state) {
    if (state == Qt::Checked) {
        checkBoxSup->setText(" Enable");
        toolButton->setDisabled ( false );
    }
    if (state == Qt::Unchecked) {
        checkBoxSup->setText("Disable");
        toolButton->setDisabled ( true );
    }
}
void  PolarImpl::stateChangedBul(int state) {
    if (state == Qt::Checked) {
        checkBoxBul->setText(" Enable");
        toolButtonBul->setDisabled ( false );
    }
    if (state == Qt::Unchecked) {
        checkBoxBul->setText("Disable");
        toolButtonBul->setDisabled ( true );
    }

}

void  PolarImpl::stateChangedSmoothing(int state) {
    if (state == Qt::Checked) {
        checkBoxSmoothing->setText(" Enable");
        toolButtonSmoothing->setDisabled ( false );
    }
    if (state == Qt::Unchecked) {
        checkBoxSmoothing->setText("Disable");
        toolButtonSmoothing->setDisabled ( true );
    }

}

void  PolarImpl::fileBrowseBul() {

    QString fileName =
        QFileDialog::getOpenFileName(this,
                                     "Open an existing bulletin file",
                                     MainWindow::self->openUserPolarityDirectory, buletinFilters());

    if (fileName.isEmpty()) return;

    MainWindow::self->openUserPolarityDirectory = QFileInfo(fileName).absolutePath ();
    if (!GetBulletinPolar(fileName)) return;

    BulletinFileName = fileName;
    if (!WriteFirstMotions(false) ) return;
}
void  PolarImpl::fileBrowse() {

    polarityData.clear();
    QString fileName =
        QFileDialog::getOpenFileName(this,
                                     "Open an existing polarity file",
                                     MainWindow::self->openUserPolarityDirectory, polarityFilters());

    if (fileName.isEmpty()) return;
    MainWindow::self->openUserPolarityDirectory = QFileInfo(fileName).absolutePath ();


    if (!GetSuplementaryPolar(fileName)) return;
    SuplementaryPolarFileName = fileName;
    /*
        line = "Polar data \n";
        for (int i = 0; i < polarityData.size(); ++i) {
            line += polarityData.at(i) + "\n";
        }
        QMessageBox::warning(0,mainWindowCaption(),line);
    */
}
///========================================================================================
void PolarImpl::SetDocumentToDialog() {
    foreach (LineDocProperty *docprop, propDocLine) docprop->SetValueFromDoc();
}
void PolarImpl::GetDocumentFromDialog() {
    doc->GetPolarityDepth()->SetValue(QString("%1").arg(lineEditDepth->text ()) );
    doc->GetPolarityVpsource()->SetValue(QString("%1").arg(lineEditVel->text ()) );
    doc->GetPolaritySmoothing()->SetValue(QString("%1").arg(doubleSpinBoxSmooth->value ()) );

    if (checkBoxSup->checkState() == Qt::Checked) {
        doc->GetIsPolaritySuplementary()->SetValue("True");
        doc->GetSuplementaryFileName()->SetValue(SuplementaryPolarFileName);
    } else {
        doc->GetIsPolaritySuplementary()->SetValue("False");
        doc->GetSuplementaryFileName()->SetValue(QString());
    }


    if (checkBoxBul->checkState() == Qt::Checked) {
        doc->GetIsPolarityBulletin()->SetValue("True");
        doc->GetBulletinFileName()->SetValue(BulletinFileName);
    } else {
        doc->GetIsPolarityBulletin()->SetValue("False");
        doc->GetBulletinFileName()->SetValue(QString());
    }

    if (checkBoxSmoothing->checkState() == Qt::Checked)
        doc->GetIsPolaritySmoothing()->SetValue("True");
    else
        doc->GetIsPolaritySmoothing()->SetValue("False");

}
