#include <QtGui>
#include "default.h"
#include "mainwindow.h"
#include "appkonsole.h"
#include "viewresult.h"
#include "projectwizard.h"
#include "info.h"
#include "sei.h"
#include "document.h"
extern "C" {
    ;
    extern int formeos_(float *alat,float *along);
    extern int smac2mod_();
}
MainWindow *MainWindow::self = 0;
MainWindow::~MainWindow() {

    if (doc != 0) delete doc;
    if (viewresult != 0) delete viewresult;
    if (infoproject != 0) delete infoproject;
    if (appkonsole != 0) delete appkonsole;
}
MainWindow::MainWindow() {
    self = this;
    setWindowTitle ( mainWindowCaption());
    setWindowIcon (QIcon(":/images/icon.png"));
    projectFileFilters = tr("Project files (*%1);;All Files (*)").
                         arg(projectFileFilterLocator());

    viewresult = 0;
    infoproject = 0;

    appkonsole    = new AppKonsole(this);
    doc = 0;
    createActions();
    createMenus();
    createToolBars();
    createStatusBar();
    curFile = "";
    setCurrentProjectFile("");
    modifiedProject = false;
    readSettings();
    MakeTempDirectory();
    FileActsShowHide(false);
}
void MainWindow::initUI() {

    splitterVert = new QSplitter(this);
    splitterVert->setOrientation(Qt::Vertical);
    splitter = new QSplitter(splitterVert);

    viewresult = new ViewResult(this);

    infoproject = new InfoProject(this);

    splitter->addWidget(viewresult);
    splitter->addWidget(infoproject);
    splitterVert->addWidget(appkonsole);

    setCentralWidget(splitterVert);
    QSettings settings("mitp.ru", "MomTens Main");
    QByteArray splitterData = settings.value("splitter").toByteArray();
    splitter->restoreState(splitterData);
    splitterData = settings.value("splitterVert").toByteArray();
    splitterVert->restoreState(splitterData);


}
void  MainWindow::MakeTempDirectory() {
    QDate date = QDate::currentDate ();
    QTime time = QTime::currentTime ();
    QDateTime datetime ( date, time ) ;
    uint currentTime = datetime.toTime_t ();
    homeDirectoryPath = temporarySystemDirectoryPath();
    homeDirectoryPath =  QDir::toNativeSeparators ( homeDirectoryPath );
    QDir d ( temporarySystemDirectoryPath() );
    homeDirectory = d;
    tempDirectoryPath = temporarySystemDirectoryPath() +
                        QString ( "/%1%2" ).arg ( temporaryDirectoryPrifix() ).arg ( currentTime );
    tempDirectoryPath =  QDir::toNativeSeparators ( tempDirectoryPath );
    d.mkdir ( tempDirectoryPath );
    QDir dd ( tempDirectoryPath );
    tempDirectory = dd;
//QMessageBox::critical(0,"",QString("temp %1").arg(tempDirectoryPath));
}
void MainWindow::removeWorkingMaterials(QString & temporaryDirectory,bool removeTempDirectory) {
    if ( temporaryDirectory.isNull() ) return;
    QDir tempDir ( temporaryDirectory );
    QFileInfoList info = tempDir.entryInfoList ( QDir::Dirs | QDir::Files | QDir::NoDotAndDotDot | QDir::Hidden,QDir::Name );

//   QMessageBox::critical(0,"", QString("temporaryDirectory %1").arg(temporaryDirectory));

    for ( int i = 0; i < info.size(); ++i ) {
        QFileInfo  infoTemp = info.at ( i );
        QString itemName = infoTemp.filePath();
        if ( infoTemp.isDir ()) {
            QDir tempCurDir ( itemName );
            QFileInfoList infolist = tempCurDir.entryInfoList (  QDir::Dirs | QDir::Files | QDir::NoDotAndDotDot | QDir::Hidden,QDir::Name );
            for ( int j = 0; j < infolist.size(); ++j ) {
                QFileInfo  infoCurTemp = infolist.at ( j );
                QString fileName = infoCurTemp.filePath();
                if ( infoCurTemp.isFile ()) {
                    if (!tempCurDir.remove ( fileName )) QMessageBox::critical(0,"",
                                QString("Can't remove file %1").arg(fileName));
                }
            }
        }

        if ( infoTemp.isDir ())
            if (!tempDir.rmdir ( itemName )) QMessageBox::critical(0,"",
                        QString("Can't remove directory %1").arg(itemName));

        if ( infoTemp.isFile ())
            if (!tempDir.remove ( itemName )) QMessageBox::critical(0,"",
                        QString("Can't remove file %1").arg(itemName));
    }
    if (removeTempDirectory) {
        QDir tempSys(temporarySystemDirectoryPath());
        if (!tempSys.rmdir ( temporaryDirectory )) QMessageBox::critical(0,"",
                    QString("Can't remove directory %1").arg(temporaryDirectory));
    }
}
void MainWindow::FileActsShowHide(bool b) {
    foreach (QAction *action, showHideFileActs)
    if (b) action->setEnabled ( TRUE );
    else action->setEnabled ( FALSE );
}
void MainWindow::closeEvent(QCloseEvent *event) {

    if ( maybeSave() ) {
        QApplication::setOverrideCursor(Qt::WaitCursor);
        removeWorkingMaterials(tempDirectoryPath);
        writeSettings();
        QApplication::restoreOverrideCursor();
        event->accept();
    } else {
        event->ignore();
    }

}
void MainWindow::about() {
    QMessageBox::about(this, tr("About Application"),
                       tr("The <b>Application</b> example demonstrates how to "
                          "write modern GUI applications using Qt, with a menu bar, "
                          "toolbars, and a status bar."));
}


void MainWindow::createActions() {
    exitAct = new QAction(QIcon(":/images/exit.png"), tr("E&xit"), this);
    exitAct->setShortcut(tr("Ctrl+Q"));
    exitAct->setStatusTip(tr("Exit the application"));
    connect(exitAct, SIGNAL(triggered()), this, SLOT(close()));

    zoominAct = new QAction(QIcon(":/images/viewmag+.png"), tr("Zoom &In"),this);
    zoominAct->setShortcut(tr("Ctrl++"));
    zoominAct->setStatusTip(tr("Zoom in"));
    connect(zoominAct, SIGNAL(triggered()),this, SLOT(zoomin()));

    zoomoutAct = new QAction(QIcon(":/images/viewmag-.png"), tr("Zoom &Out"),this);
    zoomoutAct->setShortcut(tr("Ctrl+-"));
    zoomoutAct->setStatusTip(tr("Zoom out"));
    connect(zoomoutAct, SIGNAL(triggered()),this, SLOT(zoomout()));

    aboutAct = new QAction(tr("&About"), this);
    aboutAct->setStatusTip(tr("Show the application's About box"));
    connect(aboutAct, SIGNAL(triggered()), this, SLOT(about()));

    aboutQtAct = new QAction(tr("About &Qt"), this);
    aboutQtAct->setStatusTip(tr("Show the Qt library's About box"));
    connect(aboutQtAct, SIGNAL(triggered()), qApp, SLOT(aboutQt()));

    editAct = new QAction(QIcon(":/images/project_edit.png"), tr("&Edit"), this);
    editAct->setShortcut(tr("Shift+E"));
    editAct->setStatusTip(tr("Edit the project"));
    connect(editAct, SIGNAL(triggered()), this, SLOT(editProject()));
    showHideFileActs.append(editAct);

    newAct = new QAction(QIcon(":/images/project_new.png"), tr("&New"), this);
    newAct->setShortcut(tr("Ctrl+N"));
    newAct->setStatusTip(tr("Create a new project"));
    connect(newAct, SIGNAL(triggered()), this, SLOT(newProject()));

    openAct = new QAction(QIcon(":/images/project_open.png"), tr("&Open"), this);
    openAct->setShortcut(tr("Ctrl+O"));
    openAct->setStatusTip(tr("Open an existing project location"));
    connect(openAct, SIGNAL(triggered()), this, SLOT(openProject()));

    saveAct = new QAction(QIcon(":/images/project_save.png"), tr("&Save"), this);
    saveAct->setShortcut(tr("Ctrl+S"));
    saveAct->setStatusTip(tr("Save the project location to disk"));
    connect(saveAct, SIGNAL(triggered()), this, SLOT(saveProject()));
    showHideFileActs.append(saveAct);

    saveAsAct = new QAction(QIcon(":/images/project_saveas.png"), tr("Save &As..."), this);
    saveAsAct->setShortcut(tr("Ctrl+A"));
    saveAsAct->setStatusTip(tr("Save the project location under a new name"));
    connect(saveAsAct, SIGNAL(triggered()), this, SLOT(saveAsProject()));
    showHideFileActs.append(saveAsAct);

    executeAct = new QAction(QIcon(":/images/execute.png"), tr("&Execute"), this);
    executeAct->setShortcut(tr("Ctrl+E"));
    executeAct->setStatusTip(tr("Execute the project"));
    connect(executeAct, SIGNAL(triggered()), this, SLOT(execute()));
    showHideFileActs.append(executeAct);


    for (int i = 0; i < MaxRecentFiles; ++i) {
        recentProjectFileActs[i] = new QAction(this);
        recentProjectFileActs[i]->setVisible(false);
        connect(recentProjectFileActs[i], SIGNAL(triggered()),
                this, SLOT(openRecentProjectLocation()));
    }

}

void MainWindow::createMenus() {
    fileMenu = menuBar()->addMenu(tr("&Location"));
    fileMenu->addAction(newAct);
    fileMenu->addAction(openAct);
    fileMenu->addAction(editAct);
    fileMenu->addAction(executeAct);
    fileMenu->addAction(saveAct);
    fileMenu->addAction(saveAsAct);

    separatorProjectAct = fileMenu->addSeparator();
    for (int i = 0; i < MaxRecentFiles; ++i)
        fileMenu->addAction(recentProjectFileActs[i]);
    fileMenu->addSeparator();
    fileMenu->addAction(exitAct);

    updateRecentProjectFileActions();

    menuBar()->addSeparator();

    viewMenu = menuBar()->addMenu(tr("&View"));

    viewMenu->addAction(zoominAct);
    viewMenu->addAction(zoomoutAct);

    menuBar()->addSeparator();

    helpMenu = menuBar()->addMenu(tr("&Help"));
    helpMenu->addAction(aboutAct);
    helpMenu->addAction(aboutQtAct);
}

void MainWindow::createToolBars() {
    fileToolBar = addToolBar(tr("File"));
    fileToolBar->addAction(newAct);
    fileToolBar->addAction(openAct);
    fileToolBar->addAction(editAct);
    fileToolBar->addAction(executeAct);
    fileToolBar->addAction(saveAct);
    fileToolBar->addAction(saveAsAct);
    fileToolBar->addSeparator ();

    fileToolBar->addAction(zoominAct);
    fileToolBar->addAction(zoomoutAct);
    fileToolBar->addSeparator ();

    fileToolBar->addAction(exitAct);

}

void MainWindow::createStatusBar() {
    statusBar()->showMessage(tr("Ready"));
}
void MainWindow::zoomin() {
    QFont f( font() );
    f.setPointSize( f.pointSize() + 1 );
    setFont( f );
}
void MainWindow::zoomout() {
    QFont f(font() );
    f.setPointSize( f.pointSize() - 1 );
    setFont( f );
}
void MainWindow::readSettings() {
    QSettings settings("mitp.ru", "MomTens Main");
    QPoint pos = settings.value("pos", QPoint(100, 100)).toPoint();
    QSize size = settings.value("size", QSize(400, 600)).toSize();
    resize(size);
    move(pos);
    int fontPointSize = settings.value("FontPointSize", 10).toInt();
    QFont f(font());
    f.setPointSize(fontPointSize );
    setFont( f );
    projectWizardPos  = settings.value("projectWizardPos",QPoint(100, 100)).toPoint();
    projectWizardSize = settings.value("projectWizardSize",QSize(500, 400)).toSize();

    openUserProjectDirectory    = settings.value("openUserProjectDirectory",userHomePath()).toString();
    saveUserProjectDirectory    = settings.value("saveUserProjectDirectory",userHomePath()).toString();
    openInputDataDirectory      = settings.value("openInputDataDirectory",userHomePath()).toString();
    openModelDirectory = settings.value("openModelDirectory",userHomePath()).toString();
    openUserPolarityDirectory = settings.value("openUserPolarityDirectory",userHomePath()).toString();

}

void MainWindow::writeSettings() {
    QSettings settings("mitp.ru","MomTens Main");
    settings.setValue("pos", pos());
    settings.setValue("size", size());
    settings.setValue("FontPointSize", font().pointSize());

    settings.setValue  ("projectWizardPos",projectWizardPos);
    settings.setValue  ("projectWizardSize",projectWizardSize);
    settings.setValue("openUserProjectDirectory",openUserProjectDirectory);
    settings.setValue("saveUserProjectDirectory",saveUserProjectDirectory);
    settings.setValue("openInputDataDirectory",openInputDataDirectory);
    settings.setValue("openModelDirectory",openModelDirectory);
    settings.setValue("openUserPolarityDirectory",openUserPolarityDirectory);
    if (centralWidget () != 0) {
        settings.setValue("splitter",splitter->saveState());
        settings.setValue("splitterVert",splitterVert->saveState());
    }
}

QString MainWindow::strippedName(const QString &fullFileName) {
    return QFileInfo(fullFileName).fileName();
}
const QString MainWindow::AbsDir(const QString fileName) const {

    if (!fileName.isEmpty()) return QFileInfo (fileName).absoluteDir ().absolutePath ();
    else return QDir::homePath ();

}
bool MainWindow::maybeSave() {
    if (doc == 0) return true;
    if (doc->isModified()) {
        int ret = QMessageBox::warning(this, mainWindowCaption(),
                                       "The project has been modified.\n"
                                       "Do you want to save your changes?",
                                       QMessageBox::Yes | QMessageBox::Default,
                                       QMessageBox::No,
                                       QMessageBox::Cancel | QMessageBox::Escape);
        if (ret == QMessageBox::Yes)
            return saveProject();
        else if (ret == QMessageBox::Cancel)
            return false;
    }

    return true;
}
void MainWindow::setCurrentProjectFile(const QString &fileName) {

    curFile = fileName;
    modifiedProject = false;
    if (curFile.isEmpty()) {
        setWindowTitle(QString("%1 - %2").arg("unnamed").arg(mainWindowCaption()) );
        modifiedProject = true;
        return;
    }
    QString tmp = fileName;
    if (tmp[tmp.length ()-1] == '/') tmp.remove(tmp.length ()-1,1);
    setWindowTitle(QString("%1 - %2").arg(strippedName(tmp)).arg(mainWindowCaption()) );


    QSettings settings("mitp.ru", "MomTens Main");
    QStringList files = settings.value("recentProjectFileList").toStringList();

    files.removeAll(tmp);
    files.prepend(tmp);
    while (files.size() > MaxRecentFiles)
        files.removeLast();

    settings.setValue("recentProjectFileList", files);

    foreach (QWidget *widget, QApplication::topLevelWidgets()) {
        MainWindow *mainWin = qobject_cast<MainWindow *>(widget);
        if (mainWin)
            mainWin->updateRecentProjectFileActions();
    }
}
void MainWindow::updateRecentProjectFileActions() {
    QSettings settings("mitp.ru", "MomTens Main");

    QStringList files = settings.value("/recentProjectFileList").toStringList();

    int numRecentFiles = qMin(files.size(), (int)MaxRecentFiles);

    for (int i = 0; i < numRecentFiles; ++i) {
        QString text = tr("&%1 %2").arg(i + 1).arg(strippedName(files[i]));
        recentProjectFileActs[i]->setText(text);
        recentProjectFileActs[i]->setData(files[i]);
        recentProjectFileActs[i]->setVisible(true);
    }
    for (int j = numRecentFiles; j < MaxRecentFiles; ++j)
        recentProjectFileActs[j]->setVisible(false);

    separatorProjectAct->setVisible(numRecentFiles > 0);
}
void MainWindow::openRecentProjectLocation() {
    if (maybeProjectSave()) {
        QAction *action = qobject_cast<QAction *>(sender());
        if (action) {
            QString fileName = action->data().toString();
            if (QFile::exists(fileName)) {
                loadProjectLocation(fileName);
            } else  QMessageBox::critical(this,mainWindowCaption(),
                                              QString("File '%1' does not exist").arg(fileName),
                                              QMessageBox::Close);
        }

//editProject();
    }
}

bool MainWindow::maybeProjectSave() {
    if (doc != 0 && doc->isModified()) {
        int ret = QMessageBox::warning(this, mainWindowCaption(),
                                       "The project has been modified.\n"
                                       "Do you want to save your changes?",
                                       QMessageBox::Yes | QMessageBox::Default,
                                       QMessageBox::No,
                                       QMessageBox::Cancel | QMessageBox::Escape);
        if (ret == QMessageBox::Yes)
            return saveProject();
        else if (ret == QMessageBox::Cancel)
            return false;
    }
    return true;

}

bool MainWindow::loadProjectLocation(QString &dirName) {

/// push button open - fileName has last '/' , execute menu item ricently open - fileName has not last '/'
    CopyDirectory(dirName,tempDirectoryPath);
    MDocument* tmpDoc = doc;
    QString openFileName = dirName + "/" + projectBinaryFileName();
    doc = new MDocument(openFileName);
    if (!doc->GetProjectFileGood()) {
        delete doc;
        doc = tmpDoc;
        return false;
    } else delete tmpDoc;

    if (centralWidget () == 0) initUI();
    viewresult->SetMDocument(doc);
    infoproject->SetMDocument(doc);
    removeWorkingMaterials(tempDirectoryPath,false);
    CopyDirectory(dirName,tempDirectoryPath);
//---------------------------------------------------------------------------------
    QDir tempDir(dirName);
    if (tempDir.exists ("protocol.txt")) viewresult->FileActsShowHide(true);
    else viewresult->FileActsShowHide(false);
//---------------------------------------------------------------------------------
    setCurrentProjectFile(dirName);
    appkonsole->SetDateTimeKonsole(doc);
    FileActsShowHide(true);
    viewresult->FileActsShowHide(true);
    return true;
}
void MainWindow::CopyDirectory(QString &from,QString &to) {
    QApplication::setOverrideCursor(Qt::WaitCursor);
    removeWorkingMaterials(to,false);
    QDir tempDir(from);
    QFileInfoList info = tempDir.entryInfoList (QDir::Files,QDir::Name);

    for (int i = 0; i < info.size(); ++i) {
        QString curAbsoluteFilePath =  QDir::toNativeSeparators(info.at(i).absoluteFilePath());
        QString curFileName  =  QDir::toNativeSeparators(info.at(i).fileName ());
        QString saveFileName = to + "/" + curFileName;
        QString tmp = projectBinaryFileName();
        if (tmp == curFileName) continue;
        QFile::copy(curAbsoluteFilePath,saveFileName);
    }
//QMessageBox::warning(this,0,QString("curAbsoluteFilePath %1\nsaveFileName %2").arg(curAbsoluteFilePath).arg(saveFileName));



    QApplication::restoreOverrideCursor();
}
bool MainWindow::SaveWorkingMaterials(QString &dirName) {
    removeWorkingMaterials(dirName,false);
    CopyDirectory(tempDirectoryPath,dirName);
    QString saveFileName = dirName + "/" + projectBinaryFileName();
    return doc->saveBinary(saveFileName);
}
bool MainWindow::saveProjectLocation(QString &dirName) {
    if (SaveWorkingMaterials(dirName)) {
        setCurrentProjectFile(dirName);
        setDocumentModified ();
        statusBar()->showMessage(tr("Project file saved"), 2000);

    } else {
        statusBar()->showMessage(tr("Saving Project file canceled"), 2000);
        return false;
    }

    return true;
}
void MainWindow::editProject() {

    ProWizard  dialog(this,doc,ProWizard::ProjectEdit);
    dialog.resize(projectWizardSize);
    dialog.move(projectWizardPos);
    if (dialog.exec()) {
        viewresult->SetMDocument(doc);
        infoproject->SetMDocument(doc);
        if (doc->isModified()) {
            removeWorkingMaterials(tempDirectoryPath,false);
            viewresult->FileActsShowHide(false);

        } else viewresult->FileActsShowHide(true);
    }
    projectWizardPos   =  dialog.pos();
    projectWizardSize  =  dialog.size();
}
void MainWindow::newProject() {
    if (maybeProjectSave()) {
        MDocument* tmpDoc = doc;
        doc = new MDocument();
        appkonsole->SetDateTimeKonsole(doc);
        ProWizard  dialog(this,doc,ProWizard::ProjectNew);
        dialog.resize(projectWizardSize);
        dialog.move(projectWizardPos);
        if (dialog.exec()) {
            if (centralWidget () == 0) initUI();
            viewresult->SetMDocument(doc);
            infoproject->SetMDocument(doc);
            removeWorkingMaterials(tempDirectoryPath,false);
            FileActsShowHide(true);
            viewresult->FileActsShowHide(false);
            if (tmpDoc !=0 ) delete tmpDoc;

        } else {

            delete doc;
            doc = tmpDoc;

            appkonsole->SetDateTimeKonsole(doc);
        }
        projectWizardPos   =  dialog.pos();
        projectWizardSize  =  dialog.size();
    }
}
void MainWindow::openProject() {
    if (maybeProjectSave()) {
        QString dir = QFileDialog::getExistingDirectory(this, tr("Open Input Data Directory"),
                      openUserProjectDirectory,
                      // QFileDialog::ShowDirsOnly |
                      QFileDialog::DontResolveSymlinks);

        if (dir.isEmpty()) return;
        openUserProjectDirectory = QFileInfo(dir).absolutePath ();
        loadProjectLocation(dir);
    }
}
void MainWindow::setDocumentModified () {
    doc->SetValueToResetvalue();
    doc->SetValueToInitialValue();
}
bool MainWindow::saveProject() {
    if (curFile.isEmpty()) {
        return saveAsProject();
    } else {
        return saveProjectLocation(curFile);
    }
}

bool MainWindow::saveAsProject() {
    QString fileName = QFileDialog::getExistingDirectory(this, tr("Open Project Directory to save project matrials."),
                       MainWindow::self->saveUserProjectDirectory,
                       // QFileDialog::ShowDirsOnly |
                       QFileDialog::DontResolveSymlinks);

    if (!fileName.isEmpty()) {
        QDir tempDir(fileName);
        QFileInfoList info = tempDir.entryInfoList (QDir::Files,QDir::Name);
        if (info.size() > 0)  {
            int ret = QMessageBox::warning(this, mainWindowCaption(),
                                           QString("Directory %1 is not empty.\n Do you want to remove all entries?")
                                           .arg(QDir::toNativeSeparators(fileName)),
                                           QMessageBox::Yes | QMessageBox::Default,
                                           QMessageBox::Cancel | QMessageBox::Escape);
            if (ret == QMessageBox::Cancel)
                return true;
        }
        saveProjectLocation(fileName);
        saveUserProjectDirectory = QFileInfo(fileName).absolutePath ();
        return true;
    } else return false;
}

void MainWindow::execute() {

    Get3SmacModels();


    viewresult->FileActsShowHide(true);
}
void MainWindow::Get3SmacModels() {

    QString workDirectory = tempDirectoryPath;
    QDir::setCurrent(workDirectory);
///=========================================================================================
    QStringList lst = doc->GetStationModelList()->GetList   ();
    QStringList staSupModellst;
    QStringList sta3smacModellst;
    _MLT* mlt = doc->GetMlt();
    SEISMO **cs = mlt->Get_CS();
    int n = mlt->Get_I_N_SEI();

    for (int j = 0; j < lst.size(); ++j) {
        QStringList tlst = lst.at(j).split(' ', QString::SkipEmptyParts);

        int k = 0;
        bool find = false;
        for (int i = 0; i < n; ++i) {
            if ( tlst.at(1) == QString(cs[i]->Get_st_name())) {
                find = true;
                k = i;
                break;
            }
        }
        if (!find) {
            QMessageBox::warning(this,"",QString("Can't find station name \"%1\" in header records").arg(tlst.at(1)));
            return;
        }
        if (tlst.at(0) == "3Smac") sta3smacModellst << QString("%1 %2 %3").arg(tlst.at(1) + ".3Smac").arg(cs[k]->Get_slat()).arg(cs[k]->Get_slong ( ));
        else staSupModellst << QString("%1.3Smac.mod").arg(tlst.at(1));
    }
    if (doc->GetNameSourceModel()->GetValue() == "3Smac" ) sta3smacModellst << QString("%1 %2 %3").arg("source.3Smac").arg(cs[0]->Get_elat()).arg(cs[0]->Get_elong ( ));
    else  staSupModellst << QString("source.%1.mod").arg(doc->GetNameSourceModel()->GetValue());

    if (sta3smacModellst.size() == 0 && staSupModellst.size() == 0) {
        QString tmp = "All model are exist.\n";
        appkonsole->textCursor().insertText(tmp);
        appkonsole->ensureCursorVisible ();
    }
///=========================================================================================
    if (sta3smacModellst.size() > 0 ) {
        QString formeosPrm4smacFile = "/opt/fmt/3smac/prm4smac";
        QString sysStr = "cp -f " + formeosPrm4smacFile +" .";
        system(sysStr.toAscii());
        QFile file("list_sta");
        if (!file.open(QIODevice::WriteOnly | QIODevice::Text)) {
            QMessageBox::warning(this,"",QString("Can't open file for writing\n %1").arg("list_sta"));
            return;
        }
        QTextStream out(&file);
//    QStringList lst = doc->GetStationModelList()->GetList   ();


        for (int j = 0; j < sta3smacModellst.size(); ++j) {
            QStringList tlst = sta3smacModellst.at(j).split(' ', QString::SkipEmptyParts);
            out << tlst.at(0) + "\n";
            out << tlst.at(1) << tlst.at(2) + "\n";
        }
        file.close();

        QApplication::setOverrideCursor(Qt::WaitCursor);
        appkonsole->Get3SmacModels();
        QApplication::restoreOverrideCursor();
//   system("form_eos ");
//   system("3sma2mod ");

        QFileInfoList info = tempDirectory.entryInfoList (QDir::Files,QDir::Name);

        for ( int i = 0; i < info.size(); ++i ) {
            QFileInfo  infoTemp = info.at ( i );
            QString itemName = infoTemp.filePath();
            if (itemName.contains(".3sm")  || itemName.contains("prm4smac") || itemName.contains(" list_mod")  || itemName.contains("list_sta") || itemName.contains("feos.log") || itemName.contains("list_mod"))
                if ( infoTemp.isFile ()) {
                    if (!tempDirectory.remove ( itemName )) QMessageBox::critical(0,"",
                                QString("Can't remove file %1").arg(itemName));
                }
        }

        QDir::setCurrent(rootPath() + "/bin");
        QApplication::restoreOverrideCursor();
    }
}








