//
// C++ Implementation: records
//
// Description:
//
//
// Author: fmt <fmt@localhost.localdomain>, (C) 2008
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include <QtGui>
#include <QMessageBox>
#include <QProcess>
#include "default.h"
#include "document.h"
#include "mainwindow.h"
#include "sei.h"
#include "records.h"
#include "models.h"
#include "range.h"
#include "editline.h"
#include <float.h>
#include <math.h>

bool hasInputFiles(QString directory,QStringList &list) {
	QDir dir(directory);
	if ( !dir.exists () ) {
		QMessageBox::warning ( 0, mainWindowCaption(),
		                       QString ( "Directory <%1> does not exist." )
		                       .arg (directory));
		return false;
	}
	QStringList filters;
	QString tmp = "";
	tmp = "\"ah binaries\" files";
	filters << "*" + ahFilterLocator();

	dir.setNameFilters(filters);
	QStringList listtemp;
	listtemp = dir.entryList(filters,QDir::Files);
	if (listtemp.isEmpty ()) {

		QMessageBox::warning ( 0,mainWindowCaption(),
		                       QString ( "Directory <%1> \n does not contain %2." )
		                       .arg ( dir.absolutePath () )
		                       .arg ( tmp ) );
		return false;
	}
	if (listtemp.size() < 5) {


		QMessageBox::warning ( 0,mainWindowCaption(),
		                       QString ( "Directory <%1> .\nThe number of records ( %2 ) should be more or equally 5." )
		                       .arg ( dir.absolutePath () )
		                       .arg ( listtemp.size()) );
		return false;

	}
	foreach(QString str,listtemp) {
		str = directory + "/" + str;
		list << str;
	}
	return true;

}

RecordsImpl::RecordsImpl(QWidget *parent,MDocument* doc,ProWizard::DocumentUsing documentUsing)
		: QDialog(parent),doc(doc),documentUsing(documentUsing) {
	setFont( parent->font() );
	setWindowIcon(QIcon(":/images/icon.png"));
	setupUi(this);
	InitDialog();
	if (documentUsing != ProWizard::ProjectNew) SetDocumentToDialog();
	connect( pushButtonInputDirName, SIGNAL( clicked() ), this, SLOT( browse() ) );

	connect( pushButtonAddSel, SIGNAL( clicked() ), this, SLOT( addsel() ) );
	connect( pushButtonAddAll, SIGNAL( clicked() ), this, SLOT( addall() ) );
	connect( pushButtonDelSel, SIGNAL( clicked() ), this, SLOT( delsel() ) );
	connect( pushButtonDelAll, SIGNAL( clicked() ), this, SLOT( delall() ) );

	connect( pushButtonSetOrbit, SIGNAL( clicked() ), this, SLOT( setOrbit() ) );
	connect( pushButtonSetAllOrbit, SIGNAL( clicked() ), this, SLOT( setOrbitAll() ) );

	listWidgetIn->setSelectionMode(QAbstractItemView::MultiSelection);
	listWidgetProject->setSelectionMode(QAbstractItemView::MultiSelection);
	groupBoxOrbit->hide();

}

void RecordsImpl::addsel() {
	if (!isFtanProject && CheckIsOrbit()) {
		QMessageBox::warning ( 0,mainWindowCaption(),
		                       QString ( "Input record list contains record(s) without orbit number." ));
		return;
	}
	QItemSelectionModel * selmod = listWidgetIn->selectionModel ();
	QItemSelection sel = selmod->selection ();
	if (sel.count () == 0) return;
	QItemSelectionRange range;
	for (int i = 0; i < sel.count (); ++i) {
		range = sel.at(i);
		int selTopRow    =   range.top();
		int selNumRows   =   range.height();
		for (int j = 0; j < selNumRows; ++j) {
			int row = selTopRow + j;
			QListWidgetItem * item = listWidgetIn->item (row );
			QString str = item->data ( Qt::DisplayRole).toString();
			listWidgetProject->count ();
			bool finddouble = false;
			for (int k = 0; k < listWidgetProject->count (); ++k) {
				QListWidgetItem * itemProj = listWidgetProject->item (k );
				if (str == itemProj->data ( Qt::DisplayRole).toString()) {
					finddouble = true;
					break;
				}


			}
			if (finddouble) {
				QMessageBox::warning ( 0,mainWindowCaption(),
				                       QString ( "Record <%1> is in Project record list already." )
				                       .arg ( str) );
			} else listWidgetProject->addItem (str);
		}
	}
	QMultiMap<QString,int > map;
	for (int k = 0; k < listWidgetProject->count (); ++k) {
		QListWidgetItem * itemProj = listWidgetProject->item (k );
		QString str = itemProj->data ( Qt::DisplayRole).toString();
		map.insert(str,k);
	}
	listWidgetProject->clear();
	QStringList keys = map.keys();
	listWidgetProject->addItems(keys);
	FindDoubleRecordsInlistWidgetProject();
	labelProject->setText(QString("Records: %1").arg(listWidgetProject->count()));
}
void RecordsImpl::addall() {
	if (!isFtanProject && CheckIsOrbit()) {
		QMessageBox::warning ( 0,mainWindowCaption(),
		                       QString ( "Input record list contains record(s) without orbit number." ));
		return;
	}
	listWidgetProject->clear ();

	QStringList list;
	for (int i = 0; i< listWidgetIn->count (); ++i ) {

		QListWidgetItem * item = listWidgetIn->item (i);
		QString str = item->data ( Qt::DisplayRole).toString();
		list << str;
	}

	listWidgetProject->addItems(list);
	FindDoubleRecordsInlistWidgetProject();

	labelProject->setText(QString("Records: %1").arg(listWidgetProject->count()));

}
///--------------------------------------------------------------------
bool	RecordsImpl::CheckIsOrbit() {
	bool findItemNoNumberOrbit = false;
	for (int i = 0; i< listWidgetIn->count (); ++i ) {
		QListWidgetItem * item = listWidgetIn->item (i);
		QString str = item->data ( Qt::DisplayRole).toString();
		if (str.contains('?') ) {
			findItemNoNumberOrbit = true;
			item->setForeground ( QBrush (Qt::blue) );
		} else
			item->setForeground ( QBrush (Qt::black) );
	}
	return findItemNoNumberOrbit;
}
void RecordsImpl::setOrbit() {
	QItemSelectionModel * selmod = listWidgetIn->selectionModel ();
	QItemSelection sel = selmod->selection ();
	if (sel.count () == 0) return;
	QItemSelectionRange range;
	for (int i = 0; i < sel.count (); ++i) {
		range = sel.at(i);
		int selTopRow    =   range.top();
		int selNumRows   =   range.height();
		for (int j = 0; j < selNumRows; ++j) {
			int row = selTopRow + j;
			QListWidgetItem * item = listWidgetIn->item (row);
			QString str = item->data ( Qt::DisplayRole).toString();
			QStringList list = str.split(':',QString::SkipEmptyParts);
			QString tmp = QString("%1").arg(spinBoxNumberOrbit->value());
			str = "";
			for (int k = 0; k< list.size(); ++k ) if (k == 1) str += tmp + ":";
				else str += list.at(k) + ":";
			str.remove(str.length() - 1,1);
			item->setData (Qt::DisplayRole, str);
		}
	}

	listWidgetIn->clearSelection ();
	CheckIsOrbit();
}
void RecordsImpl::setOrbitAll() {


	for (int i = 0; i< listWidgetIn->count (); ++i ) {

		QListWidgetItem * item = listWidgetIn->item (i);
		QString str = item->data ( Qt::DisplayRole).toString();
		QStringList list = str.split(':',QString::SkipEmptyParts);
		QString tmp = QString("%1").arg(spinBoxNumberOrbit->value());
		str = "";
		for (int j = 0; j< list.size(); ++j ) if (j == 1) str += tmp + ":";
			else str += list.at(j) + ":";
		str.remove(str.length() - 1,1);

		item->setData (Qt::DisplayRole, str);
	}

	CheckIsOrbit();
}

///---------------------------------------------------------------------
void RecordsImpl::delsel() {
	QItemSelectionModel * selmod = listWidgetProject->selectionModel ();
	QItemSelection sel = selmod->selection ();
	if (sel.count () == 0) return;
	QItemSelectionRange range;
	for (int i = 0; i < sel.count (); ++i) {
		range = sel.at(i);
		int selTopRow    =   range.top();
		int selNumRows   =   range.height();
		for (int j = 0; j < selNumRows; ++j) {
			int row = selTopRow + j;
			listWidgetProject->takeItem(row);
		}
	}
	FindDoubleRecordsInlistWidgetProject();

	labelProject->setText(QString("Records: %1").arg(listWidgetProject->count()));
}
void RecordsImpl::delall() {

	listWidgetProject->clear ();
	labelProject->setText(QString("Records: %1").arg(listWidgetProject->count()));
}
void RecordsImpl::FinishWizard() {
	GetDocumentFromDialog();
	foreach (PropertyValue* val, values) val->SetValueToResetvalue();
}
void RecordsImpl::browse() {

	QString dir = QFileDialog::getExistingDirectory(this, tr("Open Input Data Directory"),
	              MainWindow::self->openInputDataDirectory,
	              // QFileDialog::ShowDirsOnly |
	              QFileDialog::DontResolveSymlinks);

	if (!dir.isEmpty()) {
		QString ftanProjectFile = dir +"parameters.bin";
		QString periodlistFileName = dir + "parameters.periodlist";
		QString AhDir = QString();
		QString ftanAhDir = dir + "wfah";
		if (QFileInfo(ftanProjectFile).exists()) {
			if (!QFileInfo(periodlistFileName).exists()){

				QMessageBox::warning ( 0, mainWindowCaption(),
				                       QString ( "Ftan program output directory <%1>  exists.\nBut text file with filtering periods <%2> does not exist." )
				                       .arg (dir).arg (periodlistFileName));
				lineEditInputDirectory->setText(QString());
				return;

			}
			if (QDir(ftanAhDir).exists()) {
				isFtanProject = true;
				AhDir = ftanAhDir;
			} else {
				QMessageBox::warning ( 0, mainWindowCaption(),
				                       QString ( "Ftan program output directory <%1>  exists.\nBut directory with cleaned form <%2> does not exist." )
				                       .arg (dir).arg (ftanAhDir));
				lineEditInputDirectory->setText(QString());
				return;
			}
		} else {
			isFtanProject = false;
			AhDir = dir;
		}
		recordList.clear();
		if (!hasInputFiles(AhDir,recordList)) {
			lineEditInputDirectory->setText(QString());
			return;
		}
		if (isFtanProject) doc->GetIsFtanProject()->SetValue("True");
		else doc->GetIsFtanProject()->SetValue("False");
		lineEditInputDirectory->setText(dir);
		MainWindow::self->openInputDataDirectory = dir;
		inputDataDirectoryName = dir;
		doc->GetRecordSeisMaterials(recordList);
		if (!isFtanProject) groupBoxOrbit->show();
		SetFilesToInList();
		if (isFtanProject)
			ftanPeriodFileName = dir + "parameters.periodlist";

	}


}
bool RecordsImpl::GetFtanRange(QString &fileName){

	QFile file(fileName);
	if (!file.open(QFile::ReadOnly | QFile::Text)) {
		QMessageBox::warning(0,mainWindowCaption(),
		                     QString("Cannot read file %1:\n%2.")
		                     .arg(fileName)
		                     .arg(file.errorString()));
		return false;
	}
	QStringList list;
	QTextStream in(&file);
	QString line;
	do {
		line = in.readLine();

		list << line;

	} while (!in.atEnd ());

	file.close();

	rangeTmin = FLT_MAX;
	rangeTmax = -FLT_MAX;
//QMessageBox::warning(0,"",QString("RecordsImpl::GetFtanRange 0 size %1").arg(list.size()));
	for (int i = 0; i < list.size(); ++i) {
		QString tmp = list.at(i);
		QStringList part = tmp.split('@',QString::SkipEmptyParts);
		QString partsta = part.at(0);
		QString partcomp = part.at(1);
		QString partorbitdatatime = part.at(2);
		QString partminT = part.at(3);
		QString partmaxT = part.at(4);
		QStringList partorbitdatetime = partorbitdatatime.split(' ',QString::SkipEmptyParts);
		QString partorbit = partorbitdatetime.at(0);
		QString partdate  = partorbitdatetime.at(1);
		QString parttime  = partorbitdatetime.at(2);
//QMessageBox::warning(0,"","RecordsImpl::GetFtanRange 1");
		for (int j = 0; j < listWidgetProject->count (); ++j) {
			QListWidgetItem * itemProj = listWidgetProject->item (j);
			QString str = itemProj->data ( Qt::DisplayRole).toString();
			QStringList partPro = str.split(' ',QString::SkipEmptyParts);
			QString name = partPro.at(0);
			QString date = partPro.at(1);
			QString time = partPro.at(2);
			QStringList orbitstacomp = name.split(':',QString::SkipEmptyParts);
			QString sta = orbitstacomp.at(0);
			QString orbit = orbitstacomp.at(1);
			QString comp = orbitstacomp.at(2);
			if (partsta == sta &&   partcomp ==comp &&   partorbit == orbit &&
			        partdate == date && parttime == time) {
				float min = partminT.toFloat();
				float max = partmaxT.toFloat();
				if (min < rangeTmin ) rangeTmin = min;
				if (max > rangeTmax ) rangeTmax = max;
				QString tmp;
				tmp += sta + ":";
				tmp +=  orbit + ":";
				tmp += comp + " ";
				tmp += partminT + " ";
				tmp += partmaxT;
				RangeWaveListTmp << tmp;
//QMessageBox::warning(0,"",QString("RecordsImpl::GetFtanRange 1  '%1'").arg(tmp));
			}
		}
	}
	/*
		QString tmp = "RangeImpl::SetDocumentToDialog RangeWaveList\n";
		for (int i = 0; i < RangeWaveListTmp.size(); ++i) {
			tmp += RangeWaveListTmp.at(i) + "\n";
		}
		QMessageBox::warning(0,"",tmp);

	*/
	return true;
}

void	RecordsImpl::SetFilesToInList() {
	if (listWidgetIn->count () > 0) listWidgetIn->clear();
	if (listWidgetProject->count () > 0) listWidgetProject->clear();
	QMultiMap<QString,SEISMO *> map = doc->GetMapSeies();
	QStringList files;
	QList<QString> keyList = map.keys();
	QList<SEISMO*> valueList = map.values();
	for (int i = 0; i < keyList.size(); ++i) {
		QString key = keyList.at(i);
		//key.remove (' ');
		int k = -1;
		QStringList parts = key.split(' ',QString::SkipEmptyParts);
		if (parts.at(0)[parts.at(0).length() - 1] != 'Z') {
			bool findHcomp = false;
			if (parts.at(0)[parts.at(0).length() - 1] == 'E'){

				for (int j = i; j < keyList.size(); ++j) {
					QString keyCur = keyList.at(j);
					QStringList partskeyCur = keyCur.split(' ',QString::SkipEmptyParts);
					if (partskeyCur.at(0)[partskeyCur.at(0).length() - 1] == 'N') {
						QString tmp1 = parts.at(0);
						tmp1.remove(tmp1.length() -1,1);
						QString tmp2 = partskeyCur.at(0);
						tmp2.remove(tmp2.length() -1,1);
						if (tmp1 == tmp2 && parts.at(1) == partskeyCur.at(1) && parts.at(2) == partskeyCur.at(2) ) {
							k = j;
							findHcomp = true;
							break;
						}

					}
				}
			}

//QMessageBox::warning(0, "","SetFilesToInList 0");
			if (findHcomp) {
//QMessageBox::warning(0, "","SetFilesToInList 1");
				QString tmp3 = parts.at(0);
				if (valueList.at(i)->Get_instr()[0] == 'L')  tmp3[tmp3.length() - 1] = 'T';
				if (valueList.at(i)->Get_instr()[0] == 'R')  tmp3[tmp3.length() - 1] = 'R';
				QString name =  QString(valueList.at(i)->Get_st_name());
				int length = name.length ();
				if (length == 3) tmp3 += " ";
				QString temp = tmp3 + " " + parts.at(1) + " " + parts.at(2);
				files << temp;
				doc->AppendUsingRecordsMap(temp,i,k);
			}

//QMessageBox::warning(0, "","SetFilesToInList 2");
		} else {
			files << key;
			doc->AppendUsingRecordsMap(key,i,k);
		}
	}



	listWidgetIn->addItems(files);
	labelIn->setText(QString("Records: %1").arg(listWidgetIn->count()));
	if (isFtanProject) {
		if (documentUsing == ProWizard::ProjectNew) {
			listWidgetProject->addItems(files);
			FindDoubleRecordsInlistWidgetProject();
		}
		else {
			QStringList fileslist = doc->GetProjectListNameSeis()->GetList();
			listWidgetProject->addItems(fileslist);
		}
		labelProject->setText(QString("Records: %1").arg(listWidgetProject->count()));
	} else CheckIsOrbit();
///-----------------------------------------------------------------------------------
	/*
	    QMultiMap<QString,QString > tmap =  doc->GetRecordMap();
	    QList<QString> keyTemp = tmap.keys();
	    QList<QString> valueTemp = tmap.values();
	    QMessageBox::warning(0, "",QString("keyTemp.size() %1").arg(keyTemp.size()));
	    for (int i = 0; i < keyTemp.size(); ++i) {
	    QStringList lst = valueTemp.at(i).split('@',QString::SkipEmptyParts);
	    QString prn = keyTemp.at(i) + "\n";
	    int ii = lst.at(0).toInt();
	    QString fname = valueList.at(ii)->Get_fname();
	    prn += fname + "\n";
	    int jj = lst.at(1).toInt();
	    if (jj != -1) {
	    QString fname1 = valueList.at(jj)->Get_fname();
	    prn += fname1 + " ";
	   }
	    QMessageBox::warning(0, "",prn);
	   }
	*/
///-----------------------------------------------------------------------------------


}
int RecordsImpl::FindDoubleRecordsInlistWidgetProject() {
	int numdouble = 0;
	QMultiMap<int,int > mapdouble;
	for (int i = 0; i < listWidgetProject->count (); ++i) {
		QListWidgetItem * itemProj = listWidgetProject->item (i);
		QString str = itemProj->data ( Qt::DisplayRole).toString();
		QStringList list = str.split(' ');
		str = list.at(0);
		bool finddouble = false;
		int numfirst = 0;
		int numsecond = 0;
		for (int j = i+1; j < listWidgetProject->count (); ++j) {
			QListWidgetItem * itemdouble = listWidgetProject->item (j );
			QString strdouble = itemdouble->data ( Qt::DisplayRole).toString();
			QStringList listdouble = strdouble.split(' ');
			strdouble = listdouble.at(0);


//QMessageBox::critical(0,mainWindowCaption(),QString("str '%1'i %2 strdouble '%3' j  %4").arg(str).arg(i).arg(strdouble).arg(j));
			if (str == strdouble) {
				finddouble = true;
				numfirst = i;
				numsecond = j;
				break;
			}
		}
		if (finddouble) {
			mapdouble.insert(numdouble,numfirst);
			mapdouble.insert(numdouble,numsecond);
			numdouble++;
		}
	}
	for (int i = 0; i < listWidgetProject->count(); ++i) {
		QListWidgetItem * itemProj = listWidgetProject->item (i);
		itemProj->setForeground ( QBrush (Qt::black) );
	}
	QList<int> values = mapdouble.values();
//QMessageBox::critical(0,mainWindowCaption(),
//                                              QString("size %1").arg(values.size()));
	for (int i = 0; i < values.size(); ++i) {
		QListWidgetItem * itemProj = listWidgetProject->item (values.at(i));
		itemProj->setForeground ( QBrush (Qt::blue) );
	}
	return mapdouble.size();
}
void RecordsImpl::InitDialog() {

	lineEditInputDirectory->setReadOnly ( true );
	spinBoxNumberOrbit->setRange (1,16 );
	spinBoxNumberOrbit->setSingleStep (1);
	if (documentUsing == ProWizard::ProjectEdit) pushButtonInputDirName->hide();
	/*
	    values.append(doc->GetDbaseName ());
	    values.append(doc->GetDbaseDirectoryName ());
	    DbaseDirectoryName = QString();
	    lineEditDbaseDirectory->setReadOnly ( true );
	*/
}
void RecordsImpl::SetDocumentToDialog() {
	inputDataDirectoryName =doc->GetInputDataDirectory()->GetValue();
	lineEditInputDirectory->setText(inputDataDirectoryName);
//QMessageBox::critical(0,"",QString("SetDocumentToDialog  dir '%1'").arg(inputDataDirectoryName));
	if (doc->GetIsFtanProject()->GetValue() == "True" ) isFtanProject = true;
	else isFtanProject = false;
	SetFilesToInList();
}
bool RecordsImpl::CheckDialogParameeters() {
	if (listWidgetIn->count () == 0) {
		QMessageBox::critical(0,mainWindowCaption(),QString("Input records list is empty."));
		return false;
	}
	if (listWidgetProject->count () == 0) {
		QMessageBox::critical(0,mainWindowCaption(),QString("Project records list is empty."));
		return false;
	}
	if (FindDoubleRecordsInlistWidgetProject() > 0){
		QMessageBox::critical(0,mainWindowCaption(),QString("Project records list has 'double' records."));
		return false;
	}


	QString outOfRange = "Following project records did not found in Input record list:\n";
	bool find = false;

	for (int i = 0; i < listWidgetProject->count (); ++i) {
		QListWidgetItem * itemProj = listWidgetProject->item (i);
		QString strProj = itemProj->data ( Qt::DisplayRole).toString();
		bool tmpfind = true;
		for (int j = 0; j < listWidgetIn->count (); ++j) {
			QListWidgetItem * itemIn = listWidgetIn->item (j);
			QString strIn = itemIn->data ( Qt::DisplayRole).toString();
			if (strProj == strIn) tmpfind = false;
		}
		if (tmpfind) {
			find = true;
			outOfRange += strProj + "\n";
		}
	}

	if (find) {
		QMessageBox::critical(0,mainWindowCaption(),outOfRange);
		return false;
	}

	if (isFtanProject) {
		if (documentUsing == ProWizard::ProjectNew) {
		GetFtanRange(ftanPeriodFileName);
			ProWizard::wizardSelf->rangePage->rangeImpl->SetRangeWaveList(RangeWaveListTmp,rangeTmin,rangeTmax);
                   }
	}


ProWizard::wizardSelf->modelsPage->modelsPageImpl->SetStationLists();
ProWizard::wizardSelf->modelsPage->modelsPageImpl->SetModelStationLists();
	return true;

}
void RecordsImpl::GetDocumentFromDialog() {
//QMessageBox::critical(0,mainWindowCaption(),"GetDocumentFromDialog()");
	if (isFtanProject) doc->GetIsFtanProject()->SetValue("True");
	else doc->GetIsFtanProject()->SetValue("False");
	doc->GetInputDataDirectory()->SetValue(inputDataDirectoryName);

	QStringList temp;
	for ( int i = 0; i < listWidgetIn->count (); ++i)
		temp << listWidgetIn->item (i)->data ( Qt::DisplayRole).toString();
	doc->GetInListNameSies()->SetListToValue(temp);
	temp.clear();

	for ( int i = 0; i < listWidgetProject->count (); ++i)
		temp << listWidgetProject->item (i)->data ( Qt::DisplayRole).toString();
	doc->GetProjectListNameSeis()->SetListToValue(temp);

}
