#ifndef _FMT_SEISMOGRAMM_H
#define _FMT_SEISMOGRAMM_H
#include <QStringList>
#include <QString>
#include <QList>
#include <string.h>
class   HEADER_SEI
{
		/* ======================= common HEADER =======================*/
		char header[79];     /*  comment             */
		char comp[1];           /*  component (z,n,e)   */
		char instr[9];       /*  type of seismometer */
		char st_name[10];        /*  station name        */
		/* ----------------------- Origin time >: -----------------------*/

		long  year;
		long  month;
		long  day;
		long  hour;
		long  minute;
		float sec;

		/* ----------------------- hypocenter  >: -----------------------*/

		float elat;
		float elong;
		float depth;

		/* ----------------------- station     >: -----------------------*/

		float slat;
		float slong;
		float elevation;

		/* ---------- instrument's orientation (degrees) >: -------------*/

		float polan;         /*  angle with vertical   */
		float az;            /*  azimuth               */
		float dist;          /*  distance from Station */
		/*  to epicenter          */
		float aztos;         /*  azimuth to Station    */
		float azfrs;         /*  azimuth from Station  */
                float takeoffang;    /* take off ang           */
		/* ======================= TIME - HEADER ============================*/
		long  nsampl;        /* number of Points     */
		float  t0;            /* Start time of record */
		float  shift;            /* Shift p-Wave */
		float  tpwave;            /* P-Wave godograph time arrival */

		float  pickarrival;   /* Pick First arrival manually by procsies program */
                                      /* PLUS P-wave time arrival bygodograph  */
		float  dt;            /* Time interval        */
		float  amp;           /* signal amplitude     */
		float  DS;            /* gain	*/
		/* ===================RESPONSE - HEADER =============================*/
		long  icot;             /* type of instrument correction :   */
		/* icot   = 0  -  uncertain          */
		/* ABS(icot)  = 1  -  no correction in the past          */
		/* ABS(icot)  = 2  -  wrong information about correction */
		/* ABS(icot)  > 2  -  type of correction                 */
		/*    "icot" always is WRITTEN into file "namo"          */
		/*     but information itself ONLY if  incor > 0         */
		long   nzer;            /* number of zeros                   */
		long   npol;            /* number of poles                   */
		float  amp1;            /* a leading factor of the           */
		/*          instrumental response    */
		float  amp2;            /* sensitivity of instrument, bit/m  */
		float  zer[20][2];      /* array of zeros (re,im)   */
		float  pol[50][2];       /*array of poles (re,im)   */
		/*  ======================= End definition ======================*/
	public:
		HEADER_SEI();

		virtual             ~HEADER_SEI();
		void Write_Seismo ( QString &fileOut,float *set,bool isResponse );
		void Read_Seismo ( char *,float *,int st = 0,int fn = 0 );

		void Set_header ( char *hd )
		{
			if ( strlen ( hd ) > 80 ) hd[79] =  '\0';
			strcpy ( header,  hd );
		}

		void Set_comp ( char *cm )  {  comp[0] =    cm[0]; }
		void Set_instr ( char *inst )
		{
			if ( strlen ( inst ) > 10 ) inst[9] =  '\0';
			strcpy ( instr,   inst );
		}
		void Set_st_name ( char *st_nam )
		{
			if ( strlen ( st_nam ) >11 ) st_nam[10] =  '\0';
			strcpy ( st_name, st_nam );
		}

		char *Get_header ( char *hd )  { return strcpy ( hd,header );}
		char *Get_comp ( char *cm )  { return strcpy ( cm,comp );}
		char *Get_instr ( char *inst )  { return strcpy ( inst,instr );}
		char *Get_st_name ( char *st_nam )  { return strcpy ( st_nam,st_name );}

		char *Get_header ( )  { return header ;}
		char *Get_comp ( )  { return comp   ;}
		char *Get_instr ( )  { return instr  ;}
		char *Get_st_name ( )  { return st_name;}

		void  Set_year ( long  yea )  { year  = yea   ;}
		void  Set_month ( long  mont )  { month = mont  ;}
		void  Set_day ( long  da )  { day   = da    ;}
		void  Set_hour ( long  hou )  { hour  = hou   ;}
		void  Set_minute ( long  minut )  { minute= minut ;}
		void  Set_sec ( float se )  { sec   = se    ;}

		long  Get_year ( )  { return ( year );}
		long  Get_month ( )  { return ( month );}
		long  Get_day ( )  { return ( day );}
		long  Get_hour ( )  { return ( hour );}
		long  Get_minute ( )  { return ( minute );}
		float Get_sec ( )  { return ( sec );}

		long  Get_year ( long&  yearr )  { return ( yearr = year );}
		long  Get_month ( long&  monthh )  { return ( monthh = month );}
		long  Get_day ( long&  dayy )  { return ( dayy =  day );};
		long  Get_hour ( long&  hourr )  { return ( hourr = hour );}
		long  Get_minute ( long&  minutee )  { return ( minutee = minute );}
		float Get_sec ( float& secc )  { return ( secc = sec );}

		void  Set_elat ( float ela )  { elat  =  ela    ;}
		void  Set_elong ( float elon )  { elong =  elon   ;}
		void  Set_depth ( float dept )  { depth =  dept   ;}

		float Get_elat ( float& elatt )  { return ( elatt = elat );}
		float Get_elong ( float& elongg )  { return ( elongg = elong );}
		float Get_depth ( float& depthh )  { return ( depthh = depth );}
		float Get_elat ( )  { return ( elat );}
		float Get_elong ( )  { return ( elong );}
		float Get_depth ( )  { return ( depth );}

		void  Set_slat ( float  sla )  {  slat      = sla      ;}
		void  Set_slong ( float  slon )  {  slong     = slon     ;}
		void  Set_elevation ( float  elevatio )  {  elevation = elevatio ;}
		void  Set_polan ( float  pola )  {  polan     = pola     ;}
		void  Set_az ( float  azz )  {  az        = azz      ;}
		void  Set_dist ( float  dis )  {  dist      = dis      ;}
		void  Set_aztos ( float  azto )  {  aztos     = azto     ;}
		void  Set_azfrs ( float  azfr )  {  azfrs     = azfr     ;}
		void  Set_takeoffang ( float  takeoff )  {  takeoffang  = takeoff     ;}
                
		void  Set_nsampl ( long   nsamp )  {  nsampl    = nsamp    ;}
		void  Set_t0 ( float  t00 )  {  t0        = t00      ;}
		void  Set_shift ( float  t00 )  {  shift        = t00   ;}
		void  Set_tpwave ( float  t00 )  { tpwave        = t00   ;}
		void  Set_pickarrival ( float  t00 )  { pickarrival        = t00   ;}

		void  Set_dt ( float  dtt )  {  dt        = dtt      ;}
		void  Set_amp ( float  am )  {  amp       = am       ;}
		void  Set_DS ( float  ds )  {  DS        = ds       ;}
		void  Set_icot ( long   ico )  {  icot      = ico      ;}
		void  Set_amp1 ( float  ampp1 )  {  amp1      = ampp1    ;}
		void  Set_amp2 ( float  ampp2 )  {  amp2      = ampp2    ;}


		float Get_slat ( float&  sla )  { return ( sla     = slat );}
		float Get_slong ( float&  slon )  { return ( slon    = slong );}
		float Get_elevation ( float&  elevatio )  { return ( elevatio= elevation );}
		float Get_polan ( float&  pola )  { return ( pola    = polan );}
		float Get_az ( float&  azz )  { return ( azz     = az );}
		float Get_dist ( float&  dis )  { return ( dis     = dist );}
		float Get_aztos ( float&  azto )  { return ( azto    = aztos );}
		float Get_azfrs ( float&  azfr )  { return ( azfr    = azfrs );}
		float Get_takeoffang ( float&  takeoff )  { return ( takeoff    = takeoffang );}


		long  Get_nsampl ( long&   nsamp )  { return ( nsamp   = nsampl );}
		float Get_t0 ( float&  t00 )  { return ( t00     = t0 );}
		float Get_shift ( float&  t00 )  { return ( t00     = shift );}
		float Get_tpwave ( float&  t00 )  { return ( t00     = tpwave );}
		float Get_pickarrival ( float&  t00 )  { return ( t00     = pickarrival );}

		float Get_dt ( float&  dtt )  { return ( dtt     = dt );}
		float Get_amp ( float&  am )  { return ( am      = amp );}
		long  Get_icot ( long&   ico )  { return ( ico     = icot );}
		float Get_amp1 ( float&  ampp1 )  { return ( ampp1   = amp1 );}
		float Get_amp2 ( float&  ampp2 )  { return ( ampp2   = amp2 );}

		float Get_slat ( )  { return ( slat );}
		float Get_slong ( )  { return ( slong );}
		float Get_elevation( )  { return ( elevation );}
		float Get_polan ( )  { return ( polan );}
		float Get_az ( )  { return ( az );}
		float Get_dist ( )  { return ( dist );}
		float Get_aztos ( )  { return ( aztos );}
		float Get_azfrs ( )  { return ( azfrs );}
		float Get_takeoffang ( )  { return ( takeoffang );}

		long  Get_nsampl ( )  { return ( nsampl );}
		float Get_t0 ( )  { return ( t0 );}
		float Get_shift ( )  { return ( shift );}
		float Get_tpwave ( )  { return ( tpwave );}
		float Get_pickarrival ( )  { return ( pickarrival );}

		float Get_dt ( )  { return ( dt );}
		float Get_amp ( )  { return ( amp );}
		float Get_DS ( )  { return ( DS );}
		long  Get_icot ( )  { return ( icot );}
		float Get_amp1 ( )  { return ( amp1 );}
		float Get_amp2 ( )  { return ( amp2 );}



		void  Set_zer_pol ( int nzzer, float **zzer,int nppol,float **ppol )
		{
			nzer = nzzer;  for ( int i = 0; i < nzer;i++ )
			{
				zer[i][0] = zzer[i][0]; zer[i][1] = zzer[i][1];
			}
			npol = nppol;  for ( int i = 0; i < npol;i++ )
			{
				pol[i][0] = ppol[i][0]; pol[i][1] = ppol[i][1];
			}
		}
		void  Set_zer_pol ( int nzzer, float zzer[20][2],int nppol,float ppol[50][2] )
		{
			nzer = nzzer;  for ( int i = 0; i < nzer;i++ )
			{
				zer[i][0] = zzer[i][0]; zer[i][1] = zzer[i][1];
			}
			npol = nppol;  for ( int i = 0; i < npol;i++ )
			{
				pol[i][0] = ppol[i][0]; pol[i][1] = ppol[i][1];
			}
		}

		void  Get_zer_pol ( int& nzzer, float **zzer,int& nppol,float **ppol )
		{
			nzzer = ( int ) nzer;  for ( int i = 0; i < nzer;i++ )
			{
				zzer[i][0] = zer[i][0]; zzer[i][1] = zer[i][1];
			}
			nppol = ( int ) npol;  for ( int i = 0; i < npol;i++ )
			{
				ppol[i][0] = pol[i][0]; ppol[i][1] = pol[i][1];
			}
		}
		void  Get_zer_pol ( int& nzzer, float zzer[20][2],int& nppol,float ppol[50][2] )
		{
			nzzer = ( int ) nzer;  for ( int i = 0; i < nzer;i++ )
			{
				zzer[i][0] = zer[i][0]; zzer[i][1] = zer[i][1];
			}
			nppol = ( int ) npol;  for ( int i = 0; i < npol;i++ )
			{
				ppol[i][0] = pol[i][0]; ppol[i][1] = pol[i][1];
			}
		}
		long Get_nzer() { return ( nzer ); }
		long Get_npol() {return ( npol ); }
		float Get_re_zer ( int i ) { return ( zer[i][0] );}
		float Get_im_zer ( int i ) { return ( zer[i][1] );}
		float Get_re_pol ( int i ) { return ( pol[i][0] );}
		float Get_im_pol ( int i ) { return ( pol[i][1] );}

		void Set_nzer ( long nnzer ) { nzer = nnzer; }
		void Set_npol ( long nnpol ) { npol = nnpol; }
		void Set_re_zer ( int i,float f ) { zer[i][0] = f;}
		void Set_im_zer ( int i,float f ) { zer[i][1] = f;}
		void Set_re_pol ( int i,float f ) { pol[i][0] = f;}
		void Set_im_pol ( int i,float f ) { pol[i][1] = f;}

};

//=================================================================================
class SEISMO:public HEADER_SEI
{
		QString fname;
		float *set;
		float max;
		float mod;
		int lag;

	public:

		SEISMO();
		virtual                   ~SEISMO();


		void Set_fname ( QString & ffname )
		{
			fname = ffname;
		}
		bool  Read_Header_Sei ( const QString  &name,long step,int indDetrend );
		void TrapIntegral();
		void  Set_max ( float  ma )  { max  = ma   ;}
		void  Set_mod ( float  mo )  { mod  = mo   ;}
		void   Set_lag ( int    la )  { lag  = la   ;}

//char *Get_fname(char *ffname) { return strcpy( ffname,fname);

//}
		QString & Get_fname() { return fname;}
		float  Get_max ( )  { return ( max );}
		float  Get_mod ( )  { return ( mod );}
		int    Get_lag ( )  { return ( lag );}
		float  Get_max ( float maxx )  { return ( maxx = max );}
		float  Get_mod ( float modd )  { return ( modd = mod );}
		int    Get_lag ( int lagg )  { return ( lagg = lag );}

		void  Set_data ( long ndata,float   *data )
		{
			set = new float  [ndata];
			Set_nsampl ( ndata );
			for ( long i = 0; i < ndata;i++ )  set[i] = data[i];
		}
		float  *Get_set() {return set;}
		void  Get_data ( long& ndata,float  *data )
		{
			Get_nsampl ( ndata ); data = new float  [ndata];

			for ( long i = 0; i < ndata;i++ )  data[i] = set[i];
		}
};



class _SET_SEI
{
		int N_SEI;
		int I_N_SEI;
		char *in_dir;
		char *out_dir;
		float time_start ;
		float time_finish;
		float common_time;
		float common_max ;
		long  max_nsample;

		SEISMO  **CS;

		SEISMO  **S;
	public :
		_SET_SEI ( int n );

		virtual                   ~_SET_SEI();

		SEISMO **Get_CS() {return CS;}
		SEISMO  *Get_S ( int &n ) {return S[n]; }
		SEISMO **Get_SS() {return S; }
		void DetrendSeis();
		void RecalculateMaxSeis();
		void Recalc_parametrs();
		char *Set_in_dir ( char *ddir )
		{
			in_dir = new char [strlen ( ddir ) + 1];
			return strcpy ( in_dir,ddir );
		}
		char *Set_out_dir ( char *ddir )
		{
			out_dir = new char [strlen ( ddir ) + 1];
			return strcpy ( out_dir,ddir );
		}
		void  Set_time_start ( float time_star )  { time_start   = time_star    ;}
		void  Set_time_finish ( float time_finis )  { time_finish  = time_finis   ;}
		void  Set_common_time ( float common_tim )  { common_time  = common_tim   ;}
		void  Set_common_max ( float common_ma )  { common_max   = common_ma    ;}
		void  Set_max_nsample ( long max_nsampl )  { max_nsample   = max_nsampl ;}

		char *Get_in_dir ( char *ddir ) { return strcpy ( ddir,in_dir );}
		char *Get_out_dir ( char *ddir ) { return strcpy ( ddir,out_dir );}

		int Set_N_SEI ( int nsei ) {return ( N_SEI = nsei );}
		int Get_N_SEI ( int& nsei ) {return ( nsei = N_SEI );}
		int Get_N_SEI() {return ( N_SEI );}
		float  Get_time_start ( )  { return ( time_start );}
		float  Get_time_finish ( )  { return ( time_finish );}
		float  Get_common_time ( )  { return ( common_time );}
		float  Get_common_max ( )  { return ( common_max );}
		long   Get_max_nsample ( )  { return ( max_nsample );}

		int Set_I_N_SEI ( int nsei ) {return ( I_N_SEI = nsei );}

		int Get_I_N_SEI ( int& nsei ) {return ( nsei = I_N_SEI );}

		int Get_I_N_SEI() {return ( I_N_SEI );}

};
//=================================================================================
class  _MLT : public _SET_SEI
{
	public:
		_MLT ( int n_sei, QStringList &list,int indDetrend );
		virtual   ~_MLT();
bool seiFilesGood;
} ;

#endif
