#!/bin/csh -f

if($#argv == 0) then
    echo ================================================================
    echo " "
    echo 'fmech_pol.com <pol_file> <ps_file> <mech_file1> [<mech_file2>]'
    echo " "
    echo GMT VERSION is 3.4
    echo ================================================================
    exit
endif

# decoding command line
set pol_file = $argv[1]
set ps_file = $argv[2]

set argv[1] = ""
set argv[2] = ""
set argv = ($argv)

set mech_file = ($argv)

# get number of title lines in mech files
set ntitle = " "
foreach file ($mech_file)
    set t = `awk 'BEGIN{t=0}{if(substr($0,1,1)=="t") t=t+1;}END{print t}' $file`
    set ntitle = ($ntitle $t)
end

set radius2 = 3.0
set radius = `(echo scale=2;echo $radius2/2.)|bc`
set xoffset0 = 4.1
set yoffset0 = 10.5

gmtdefaults -D > .gmtdefaults
gmtset ANOT_FONT_SIZE 10p COLOR_NAN 255/255/255
set inch = i
set projx = (-Jx1$inch -P)
set transparent = -G255
set wpen = 0.50p
set size = -M$radius2$inch
set outline = (-f -e -g)

echo 0 0 0 | psxy -R-1/1/-1/1 $projx -Sc0.1 -G255 -K \
    -X$xoffset0$inch -Y$yoffset0$inch > $ps_file

@ i = 0
foreach file ($mech_file)
    @ i++
    @ j = 0

# print title
    while ($j < $ntitle[$i])
        @ j++
        set title = `head -$j $file |tail -1|cut -c2-120`
        pstext -R0/1/0/1 $projx -N -O -K << enddata >> $ps_file
0 `echo "-($i-1)*$radius-0.4*$j"|bc` 14 0 1 2 `echo "$title"`
enddata
    end
    set yoffset = `echo $radius\*$i+0.4+0.4\*$ntitle[$i]|bc`

# plot double couple
    awk '{if(substr($0,1,1) != "t") print "0 0 0", $4, $5, $6, "5 0 0";}' $file |\
        psmeca -R0/6/0/10 $projx -Sa$radius2$inch/-1 -L -K -O -N -Y-$yoffset$inch \
            -W$wpen/0/0/255 $transparent >> $ps_file

# plot compressional first arrival polarities
    awk '{if(substr($0,1,1) != "t" && substr($7,3,1) == "u") print $0;}' $pol_file | \
        pspolar -h -R $projx $size -D0/0 -G0/0/255 -Sc0.1$inch \
            -W$wpen/0/0/255 $outline -N -P -K -O >> $ps_file

# plot rarefactional first arrival polarities
    awk '{if(substr($0,1,1) != "t" && substr($7,3,1) == "d") print $0;}' $pol_file | \
        pspolar -h -R $projx $size -D0/0 -G0/0/255 -Sc0.1$inch \
            -W$wpen/0/0/255 $outline -N -P -O -K >> $ps_file
end

echo 0 0 0 | psxy -R-1/1/-1/1 $projx -Sc0.1 -G255 -O >> $ps_file

if ( $FMT_ARCHITECTURE_ =~ linux ) then
ghostview $ps_file 
endif
if ( $FMT_ARCHITECTURE_ =~ solaris ) then
gv -media a4 $ps_file  
endif
